/*
 * Anomalous_Dim.h
 *
 *  Created on: May 7, 2014
 *      Author: daniel
 */

#ifndef ANOMALOUS_DIM_H
#define ANOMALOUS_DIM_H

//#include <vector>
//#include <iostream>
#include "konst.h"
#include "enums.h"
#include "utils.h"
#include "vecmath.h"
//#include "options.h"

namespace CuTe::NS_Anomalous_Dimension {


    using std::vector;
    using std::cout;
    using std::endl;
    using namespace NS_enums::NS_PROD;
    using namespace NS_Utils::NS_pow;

    using NS_enums::NS_Partons::N_FMAX;


    class Anomalous_Dimension {
    protected:
        const int order;
        int n_f;
        vector<vector<vector<double> > > V;
        vector<vector<vector<double> > >::iterator V_nf;


        inline virtual void v_update() {
            cout << "*************************************************************************" << endl
                 << "**     class Anomalous_Dimension::v_update not defined -> exit(1)      **" << endl
                 << "*************************************************************************" << endl;
            exit(1);

        }

        inline void calculate() {
//		cout	<<"\t\t->Anomalous_Dimension::calculate()"	<<endl;
            int tmp = get_n_f();

            for (n_f = 1; n_f <= N_FMAX; ++n_f) {
                select(n_f);

                // set AD_nf(i) = (*V_nf)[i][1]
                v_update();

                if ((*V_nf)[0][1] != 0.) {    //if AD_nf(0) != 0

                    double iAD0(1. / (*V_nf)[0][1]);

                    // set AD_nf(0,0) = 1./ AD_nf(0)
                    (*V_nf)[0][0] = iAD0;

                    // set AD_nf(i,0) =  AD_nf(i)/ AD_nf(0)
                    for (int i = 1; i <= order; ++i) (*V_nf)[i][0] = (*V_nf)[i][1] * iAD0;

                    if (order >= 2) {

                        // set AD_nf(0,2) = 1./ AD_nf(0)^2
                        (*V_nf)[0][2] = pow_2(iAD0);

                        // set AD_nf(i,2) = ( AD_nf(i)/ AD_nf(0) )^2
                        for (int i = 1; i <= order; ++i) (*V_nf)[i][2] = pow_2((*V_nf)[i][0]);
                    }
                    for (int j = 3; j <= order; ++j) {

                        // set AD_nf(0,j) = 1./ AD_nf(0)^j
                        (*V_nf)[0][j] = (*V_nf)[0][j - 1] * iAD0;

                        // set AD_nf(i,j) = ( AD_nf(i)/ AD_nf(0) )^j
                        for (int i = 1; i <= order; ++i) (*V_nf)[i][j] = (*V_nf)[i][j - 1] * (*V_nf)[i][0];
                    }
                }
            }
            select(tmp);
//		cout	<<"\t\t<-Anomalous_Dimension::calculate()"	<<endl;
        }

    public:
        inline void select(int const n) {
//		NS_Utils::DOUT dout("Anomalous_Dimension::select","n");
//		dout()<<	"n = "<<n	<<"\tn_f = "<<n_f	<<endl;
            if ((1 > n) || (n > 6)) {
                std::cerr << "Wrong nf=" << n << " in Anomalous_Dimension::select(nf)" << std::endl;
                exit(1);
            }
            n_f = n;
            V_nf = V.begin() + (n_f - 1);
        }

        inline int get_n_f() const {
            return n_f;
        }

        inline int get_order() const { return order; }

        inline double operator()(int const i) const {// return AD[i]
//		if(i>order){// debug
//			std::cerr	<<"Wrong i="<<i<<">"<<order<<" in Anomalous_Dimension::operator(i)"<<std::endl;
//			exit(1);
//		}
            return (*V_nf)[i][1];
        }

        inline double operator()(int const i, int const j) const {
//		if(i>order){// debug
//			std::cerr	<<"Wrong i="<<i<<">"<<order<<" in Anomalous_Dimension::operator(i,j)"<<std::endl;
//			exit(1);
//		}
//		if(j>order){// debug
//			std::cerr	<<"Wrong j="<<j<<">"<<order<<" in Anomalous_Dimension::operator(i,j)"<<std::endl;
//			exit(1);
//		}
//		if(n_f<=0) cout << "Anomalous_Dimension::operator() V["<<n_f-1<<"]("<<i<<","<<j<<")"<< endl;	// debug
//		return	1/AD[0],		AD[1]/AD[0],		AD[2]/AD[0],		AD[3]/AD[0],	....
//				AD[0],			AD[1],				AD[2],				AD[3],			...
//				(1/AD[0])^2,	(AD[1]/AD[0])^2,	(AD[2]/AD[0])^2,	(AD[3]/AD[0])^2,	....
//				(1/AD[0])^3,	(AD[1]/AD[0])^3,	(AD[2]/AD[0])^3,	(AD[3]/AD[0])^3,	....
//					...				...					...					...
            return (*V_nf)[i][j];
        }

    private:

        inline Anomalous_Dimension();

    protected:

        inline Anomalous_Dimension(int Max_Order) :
                order(Max_Order),
                n_f(5),
                V(N_FMAX, vector<vector<double> >(order + 1, vector<double>(order + 1, 0.))) {
        }

        inline Anomalous_Dimension(const Anomalous_Dimension &AD)
                : order(AD.order), n_f(AD.n_f), V(AD.V) {
//		NS_Utils::DOUT dout("Anomalous_Dimension","AD");
            select(n_f);
        }

        inline Anomalous_Dimension &operator=(const Anomalous_Dimension &AD) {
            if (order != AD.order) {
                std::cerr << "Wrong order in Anomalous_Dimension::operator=" << std::endl;
                exit(1);
            }
            n_f = (AD.n_f);
            V = (AD.V);
            select(n_f);
            return *this;
        }

    public:
        inline virtual ~Anomalous_Dimension() {
        }
    };

    std::ostream &operator<<(std::ostream &os, const Anomalous_Dimension &AD);

    class AD_GammaCusp : public Anomalous_Dimension {
        /*	Cusp anomalous dimension \Gamma_{cusp}
         *		\Gamma_{cusp}^{V} = C_F * \Gamma_{cusp}
         *		\Gamma_{cusp}^{S} = C_A * \Gamma_{cusp}
         *
         *	\Gamma_{(0,1,2)cusp} from arXiv:1604.03126 (2.6)
         *	\Gamma_{(3)cusp} from arXiv:1805.09638 (4.1, 4.2)
         *
         *	large-N_c limit (+ n_f-terms)
         *	\Gamma_{(3)cusp} from arXiv:1612.04389 (11)
         */
    protected:
        PROD prod;
        double C;
        double iC;
        static const int Max_Order = 3;


        inline void v_update() {
            (*V_nf)[0][1] = 4.;
            (*V_nf)[1][1] = C_A * (268.0 / 9.0 - 8. * zeta_2) - 80.0 / 9.0 * T_F * n_f;
            (*V_nf)[2][1] = (
                                    C_A * C_A * (
                                            4410.
                                            - 3216 * zeta_2
                                            + 792. * zeta_3
                                            + 2376 * zeta_4
                                    )
                                    + C_A * T_F * n_f * (
                                            -1672.
                                            + 960. * zeta_2
                                            - 2016. * zeta_3
                                    )
                                    + C_F * T_F * n_f * (
                                            -1980.
                                            + 1728. * zeta_3
                                    )
                                    - pow_2(8. * T_F * n_f)
                            ) / (27.);
            switch (prod){
                case PROD_S:
                case PROD_Higgs:
                case PROD_H_RS:
                    (*V_nf)[3][1] = iC*(40880. + n_f * (-11714. + n_f*(440.0488 + n_f*(7.362774))));
                    break;
                case PROD_V:
                case PROD_DY:
                case PROD_W:
                case PROD_Z:
                    (*V_nf)[3][1] = iC*(20702. + n_f * (-5171.9 + n_f*(195.5772 + n_f*(3.272344))));
                    break;
                default:
                    // large-N_c limit (+ n_f-terms) \Gamma_{(3)cusp} from arXiv:1612.04389 (11)
                    (*V_nf)[3][1] = +pow_3(n_f) * (64. * zeta_3 / 27. - 32. / 81.)
                                    + pow_2(n_f) * C_A * (-32. * 90. * zeta_4 / 135.
                                                          + 1280. * zeta_3 / 27.
                                                          - 304. * 6. * zeta_2 / 243.
                                                          + 2119. / 81.)
                                    + n_f * C_A2 * (+224. * zeta_5
                                                    + 128. * 6. * zeta_2 * zeta_3 / 9.
                                                    - 44. * 90. * zeta_4 / 27.
                                                    - 16252. * zeta_3 / 27.
                                                    + 13346. * 6. * zeta_2 / 243.
                                                    - 39883. / 81.)
                                    + C_A3 * (-292. * 945. * zeta_6 / 315.
                                              - 32. * pow_2((double) zeta_3)
                                              - 352. * zeta_5
                                              - 176. * 6. * zeta_2 * zeta_3 / 9.
                                              + 902 * 90. * zeta_4 / 45.
                                              + 20992. * zeta_3 / 27.
                                              - 44416. * 6. * zeta_2 / 243.
                                              + 84278. / 81.);
            }

        }

        inline AD_GammaCusp()
                :Anomalous_Dimension(Max_Order),
                prod(PROD_WRONG_STATEMENT),
                C(1.),iC(1.) {
        }

    public:

        inline void init(const PROD prod) {
            this->prod = prod;
            switch (prod) {
                case PROD_S:
                case PROD_Higgs:
                case PROD_H_RS:
                    this->C = C_A;
                    break;
                case PROD_V:
                case PROD_DY:
                case PROD_W:
                case PROD_Z:
                    this->C = C_F;
                    break;
                default:
                    this->C = 1.;
            }
            this->iC = 1./this->C;

            calculate();
        }


        inline AD_GammaCusp(const PROD prod)
        :Anomalous_Dimension(Max_Order),
         prod(PROD_WRONG_STATEMENT),
         C(1.),iC(1.){
            init(prod);
        }
    };


    class AD_Beta : public Anomalous_Dimension {
    private:

        static const int Max_Order = 3;

        inline void v_update() {
//		cout	<<"\t\t\tAD_Beta::v_update()"	<<endl;
            (*V_nf)[0][1] = (11.0 * C_A - 4.0 * T_F * n_f) / 3.0;
            (*V_nf)[1][1] = 34.0 / 3.0 * C_A * C_A
                            + (-20.0 / 3.0 * C_A - 4.0 * C_F) * T_F * n_f;
            (*V_nf)[2][1] = 2857.0 / 54.0 * C_A * C_A * C_A
                            + (2.0 * C_F * C_F - 205.0 / 9.0 * C_A * C_F - 1415.0 / 27.0 * C_A * C_A) * T_F * n_f
                            + (44.0 / 9.0 * C_F + 158.0 / 27.0 * C_A) * T_F * T_F * n_f * n_f;
            (*V_nf)[3][1] = 149753. / 6. + 3564. * zeta_3 - (1078361. / 162. + 6508. / 27. * zeta_3) * n_f
                            + (50065. / 162 + 6472. / 81. * zeta_3) * n_f * n_f + 1093. / 729. * pow_3(n_f);
        }

    public:
        inline AD_Beta() : Anomalous_Dimension(Max_Order) {
            calculate();
        }
    };


    class AD_gamma_q : public Anomalous_Dimension {
    private:

        static const int Max_Order = 2;

        inline void v_update() {
//		cout	<<"\t\t\tAD_gamma_q::v_update()"	<<endl;
            (*V_nf)[0][1] = -3 * C_F;
            (*V_nf)[1][1] = C_F * C_F * (-3.0 / 2.0 + 2.0 * PI2 - 24.0 * zeta_3)
                            + C_F * C_A * (-961.0 / 54.0 - 11.0 / 6.0 * PI2 + 26.0 * zeta_3)
                            + C_F * T_F * n_f * (130.0 / 27.0 + 2.0 / 3.0 * PI2);
            (*V_nf)[2][1] = pow_3(C_F) * 0.5 * (
                    -29.
                    - 36. * zeta_2
                    - 136. * zeta_3
                    + 64. * zeta_2 * zeta_3
                    - 288. * zeta_4
                    + 480. * zeta_5
            )
                            + pow_2(C_F) * (
                    C_A * (
                            -453.
                            + 1640. * zeta_2
                            - 3376. * zeta_3
                            - 192. * zeta_2 * zeta_3
                            + 1976. * zeta_4
                            - 1440. * zeta_5
                    ) / (12.)
                    + T_F * n_f * (
                            2953.
                            - 468. * zeta_2
                            + 1536. * zeta_3
                            - 2520. * zeta_4
                    ) / (27.)
            )
                            + C_F * (
                    pow_2(C_A) * (
                            -139345.
                            - 257868. * zeta_2
                            + 1142424. * zeta_3
                            - 85536. * zeta_2 * zeta_3
                            - 242028. * zeta_4
                            - 396576. * zeta_5
                    ) / (2916.)
                    + 2 * C_A * T_F * n_f * (
                            -8659.
                            + 23346. * zeta_2
                            - 26028. * zeta_3
                            + 16038. * zeta_4
                    ) / (729.)
                    - pow_2(2. * T_F * n_f) * (
                            -2417.
                            + 1620. * zeta_2
                            + 216. * zeta_3
                    ) / (729.)
            );
        }

    public:
        inline AD_gamma_q() : Anomalous_Dimension(Max_Order) {
            calculate();
        }
    };


    class AD_gamma_t : public Anomalous_Dimension {
    private:
        AD_Beta *Beta;

        static const int Max_Order = 3;

        inline void v_update() {
//		cout	<<"\t\t\tAD_gamma_t::v_update()"	<<endl;
            (*V_nf)[0][1] = 0.;
            Beta->select(n_f);
//		select()
            for (int i = 1; i <= Max_Order; ++i) (*V_nf)[i][1] = -2. * i * (*Beta)(i);
        }

    public:
        inline AD_gamma_t(AD_Beta *Beta_) :
                Anomalous_Dimension(Max_Order), Beta(Beta_) {
            int tmp = Beta->get_n_f();
            calculate();
            Beta->select(tmp);
        }

        inline AD_gamma_t(const AD_gamma_t &AD)
                : Anomalous_Dimension(AD), Beta(AD.Beta) {
        }

        inline AD_gamma_t &operator=(const AD_gamma_t &AD) {
            Anomalous_Dimension::operator=(AD);
            Beta = (AD.Beta);
            return *this;
        }

        inline ~AD_gamma_t() {
        }
    };

    class AD_gamma_g : public Anomalous_Dimension {
    private:

        static const int Max_Order = 2;

        inline void v_update() {
//		cout	<<"\t\t\tAD_gamma_g::v_update()"	<<endl;
            (*V_nf)[0][1] = (-11. * C_A + 4. * T_F * n_f) / 3.;
            (*V_nf)[1][1] = C_A * C_A * (
                    -692.
                    + 99. * zeta_2
                    + 54. * zeta_3
            ) / (27.)
                            + n_f * T_F * (
                    4. * C_F
                    - 4. * C_A * (
                            -64.
                            + 9. * zeta_2
                    ) / 27.
            );
            (*V_nf)[2][1] = pow_3(C_A) * (
                    -97186.
                    + 54981. * zeta_2
                    + 29646. * zeta_3
                    - 9720. * zeta_2 * zeta_3
                    - 77517. * zeta_4
                    - 11664. * zeta_5
            ) / (729.)
                            + n_f * T_F * (
                    -2. * C_F * C_F
                    - 2. * C_A * C_F * (
                            -1217.
                            + 54. * zeta_2
                            + 456. * zeta_3
                            + 216. * zeta_4
                    ) / (27.)
                    + C_A * C_A * (
                            30715.
                            - 21564. * zeta_2
                            + 19224. * zeta_3
                            + 39852. * zeta_4
                    ) / (729.)
            )
                            + pow_2(n_f * T_F) * (
                    -44. / 9. * C_F
                    + 2. / 729. * C_A * (
                            -269.
                            + 1080. * zeta_2
                            - 3024. * zeta_3
                    )
            );
        }

    public:
        inline AD_gamma_g() : Anomalous_Dimension(Max_Order) {
            calculate();
        }
    };

    class AD_gamma_S : public Anomalous_Dimension {
        //gamma_S = 2gamma_g-gamma_t-beta/alpha
    private:
        AD_gamma_g *g_g;
        AD_gamma_t *g_t;
        AD_Beta *b;

        static const int Max_Order = 2;

        inline void v_update() {
//		cout	<<"\t\t\tAD_gamma_S::v_update()"	<<endl;
            b->select(n_f);
            g_g->select(n_f);
            g_t->select(n_f);

            for (int i = 0; i <= Max_Order; ++i) {
                (*V_nf)[i][1] = 2. * (*g_g)(i) - (*g_t)(i) + 2. * (*b)(i);
            }


        }

    public:
        inline AD_gamma_S(AD_gamma_g *g_g_, AD_gamma_t *g_t_, AD_Beta *b_) :
                Anomalous_Dimension(Max_Order), g_g(g_g_), g_t(g_t_), b(b_) {
            int tmp_g = g_g->get_n_f();
            int tmp_t = g_t->get_n_f();
            int tmp_b = b->get_n_f();
            calculate();
            g_g->select(tmp_g);
            g_t->select(tmp_t);
            b->select(tmp_b);
        }

        inline AD_gamma_S(const AD_gamma_S &AD)
                : Anomalous_Dimension(AD), g_g(AD.g_g), g_t(AD.g_t), b(AD.b) {
        }

        inline AD_gamma_S &operator=(const AD_gamma_S &AD) {
            Anomalous_Dimension::operator=(AD);
            g_g = (AD.g_g);
            g_t = (AD.g_t);
            b = (AD.b);
            return *this;
        }

        inline ~AD_gamma_S() {

        }
    };

    class AD_gamma_V : public Anomalous_Dimension {
        //gamma_V = 2gamma_q
    private:
        AD_gamma_q *g_q;

        static const int Max_Order = 2;

        inline void v_update() {
//		cout	<<"\t\t\tAD_gamma_V::v_update()"	<<endl;
            g_q->select(n_f);

            for (int i = 0; i <= Max_Order; ++i) {
                (*V_nf)[i][1] = 2. * (*g_q)(i);
            }


        }

    public:
        inline AD_gamma_V(AD_gamma_q *g_q_) :
                Anomalous_Dimension(Max_Order), g_q(g_q_) {
            int tmp_q = g_q->get_n_f();
            calculate();
            g_q->select(tmp_q);
        }

        inline AD_gamma_V(const AD_gamma_V &AD)
                : Anomalous_Dimension(AD), g_q(AD.g_q) {
        }

        inline AD_gamma_V &operator=(const AD_gamma_V &AD) {
            Anomalous_Dimension::operator=(AD);
            g_q = (AD.g_q);
            return *this;
        }

        inline ~AD_gamma_V() {
        }
    };


    class AD_gamma_C_over_C : public Anomalous_Dimension {
        //gamma_V = 2gamma_q
    protected:
        PROD prod;
        double iC;
        Anomalous_Dimension *gamma_C;

        static const int Max_Order = 2;

        inline void v_update() {
//		NS_Utils::DOUT dout("AD_gamma_C_over_C::v_update");
            gamma_C->select(n_f);

            for (int i = 0; i <= Max_Order; ++i) {
                (*V_nf)[i][1] = iC * (*gamma_C)(i);
            }
        }

        inline AD_gamma_C_over_C()
                : Anomalous_Dimension(Max_Order), prod(), iC(0.), gamma_C(NULL) {
        }

    public:

        inline void init(PROD prod) {
            this->prod = prod;

            delete gamma_C;    // No = NULL
            switch (prod) {
                case PROD_Higgs:
                case PROD_H_RS:
                    iC = 1. / C_A;
                    gamma_C = new AD_gamma_g();
                    break;
                default:
                    iC = 1. / C_F;
                    gamma_C = new AD_gamma_q();
                    break;
            }

            calculate();
        }


        inline AD_gamma_C_over_C(PROD prod)
                : Anomalous_Dimension(Max_Order), prod(prod), iC(), gamma_C(NULL) {
            init(prod);
        }

        inline AD_gamma_C_over_C(const AD_gamma_C_over_C &AD)
                : Anomalous_Dimension(AD), prod(), iC(), gamma_C(NULL) {
//		NS_Utils::DOUT	dout("AD_gamma_C_over_C","AD");
            init(AD.prod);
        }

        inline AD_gamma_C_over_C &operator=(const AD_gamma_C_over_C &AD) {
            Anomalous_Dimension::operator=(AD);
            init(AD.prod);
            return *this;
        }

        inline ~AD_gamma_C_over_C() {
            delete gamma_C;    // No = NULL
        }
    };


    class AD_Casimir_d : public Anomalous_Dimension {
/*	This coefficient d is rescaled in comparison to https://arxiv.org/pdf/1007.4005v3
 *	in such a way that:
 *		\Gamma_{cusp}^{q/g (0)} * d = F^{q/g}\left(L_\perp=0\right)
 *
 *		arXiv:1604.01404 Equation (9)
 *		d = -\gamma^r / 4C_a = -\gamma^r / \Gamma_{cusp}^{q/g (0)}
 */
    private:
        static const int Max_Order = 3;

        inline void v_update() {
//		cout	<<"\t\t\tAD_Casimir_d::v_update()"	<<endl;
            (*V_nf)[0][1] = 0.;
            (*V_nf)[1][1] = 0.;
            (*V_nf)[2][1] = C_A * (202.0 / 27.0 - 7.0 * zeta_3) - 56.0 / 27.0 * T_F * n_f;
//		(*V_nf)[3][1] =(*V_nf)[2][1]*(4.*M_PI)*d3_factor;
            (*V_nf)[3][1] = C_A2 * (192. * zeta_5
                                    + 176. * zeta_3 * zeta_2 / 3.
                                    - 154. * zeta_4 / 3.
                                    - 12328. * zeta_3 / 27.
                                    - 6392. * zeta_2 / 81.
                                    + 297029. / 729.
            )
                            + C_A * n_f * (-20. * zeta_4 / 3.
                                           + 904. * zeta_3 / 27.
                                           + 824. * zeta_2 / 81.
                                           - 62626 / 729.
            )
                            + pow_2(n_f) * (32. * zeta_3 / 9. + 1856 / 729.)
                            + C_F * n_f * (16. * zeta_4
                                           + 304. * zeta_3 / 9.
                                           - 1711. / 27.
            );
            (*V_nf)[3][1] *= 0.25;    // / G_0
        }

    public:
        inline AD_Casimir_d() : Anomalous_Dimension(Max_Order) {
            calculate();
        }
    };


}  // namespace CuTe::NS_Anomalous_Dimension

#endif


