#include "CuTe.h"
#include "CuTe4HERA.h"
#include <LHAPDF/LHAPDF.h>

#ifdef __cplusplus
extern "C" {
#endif
	#include "gsl/gsl_errno.h"
#ifdef __cplusplus
}
#endif

namespace CuTe4HERA {

using CuTe::NS_LHAFUNC::OWN_LHAPDF;
using CuTe::NS_Utils::NS_min_max::max_of;

using namespace CuTe::NS_enums::NS_RESULTS;
using namespace CuTe::NS_enums::NS_RES;
using namespace CuTe::NS_enums::NS_NP;

class _CuTe4HERA_Calculator:protected CuTe::Crosssection{
public:
	SOPS_CS OP;

protected:

	OWN_LHAPDF own_pdf;

	inline void _init_OP(){
		OP.PDF.PDF_Error.set(false);

//		OP.Scales.Scale_Error.set(false);
		OP.Scales.scale_variation.set(0.);

		if(!OP.Scales.LambdaNP.is_set){
			OP.Scales.LambdaNP.set(0.);
		}
		if(!OP.Scales.cutoff.is_set){
			OP.Scales.LambdaNP.set(0.);
			OP.Scales.cutoff.set(CUTOFF_NO);
		}

		OP.Scheme.exponent_errors.set(false);
		OP.Scheme.partial_results.set(true);
		OP.Scheme.resummation.set(CuTe::NS_enums::NS_RES::RES_RES_CORR);
	}

public:

	inline void init(){
		_init_OP();
		Crosssection::init(OP);

//		QDCS_central.without_combined_result = true;
//		--N_Centre;
//		--N_Comp;

	}
	// Initialize with new infile or option set
	inline void init(const string& infilename){
		OP.open(infilename);
		return init();
	}
	inline void init(const SOPS_CS& OP){
		this->OP	= OP;
		return init();
	}

	// Constructors
	inline _CuTe4HERA_Calculator()
	:Crosssection(false),OP()
	,own_pdf(){
		_init_OP();
	}
	inline _CuTe4HERA_Calculator(const string& infilename)
	:Crosssection(true),OP(infilename)
	,own_pdf(){
		init();
	}
	inline _CuTe4HERA_Calculator(const SOPS_CS& OP)
	:Crosssection(true),OP(OP)
	,own_pdf(){
		init();
	}
	_CuTe4HERA_Calculator(const _CuTe4HERA_Calculator& other)
	:Crosssection(true),OP(other.OP)
	,own_pdf(other.own_pdf){
		init();
	}
	inline _CuTe4HERA_Calculator& operator=(const _CuTe4HERA_Calculator& other){
		own_pdf	= (other.own_pdf);
		init(other.OP);
		return *this;
	}

	inline ~_CuTe4HERA_Calculator(){

	}



	// Change currently used value of mu to a fixed value
	inline void change_mu_to_value(const double mu){
		OP.Scales.fix_mu_to_const.unset();
		OP.Scales.fix_mu_to_value.set(mu);
		OP.Scales.q_star.unset();
		Crosssection::change_mustar(OP);
	}
	// Change currently used value of q*.
	// "Automatic" solves eta(mu==q*)==1 which depends on alpha_s.
	// The scale mu will be dynamically calculated by mu(qT,q*) = qT + q* x exp( -qT/q* )
	inline void change_qstar_to_value(const double qstar){
		OP.Scales.fix_mu_to_const.unset();
		OP.Scales.fix_mu_to_value.unset();
		OP.Scales.q_star.set(qstar);
		Crosssection::change_mustar(OP);
	}
	inline void change_qstar_to_automatic(){
		OP.Scales.fix_mu_to_const.unset();
		OP.Scales.fix_mu_to_value.unset();
		OP.Scales.q_star.unset();
		Crosssection::change_mustar(OP);
	}
	double get_qstar()const{ return q_star;}


//	// Change current used PDF and alpha_s routine
//	inline void change_PDF(const string& LHAPDF_Name, const int member=0){
//		own_pdf.unset();
//
//		OP.PDF.PDFset.set(LHAPDF_Name);
//		OP.PDF.PDF_member.set(member);
//
//		Crosssection::change_PDF(OP);
//	}
	inline void change_PDF(LHAPDF::PDF* lhapdf, LHAPDF::AlphaS* alphas){
		own_pdf.set(lhapdf,alphas);

		OP.PDF.PDFset.unset();
		OP.PDF.PDF_member.unset();

		Crosssection::change_PDF(OP,&own_pdf);
	}
//	inline void change_PDF(LHAPDF::PDFSet* lha_pdfset, const int member=0){
//		own_pdf.unset();
//
//		OP.PDF.PDFset.unset();
//		OP.PDF.PDF_member.set(member);
//
//		Crosssection::change_PDF(OP,lha_pdfset);
//	}

	// Change current used Lambda_NP and NPE-scheme
	inline void change_NPE(const double Lambda_NP, const CUTOFF cutoff){
		OP.Scales.LambdaNP.set(Lambda_NP);
		OP.Scales.cutoff.set(cutoff);
		return Crosssection::change_NPE(Lambda_NP,cutoff);
	}
	inline void change_NPE(const double Lambda_NP){
		return change_NPE(Lambda_NP,OP.Scales.cutoff());
	}

	// Calculate double differential results for qT and y
	// return if numerical accuracy goal was reached
	inline Results& calculate(const double qT, const double y, Results& HERA_results){

		if(without_combined_result){
			update_qT_y(qT,y);
			HERA_results.RES		= cuba.integral[0];
			HERA_results.RES_DInt	= cuba.error[0];
			HERA_results.SQT		= cuba.integral[1];
			HERA_results.SQT_DInt	= cuba.error[1];
			HERA_results.CUBA_FAIL	= cuba.fail;
		}else{
			const int dim(2);
			const int N_Comp(1);

			bool OK(Crosssection::_set_qT_mu_y_get_OK<RES_RES_CORR>(qT,mu_star(qT),y));

			if(OK){
				cuba.set(dim,N_Comp);
				cuba.cube(_cuba_int_xx<RES_LL>,this);
				HERA_results.RES		= cuba.integral[0];
				HERA_results.RES_DInt	= cuba.error[0];
				HERA_results.CUBA_FAIL	= cuba.fail;

				cuba.set(dim,N_Comp);
				cuba.cube(_cuba_int_xx<RES_LLO>,this);
				HERA_results.SQT		= cuba.integral[0];
				HERA_results.SQT_DInt	= cuba.error[0];
				HERA_results.CUBA_FAIL	= max_of(HERA_results.CUBA_FAIL,cuba.fail);
			}

		}
		HERA_results.Rms		= QDCS_central.get_MSF();
		HERA_results.TF			= QDCS_central.get_TF();//transition_to_FO.f(qT);

		return HERA_results;
	}
};

// Initialize with new infile or option set
void Interface::init(){
	return calc->init();
}
void Interface::init(const string& infilename){
	return calc->init(infilename);
}
void Interface::init(const SOPS_CS& OP){
	return calc->init(OP);
}

// Constructors
Interface::Interface()
:calc(new _CuTe4HERA_Calculator){
	gsl_set_error_handler_off();
}
Interface::Interface(const string& infilename)
:calc(new _CuTe4HERA_Calculator(infilename)){
	gsl_set_error_handler_off();
}
Interface::Interface(const SOPS_CS& OP)
:calc(new _CuTe4HERA_Calculator(OP)){
	gsl_set_error_handler_off();
}
Interface::Interface(const Interface& other)
:calc(new _CuTe4HERA_Calculator(*(other.calc))){
	gsl_set_error_handler_off();
}
Interface& Interface::operator=(const Interface& other){
	delete calc;
	calc 	= new _CuTe4HERA_Calculator(*(other.calc));
	return *this;
}


Interface::~Interface(){
	delete calc;
}

// Change currently used value of mu to a fixed value
void Interface::change_mu_to_value(const double mu){
	calc->change_mu_to_value(mu);
}
// Change currently used value of q*.
// "Automatic" solves eta(mu==q*)==1 which depends on alpha_s.
// The scale mu will be dynamically calculated by mu(qT) = qT + q* x exp( -qT/q* )
void Interface::change_qstar_to_value(const double qstar){
	calc->change_qstar_to_value(qstar);
}
void Interface::change_qstar_to_automatic(){
	calc->change_qstar_to_automatic();
}
double Interface::get_qstar()const{
	return calc->get_qstar();
}


void Interface::change_PDF(LHAPDF::PDF* lhapdf, LHAPDF::AlphaS* alphas){
	return calc->change_PDF(lhapdf,alphas);
}

// Change current used Lambda_NP and NPE-scheme
void Interface::change_NPE(const double Lambda_NP, const CUTOFF NP_scheme){
	return calc->change_NPE(Lambda_NP,NP_scheme);
}
void Interface::change_NPE(const double Lambda_NP){
	return change_NPE(Lambda_NP,calc->OP.Scales.cutoff());
}

// Calculate double differential results for qT and y
// return if numerical accuracy goal was reached
int Interface::calculate(const double qT, const double y, Results &results){
	return (calc->calculate(qT,y,results)).CUBA_FAIL;
}

const SOPS_CS& Interface::get_options()const{
	return calc->OP;
}



}  // namespace CuTe4HERA
