#ifndef CUTE_OPTIONS_H
#define CUTE_OPTIONS_H

////#include "options.h"
//#include "enums.h"

#include "param_options.h"
#include "cuba_options.h"
#include "Kinematics_Options.h"

namespace CuTe {

using NS_OPTIONS::Option;
using NS_OPTIONS::VOption;
using std::string;
using std::vector;

class OPS_Plot: public NS_OPTIONS::_Options_Set{
public:
	Option<vector<double> > qT;
	Option<int> qT_N;
	Option<vector<double> > y;
	Option<int> y_N;
	Option<int> Gridplot;

protected:

	inline const vector<double> default_qT()const{
		vector<double> qT;
		qT.push_back(  0.);
		qT.push_back(  8.);
		qT.push_back( 12.);
		qT.push_back( 20.);
		qT.push_back( 60.);
		return qT;
	}

	void v_init_ops(){
//		std::cout<<"\t"<<"OPS_Plot::v_init_ops()"<<std::endl;
		qT			= Option<vector<double> >("qT","[qT1 qT2 ... ] GeV",default_qT());
		qT_N		= Option<int>("qT_N","[int]",10);
		qT_N.set_def_val(1);

		y			= Option<vector<double> >("y","[y1 y2 ... ]");
		y.set_def_exp("Integrate over y");
		y_N		= Option<int>("y_N","[int]");
		y_N.set_def_val(1);

		Gridplot	= Option<int>("Gridplot","[0=false, N = min evaluations]");
		Gridplot.set_def_val(false);
	}

	void v_init_vec(){
//		std::cout<<"\t"<<"OPS_Plot::v_init_vec()"<<std::endl;

		options.push_back(&qT);
		options.push_back(&qT_N);
		options.push_back(&y);
		options.push_back(&y_N);
		options.push_back(&Gridplot);
	}

public:

	OPS_Plot():_Options_Set("Plot Options"){
//		std::cout<<"\t"<<"OPS_Plot()"<<std::endl;
		init();
	}

	OPS_Plot(const string& filename):_Options_Set("Plot Options"){
//		std::cout<<"\t"<<"OPS_Plot("<<filename<<")"<<std::endl;
		init(filename);
	}

	OPS_Plot(const OPS_Plot& OS)
	:_Options_Set(OS)
	,qT(OS.qT),qT_N(OS.qT_N)
	,y(OS.y),y_N(OS.y_N)
	,Gridplot(OS.Gridplot){
//		std::cout<<"\t"<<"OPS_Plot(const OPS_Plot& OS)"<<std::endl;
		init_vec();
	}

};


class SOPS_CS:public NS_CS_Params::SOPS_CS_Params {
public:
	NS_CUBA_integration::OPS_CUBA CUBA;
	NS_Kinematics::OPS_Cuts Cuts;

private:

	void init(){
//		std::cout<<"SOPS_CuTe::init()"<<std::endl;
//		SOPS_CS_Params::v_init_all();
		_all.push_back(&CUBA);
		_all.push_back(&Cuts);
		_init();
	}

public:

	SOPS_CS():SOPS_CS_Params(){
//		std::cout<<"SOPS_CuTe()"<<std::endl;
		init();
	}

	SOPS_CS(const string& filename):SOPS_CS_Params(){
//		std::cout<<"SOPS_CuTe("<<filename<<")"<<std::endl;
		init();
		open(filename);
	}

	SOPS_CS(const SOPS_CS& OP)
	:SOPS_CS_Params(OP)
	,CUBA(OP.CUBA)
	,Cuts(OP.Cuts){
//		std::cout<<"SOPS_CuTe(const SOPS_CuTe& OP)"<<std::endl;
		init();
	}

};


class SOPS_CuTe:public SOPS_CS {
public:
	OPS_Plot Plot;

private:

	void init(){
//		std::cout<<"SOPS_CuTe::init()"<<std::endl;
//		SOPS_CS_Params::v_init_all();
		_all.push_back(&Plot);
		_init();
	}

public:

	SOPS_CuTe():SOPS_CS(){
//		std::cout<<"SOPS_CuTe()"<<std::endl;
		init();
	}

	SOPS_CuTe(const string& filename):SOPS_CS(){
//		std::cout<<"SOPS_CuTe("<<filename<<")"<<std::endl;
		init();
		open(filename);
	}

	SOPS_CuTe(const SOPS_CuTe& OP)
	:SOPS_CS(OP)
	,Plot(OP.Plot){
//		std::cout<<"SOPS_CuTe(const SOPS_CuTe& OP)"<<std::endl;
		init();
	}

};

}  // namespace CuTe

#endif
