#include "FO_Parameter.h"

#ifdef __cplusplus
extern "C" {
#endif
	#include "gsl/gsl_sf_dilog.h"
#ifdef __cplusplus
}
#endif

using CuTe::NS_FO::FO_NLO_Kinematic_Parameter;
using CuTe::NS_FO::FO_NNLO_Kinematic_Parameter;

std::ostream& FO_NLO_Kinematic_Parameter::properties(std::ostream &os, const int ntabs)const{
	std::string tab("");
	for(int i=0; i<ntabs; ++i) tab += "\t";

	os<<tab<<"FO_NLO_Kinematic_Parameter:"<<endl;
	os<<tab;
	os<<"\t sS = "<<sS;
	os<<"\t M = "<<sqrt(M2);
	os<<"\t qT = "<<sqrt(qT2);
	os<<endl;
	os<<tab;
	os<<"\t T = "<<T;
	os<<"\t U = "<<U;
	os<<"\t X2 = "<<X2;
	os<<endl;
	os<<tab;
	os<<"\t xi+- = ["<<_xip<<","<<_xim<<"]";
	os<<"\t xi = ["<<xi[0]<<","<<xi[1]<<"]";
	os<<endl;
	os<<tab;
	os<<"\t s = "<<s;
	os<<"\t t = "<<t;
	os<<"\t u = "<<u;
	os<<"\t x2 = "<<x2;
	os<<endl;
	os<<tab;
	os<<"\t A = "<<A;
	os<<"\t B = "<<B;
	os<<"\t C = "<<C;
	os<<endl;
	os<<tab;
	os<<"\t Jac_1d = "<<Jac_1d;
	os<<"\t Jac_2d = "<<Jac_2d;
	os<<endl;
	return os;
}

std::ostream& FO_NNLO_Kinematic_Parameter::properties(std::ostream &os, const int ntabs)const{
	std::string tab("");
	for(int i=0; i<ntabs; ++i) tab += "\t";

	os<<tab<<"FO_NNLO_Kinematic_Parameter:"<<endl;
	FO_NLO_Kinematic_Parameter::properties(os,ntabs+1);
	os<<tab;
	os<<"\t ds = "<<ds;
	os<<"\t dt = "<<dt;
	os<<"\t du = "<<du;
	os<<endl;
	os<<tab;
	os<<"\t za = "<<za;
	os<<"\t zb = "<<zb;
	os<<endl;
	os<<tab;
	os<<"\t fs = "<<fs;
	os<<"\t ft = "<<ft;
	os<<"\t fu = "<<fu;
	os<<"\t fx2 = "<<fx2;
	os<<"\t fA = "<<fA;
	os<<endl;
	return os;
}

void FO_NNLO_Kinematic_Parameter::_update_nnlo(){
	//		NS_Utils::DOUT dout("FO_NNLO_Kinematic_Parameter::_update_nnlo");
	double tmp;

	M2t	= M2-t;
	iM2t= 1./M2t;
	M2u	= M2-u;
	iM2u= 1./M2u;

	ids	= (s+M2-x2);
	idt	= (x2-t);
	idu	= (x2-u);

	ds	= 1./ids;
	dt	= 1./idt;
	du	= 1./idu;

	za	= -t*dt;
	iza	= -it*idt;
	iza2= iza*iza;
	iza4= iza2*iza2;

	zb	= -u*du;
	izb	= -iu*idu;
	izb2= izb*izb;
	izb4= izb2*izb2;

	GS_B2	= ids*ids-4.*M2*s;
	GS_B	= sqrt(GS_B2);
	GS_iB	= 1./GS_B;
	GS_iB2	= GS_iB*GS_iB;
	GS_iB4	= GS_iB2*GS_iB2;


	fL2a	= log(M2*s/pow_2(ids-s*za));
	fL2b	= log(M2*s/pow_2(ids-s*zb));
	fL3		= log((ids+GS_B)/(ids-GS_B));

	//		double ds2	= pow_2(ds);
	double dt2	= pow_2(dt);
	double du2	= pow_2(du);

	dst		= 1./(s-idt);
	dsu		= 1./(s-idu);
	tmp		= (t*u-x2*M2);
	dtu 	= 1./tmp;
	idtu2	= tmp*tmp;
	ftu 	= log(tmp*dt*du);

	l	= sqrt(pow_2(u+t)-4.*x2*M2);
	il	= 1./l;
	il2	= il*il;
	il3	= il2*il;
	il4	= il3*il;
	il5	= il4*il;

	fs	= log( s*iM2);
	ft	= log(-t*iM2);
	fu	= log(-u*iM2);

	fx2	= ( (x2>0.) ? log( x2*iM2) : 0. );
	fA	= log(  A*iM2);
	if(fA!=fA){
//		cerr<<"NAN FO_NNLO_Kinematic_Parameter::_update_nnlo fA"<<endl;
//		cerr<<"\t xi[0]-B"<<xi[0]-B<<endl;
		fA=0.;
	}

	fst	= log(s*iM2*za*za);
	fsu = log(s*iM2*zb*zb);
	fstu= log(s*M2*dt*du);

	fl	= log((ids+l)/(ids-l));

	tmp	= 2.*M2-u;
	tmp = x2*tmp-M2*t;
	tmp = idt/tmp;
	tmp = s*M2*pow_2(tmp);
	flt = log(tmp);

	tmp	= 2.*M2-t;
	tmp = x2*tmp-M2*u;
	tmp = idu/tmp;
	tmp = s*M2*pow_2(tmp);
	flu = log(tmp);

	tmp = M2*iM2t;
	f1t	= gsl_sf_dilog(tmp) + 0.5*pow_2(log(tmp));
	tmp = M2*iM2u;
	f1u = gsl_sf_dilog(tmp) + 0.5*pow_2(log(tmp));

	tmp = gsl_sf_dilog(M2*is);
	f2t	= tmp + fs*( 0.5*fs - log((M2-s)*it) );
	f2u	= tmp + fs*( 0.5*fs - log((M2-s)*iu) );

	LtX2	= log(-t*iX2);
	LuX2	= log(-u*iX2);
	Lx2pX2	= log(x2T*iX2);

}
