#include "utils.h"


namespace CuTe {
namespace NS_Interval {

using namespace NS_Utils::NS_min_max;
using std::ostream;
using std::endl;

enum E_POS_VAL_TO_INT {V_I_left=-1,V_I_inside=0,V_I_right=+1};
enum E_POS_INT_TO_INT {I_I_left,I_I_left_overlapping,I_I_inside,I_I_over,I_I_right_overlapping,I_I_right};

class Interval {
public:
	double min,max;

	inline Interval():min(0.),max(0.){}
	inline Interval(const double min, const double max):min(min),max(max){}


	inline double size()const{return max_of(max-min,0.);}
	inline bool is_valid()const{return (max>min);}
	inline bool is_unvalid()const{return !is_valid();}
	inline Interval& set_unvalid(){
		min	= +1;
		max	= -1;
		return *this;
	}

	inline Interval& init_min(const double min){
		this->min	= min;
		return *this;
	}
	inline Interval& init_max(const double max){
		this->max	= max;
		return *this;
	}
	inline Interval& init_min_max(const double min, const double max){
		this->min	= min;
		this->max	= max;
		return *this;
	}
	inline Interval& init(const double a, const double b){
		if(a<b) return init_min_max(a,b);
		else	return init_min_max(b,a);
	}

	inline E_POS_VAL_TO_INT position_of(const double value)const{
		if(value<=min)		return V_I_left;
		else if(max<=value)	return V_I_right;
		else 				return V_I_inside;
	}

	inline bool contains(const double value)const{
		return (is_valid() && min<value && value<max);
	}

	inline bool contains_not(const double value)const{
		return !contains(value);
	}

	inline E_POS_INT_TO_INT position_of(const Interval &I)const{
		/*
		 * (1) : min(I) < max(I) < min(_) < max(_)
		 * (2) : min(I) < min(_) < max(I) < max(_)
		 * (3) : min(I) < min(_) < max(_) < max(I)
		 * (4) : min(_) < min(I) < max(I) < max(_)
		 * (5) : min(_) < min(I) < max(_) < max(I)
		 * (6) : min(_) < max(_) < min(I) < max(I)
		 */
		if(I.max<min){	return I_I_left;// (1)
		}else{	// (2)-(6)
			if(I.min<min){	// (2),(3)
				if(I.max<max){	return I_I_left_overlapping;// (2)
				}else{	return I_I_over;}// (3)
			}else{	// (4)-(6)
				if(I.min<max){	// (4),(5)
					if(I.max<max){	return I_I_inside;// (4)
					}else{	return I_I_right_overlapping;}// (5)
				}else{return I_I_right;}//(6)
			}
		}
	}

	inline ostream& into_stream(ostream &os)const{
		return (os	<<"["<<min<<","<<max<<"]");
	}
};

ostream& operator<<(ostream &os, const Interval &I);

class Variable_Limit: public Interval{
public:

	inline Variable_Limit():Interval(){}
	inline Variable_Limit(const double min, const double max):Interval(min,max){}
//	Variable_Limit(const Variable_Limit&)	= default;
//	Variable_Limit(const Variable_Limit&&)	= default;
//	Variable_Limit& operator=(const Variable_Limit&)	= default;
//	Variable_Limit& operator=(const Variable_Limit&&)	= default;

	inline Variable_Limit& add_limit_min(const double min){
		this->min	= max_of(min,this->min);
		return *this;
	}
	inline Variable_Limit& add_limit_max(const double max){
		this->max	= min_of(max,this->max);
		return *this;
	}
	inline Variable_Limit& add_limit_min_max(const double min, const double max){
		this->min	= max_of(min,this->min);
		this->max	= min_of(max,this->max);
		return *this;
	}
	inline Variable_Limit& add_limit(const Interval &additional_Limit){
		if(is_valid() && additional_Limit.is_valid()){
			this->min	= max_of(additional_Limit.min,this->min);
			this->max	= min_of(additional_Limit.max,this->max);
		}else{
			set_unvalid();
		}
		return *this;
	}

	inline double get_value_linear(const double x)const{
		return (min + x*size());
	}

	inline double get_value_linear_reversed(const double x)const{
		return (max - x*size());
	}
};

class Variable_Exclusion: public Interval{
public:

	inline Variable_Exclusion& apply_to_limit(Variable_Limit &Limit){
		if(is_valid() && Limit.is_valid()){
			switch (position_of(Limit)){
			case I_I_left:
			case I_I_right:
				set_unvalid();
				break;
			case (I_I_left_overlapping):
				Limit.add_limit_max(min);
				set_unvalid();
				break;
			case (I_I_inside):
				Limit.set_unvalid();
				set_unvalid();
				break;
//			case (I_I_over):
//				//nothing
//				break;
			case (I_I_right_overlapping):
				Limit.add_limit_min(max);
				set_unvalid();
				break;
			default:
				break;
			}

		}
		return *this;
	}

};


}  // namespace NS_Interval



}  // namespace CuTe
