#include "Res.h"
#include "Res_IQI.h"
#include "Res_K_Integral.h"
#include "Res_Parameter.h"
#include "param.h"

using CuTe::NS_Resumed::_Quad_Differential_Kernel;
using CuTe::NS_Resumed::Quad_Differential_Kernel_LL;
using CuTe::NS_Resumed::Quad_Differential_Kernel_LLO;
using CuTe::NS_Resumed::Pol_ae;

void _Quad_Differential_Kernel::_init(const CS_Parameter *p, const Res_Kinematic_Parameters *RKP){
	qT		= &(RKP->qT);
	order(p->order);
	gexp->init(p->order,p->pc,p->exponent,&(p->nf),&(p->as),&(p->LM),p->prod);
	iqi->init(p->prod,p->order,p->improved_pc(),p->pdfs,&(p->mu),&(p->as),&(p->nf),p->ckm,p->Theta_W,p->active_flv_charges);
}

void _Quad_Differential_Kernel::set_qT_mu(const Res_Kinematic_Parameters *RKP){//,const Continious_Order_Reduction *reduction){
//	NS_Utils::DOUT dout("Quad_Differential_Kernel::set_qT_mu");
//	this->reduction	= reduction;
	K->update_mu();//reduction);
//	K->print_Kl(dout());
//	RKP->properties(cout,dout.get_ntab());
}

void _Quad_Differential_Kernel::set_y(const Res_Kinematic_Parameters *RKP){
//	NS_Utils::DOUT dout("Quad_Differential_Kernel::set_y");
	iqi->set_y(*RKP);
//	dout()	<<"IQI:"<<*iqi<<endl;
}

Pol_ae& _Quad_Differential_Kernel::update_zz(const Res_Kinematic_Parameters *RKP){
//	NS_Utils::DOUT dout("Quad_Differential_Kernel::update_zz");
//	RKP->properties(cout,dout.get_ntab());


	iqi->update_zz(*RKP);//,reduction);

//	iqi->properties(cout,dout.get_ntab());

	K->replaceL_E(*iqi);

	set_equalt_to(*K,_order);

//		bool ok=true;
//		for(int i=0; i<=_order; ++i) if(_P[i]!=_P[i]) ok=false;	 DEBUG
//		if(!ok){
//			cerr	<<"NAN:	Quad_Differential_Kernel::update_zz\tzz=["<<RKP.zz[0]<<","<<RKP.zz[1]<<"]"<<endl
//					<<"     IQI:"<<endl<<iqi<<endl;
//			cerr	<<RKP[0]->to_String()	<<RKP[1]->to_String();
//		}

//		dout()	<<"this:"<<endl;
//		dout()	<<*this<<endl;

	mul_this_by_scalar(RKP->Jac_z);

//		dout()	<<"this*Jac_z(="<<RKP.Jac_z<<"):"<<endl;
//		dout()	<<*this<<endl;

	return *this;
}


_Quad_Differential_Kernel::_Quad_Differential_Kernel(const RES res)
:Pol_ae(5,0.)
,res(res)
,qT(NULL)
,gexp(new G_exp_Pol()),iqi(new IQI())
,K(NULL)
//,reduction(NULL)
{
//		NS_Utils::DOUT dout("Quad_Differential_Kernel");
}

_Quad_Differential_Kernel::_Quad_Differential_Kernel(const _Quad_Differential_Kernel& rv)
:Pol_ae(5,0.)
,res(rv.res)
,qT(rv.qT)
,gexp(new G_exp_Pol(*(rv.gexp))),iqi(new IQI(*(rv.iqi)))
,K(NULL)
//,reduction(rv.reduction)
{
	set_equalt_to(rv);
}

_Quad_Differential_Kernel& _Quad_Differential_Kernel::operator=(const _Quad_Differential_Kernel& rv){
//		NS_Utils::DOUT dout("Quad_Differential_Kernel::operator=","rv");
	set_equalt_to(rv);

	qT		= (rv.qT);
	*gexp	= *(rv.gexp);
	*iqi	= *(rv.iqi);
//	reduction	= (rv.reduction);

	return *this;
}

_Quad_Differential_Kernel::~_Quad_Differential_Kernel(){
	delete gexp;	// No =NULL
	delete iqi;	// No =NULL
}


void _Quad_Differential_Kernel::print_pdf(const string s)const{
	iqi->print_pdf(s);
}

void _Quad_Differential_Kernel::change_PDF(const CS_Parameter *p){
//	K	-> pointer
//	gexp-> pointer
	iqi->change_PDF(p->pdfs);
}

void Quad_Differential_Kernel_LLO::_del_K(){
	delete static_cast<K_LLO*>(K);	K = NULL;
}

void Quad_Differential_Kernel_LLO::_init_K(){
	_del_K();
	K	= new K_LLO(*gexp,qT,Lmu,etaB);
}


void Quad_Differential_Kernel_LLO::init(const CS_Parameter_LLO *p, const Res_Kinematic_Parameters *RKP){
	_init(p,RKP);
	Lmu	= &(p->Lmu);
	etaB= &(p->etaB);
	_init_K();
}

void Quad_Differential_Kernel_LL::_init_K(){
	_del_K();
	K	= new K_LL(*gexp,qT,mu,LM,*cutoff,Lambda);
}

void Quad_Differential_Kernel_LL::_del_K(){
	delete static_cast<K_LL*>(K);	K = NULL;
}

void Quad_Differential_Kernel_LL::change_NPE(){
	if(K!=NULL) static_cast<K_LL*>(K)->change_NPE(*cutoff);
}


void Quad_Differential_Kernel_LL::init(const CS_Parameter_LL *p, const Res_Kinematic_Parameters *RKP){
	_init(p,RKP);
	mu		= &(p->mu);
	LM		= &(p->LM);
//	q_star	= &(p->q_star);
	cutoff	= &(p->cutoff);
	Lambda	= &(p->Lambda);
	_init_K();
}
