#include "Res_Parameter.h"
#include "Kinematics.h"
#include "Ibar_G_Parameter.h"
#include "Ibar_Q_Parameter.h"

using CuTe::NS_Resumed::Res_Kinematic_Parameters;
using CuTe::NS_Resumed::Ibar_Parameter_Set;

using CuTe::NS_Ibar::Ibar_G_Parameter;
using CuTe::NS_Ibar::Ibar_Q_Parameter;

void Ibar_Parameter_Set::_exit_I(){
	for(int i=0; i<2; ++i){
		switch (prod) {
			case PROD_Higgs:
			case PROD_H_RS:
				delete static_cast<Ibar_G_Parameter*>(I[i]);
				break;
			case PROD_DY:
			case PROD_Z:
			case PROD_W:
				delete static_cast<Ibar_Q_Parameter*>(I[i]);
				break;
			default:
				break;
		}
		I[i]	= NULL;
	}
}
void Ibar_Parameter_Set::_init_I(){
	for(int i=0; i<2; ++i){
		switch (prod) {
			case PROD_Higgs:
			case PROD_H_RS:
				I[i]	= new Ibar_G_Parameter();
				break;
			case PROD_DY:
			case PROD_Z:
			case PROD_W:
				I[i]	= new Ibar_Q_Parameter();
				break;
			default:
				break;
		}
		I[i]->init(order,improved,order_a);
	}
}

void Ibar_Parameter_Set::set_y(const Kinematics *Kin){
	for(int i=0; i<2; ++i) I[i]->set_x(Kin->xi_pm[i]);
}

void Ibar_Parameter_Set::update_zz(const Kinematics *Kin){
	Jac_z	= Kin->Jac_z;
	for(int i=0; i<2; ++i) I[i]->update(Kin->z[i]);
}

void Res_Kinematic_Parameters::init(const Kinematics *Kin,PROD prod, const int order,const bool improved, const int order_a){
//	NS_Utils::DOUT dout("Res_Kinematic_Parameters::init");
	Ibar_Parameter_Set::init(prod,order,improved,order_a);

//	dout()	<<"ord="<<order	<<"\timp="<<improved	<<"\tord_a="<<order_a	<<endl;
	qT	= Kin->qT;
	zz	= Kin->z;
//	xx	= Kin->x;
}

void Res_Kinematic_Parameters::set_qT(const Kinematics *Kin){
	qT	= Kin->qT;
}



std::ostream& Ibar_Parameter_Set::properties(std::ostream & os, const int ntab)const{
	std::string tab;
	for(int i=0; i<ntab; ++i) tab += "\t";
	os	<<tab<<"Ibar_Parameter_Set:"<<endl;
	os	<<tab;
	os	<<"\t"<<"prod = "<<prod;
	os	<<endl;
	os	<<tab;
	os	<<"\t"<<"order = "<<order;
	os	<<"\t"<<"imp = "<<improved;
	os	<<"\t"<<"order_a = "<<order_a;
	os	<<endl;
	os	<<tab;
	os	<<"\t"<<"I[0]:"<<endl<<I[0]->to_String(ntab+1);
	os	<<endl;
	os	<<tab;
	os	<<"\t"<<"I[1]:"<<endl<<I[1]->to_String(ntab+1);
	os	<<endl;

	return os;
}
std::ostream& Res_Kinematic_Parameters::properties(std::ostream & os, const int ntab)const{
	std::string tab;
	for(int i=0; i<ntab; ++i) tab += "\t";
	os	<<tab<<"Res_Kinematic_Parameters:"<<endl;
	os	<<tab;
	os	<<"\t"<<"qT = "<<qT;
	os	<<"\t"<<"zz = ["<<zz[0]<<","<<zz[1]<<"]";
//	os	<<"\t"<<"xx = ["<<xx[0]<<","<<xx[1]<<"]";
	os	<<endl;
	return Ibar_Parameter_Set::properties(os,ntab+1);
}
