#ifndef STEPFUNCTION_H_
#define STEPFUNCTION_H_

#include "enums.h"
#include <vector>


namespace CuTe {

namespace NS_CS_Params {
	class SOPS_CS_Params;
}  // namespace NS_CS_Params

namespace NS_Stepfunction {

using namespace NS_enums::NS_Transition_Function;
using NS_CS_Params::SOPS_CS_Params;
using std::vector;
using std::ostream;

class Sigmoid{
protected:
//	double l,r,R;
	double m,w;

	inline Sigmoid()
//	:l(0),r(0),R(0)
	:m(0),w(0){
	}

	inline Sigmoid(const double l, const double r, const double R)
//	:l(l),r(r),R(R)
	:m(0.5*(r+l)),w(0.5*(r-l)){
	}

	inline Sigmoid(const Sigmoid& S)
//	:l(S.l),r(S.r),R(S.R)
	:m(S.m),w(S.w){
	}

	inline Sigmoid& operator=(const Sigmoid& S){
//		l	= (S.l);
//		r	= (S.r);
//		R	= (S.R);
		m	= (S.m);
		w	= (S.w);

		return *this;
	}

	virtual ~Sigmoid(){
	}

public:
	inline virtual double f(const double x)const{return 0.;}

	inline virtual double barf(const double x)const{return 0.;}

	inline double operator()(const double x)const{return f(x);}

	virtual ostream& to_stream(ostream &os, const int ntabs=0)const;
};

//class Stepfunction{
//protected:
//	Sigmoid *S;
//
//	double mu;
//	double width;		// = (right-left)/2
//	double left;
//	double right;
//	double reduction;	// = f(left)	= 1. - f(right)
//	ENUM_Stepfunction_Type type;
//
//	void _init(const ENUM_Stepfunction_Type type);
//	void _exit();
//
//
//public:
//
//	// f(left)=1-reduction	f(right)=reduction
//	inline double operator()(const double x)const{ return (*S)(x);}
//	inline double f(const double x)const{ return (*S).f(x);}
//	inline double barf(const double x)const{ return (*S).barf(x);}
//
//
//	bool set_to_one();
//	bool init_mu_sigma(const double mu=0., const double width=1., const double reduction=0.05, const ENUM_Stepfunction_Type type=SFT_Erf);
//	inline bool init_left_right(const double left=(-1.), const double right=(+1.), const double reduction=0.05, const ENUM_Stepfunction_Type type=SFT_Erf){
//		return init_mu_sigma(0.5*(right+left),0.5*(right-left),reduction,type);
//	}
//
//	inline Stepfunction()
//	:S(NULL)
//	,mu(0.),width(1.),reduction(0.05),type(){
//		_init(SFT_Erf);
//	}
//
//	inline Stepfunction(const Stepfunction &RV)
//	:S(NULL)
//	,mu(RV.mu),width(RV.width),reduction(RV.reduction),type(){
//		_init(RV.type);
//	}
//
//	inline Stepfunction& operator=(const Stepfunction &RV){
//		mu			= (RV.mu);
//		width		= (RV.width);
//		reduction	= (RV.reduction);
//		_init(RV.type);
//		return *this;
//	}
//
//	inline ~Stepfunction(){ _exit();};
//
//	bool init(const SOPS_CS_Params *OP,const ENUM_Stepfunction_Type type);
//	bool init(const SOPS_CS_Params *OP);
//
//	ostream& to_stream(ostream &os, const int ntabs=0)const;
//
//	inline ENUM_Stepfunction_Type get_type()const{	return type;}
//	inline bool is_active()const{return (type!=SFT_No);};
//
//};

class TransitionFunction{
protected:
	Sigmoid *S;
	double iS0;

	ENUM_Transition_Function_Type	type;
	ENUM_Transition_Function_Arg	arg;
	double left,right,Reduction;
	double M,CB;



	void _init(const ENUM_Transition_Function_Type type);
	void _exit();


public:

	inline double eta_of(const double lambda)const{
		return -2.*CB*M_1_PI*log(lambda);
	}

	inline double lambda_of(const double eta)const{
			return exp(-eta*M_PI/(2.*CB));
	}

	// f(left)=1-reduction	f(right)=reduction
	double f(const double qT)const;
	inline double       barf(const double qT)const{ return 1.-f(qT);}
	inline double operator()(const double qT)const{ return f(qT);}

	bool set_to_one();
	inline ENUM_Transition_Function_Type get_type()const{	return type;}
	inline ENUM_Transition_Function_Arg get_argument()const{	return arg;}
	inline bool is_active()const{return (type!=TFT_NO);};

	inline TransitionFunction()
	:S(NULL),iS0()
	,type(),arg()
	,left(),right(),Reduction()
	,M(),CB(){
		set_to_one();
	}

	inline TransitionFunction(const TransitionFunction &RV)
	:S(NULL),iS0()
	,type(),arg(RV.arg)
	,left(RV.left),right(RV.right),Reduction(RV.Reduction)
	,M(RV.M),CB(RV.CB){
		_init(RV.type);
	}

	inline TransitionFunction& operator=(const TransitionFunction &RV){
		arg			= (RV.arg);

		left		= (RV.left);
		right		= (RV.right);
		Reduction	= (RV.Reduction);

		M	= (RV.M);
		CB	= (RV.CB);


		_init(RV.type);

		return *this;

	}

	inline ~TransitionFunction(){ _exit();}

	bool init(vector<double>::const_iterator lrR, const double M, const double CB, ENUM_Transition_Function_Type type, const ENUM_Transition_Function_Arg arg);
	bool init(const SOPS_CS_Params *OP,const ENUM_Transition_Function_Type type, const ENUM_Transition_Function_Arg arg);
	bool init(const SOPS_CS_Params *OP);

	ostream& to_stream(ostream &os, const int ntabs=0)const;


};

}	// namespace NS_Stepfunction
}	// namespace CuTe

#endif //STEPFUNCTION_H_
