#ifndef TDPDF_PLOT_H
#define TDPDF_PLOT_H

#include "TDPDF.h"
#include "TDPDF_options.h"
#include "param.h"

namespace CuTe {

namespace NS_TDPDF {

//using NS_CS_Params::MU_star;

using namespace NS_enums::NS_BFUNCS;
using namespace NS_enums::NS_MU_OPS;

using NS_CUBA_integration::OPS_CUBA;
using NS_TDPDF_Integrand::_TMD_PDF_Integrand;
using NS_TDPDF_Integrand::INT_B;
using NS_TDPDF_Integrand::INT_XI_B;
using NS_TDPDF_Integrand::INT_B_over_Phi;
using NS_TDPDF_Integrand::INT_Btilde;
using NS_TDPDF_Integrand::INT_XI_Btilde;
using NS_TDPDF_Integrand::INT_Btilde_over_Phi;

class MU_Star_XT:public NS_CS_Params::MU_star{
protected:
//	bool to_calc;
//	double q_star;
//	double mu;
	double b0_active;

	inline void init(const SOPS_CS_Params& op,const NS_LHAFUNC::A_strong* as);
	inline void init(const PROD prod, const double M, const OPS_Scales& op,const NS_LHAFUNC::A_strong* as);

public:
//	double _MUFAC;
//	bool errors;
//	vector<double> mus;


	inline void init(const PROD prod, const double M, const bool FT, const OPS_Scales& op_s,const NS_LHAFUNC::A_strong* as){
		if(FT){
			MU_star::init(prod,M,op_s,as);
		}else{
			_MUFAC	= (op_s.scale_variation());
			errors	= (_MUFAC>0);

			if(op_s.fix_mu_to_const.is_set){
				switch (op_s.fix_mu_to_const()) {
				case ENUM_MU_M:
					mu	= M;
					to_calc	= false;
					break;
				case ENUM_MU_LP0:
					to_calc		= true;
					b0_active	= b_0;
					break;
				case ENUM_MU_B0:
					to_calc	= true;
					b0_active	= 1.;
					break;
				case ENUM_MU_QSS:
					to_calc	= false;
					{
						NS_CS_Params::Q_star QS(as,M,C_A*GAMMA_0);
						mu	= QS();
					}
					break;
				case ENUM_MU_QSV:
					to_calc	= false;
					{
						NS_CS_Params::Q_star QS(as,M,C_F*GAMMA_0);
						mu	= QS();
					}
					break;
				default:
					cerr	<<"***************************************************************"<<endl
							<<"**** MU_star_XT::init WRONG fix_mu_to_const Option -> exit(1) ****"<<endl
							<<"***************************************************************"<<endl;
					exit(1);
					break;
				}
			}else if(op_s.fix_mu_to_value.is_set){
				mu	= op_s.fix_mu_to_value();
				to_calc	= false;
			}else{
				to_calc	= true;
			}

			if(errors){
	//			dout()	<<"errors"<<endl;
				mus.resize(2);
				if(!to_calc){
					mus[0] = mu/_MUFAC;
					mus[1] = mu*_MUFAC;
				}
			}else{
	//			dout()	<<"No errors"<<endl;
				mus.resize(0);
			}
		}
	}

	inline void init(const SOPS_TDPDF& op,const NS_LHAFUNC::A_strong* as){
		return init(op.Process.prod(),op.Process.M(),op.TDPDF.TDPDF_Fourier_Transform(),op.Scales,as);
	}

//	inline int N_errors()const{return mus.size();}

//	inline double operator() (){
//		return mu;
//	}

protected:
	inline double operator() (double qT);
//	{
//		if(to_calc){
//			mu = qT;
//			if(q_star!=0.)	mu += q_star*exp(-qT/q_star);
//			if(errors){
//				mus[0] = mu/_MUFAC;
//				mus[1] = mu*_MUFAC;
//			}
//		}
//		return mu;
//	}
public:
	inline double mu_of_qT(double qT){return MU_star::operator ()(qT);}
	inline double mu_of_xT(double xT){
		if(to_calc){
			mu	= b0_active/xT;
			if(errors){
				mus[0] = mu/_MUFAC;
				mus[1] = mu*_MUFAC;
			}
		}
		return mu;
	}

	/*
	 * Only for Polynom
	 *  if
	 * 		fix_mu_to_const is not set
	 * 		&&
	 * 		fix_mu_to_value is not set
	 * 	-> to_calc use values given
	 * 	else
	 * 		use fixed values from init
	 */
	inline double mu_of_mu(double mu){
		if(to_calc){
			this->mu	= mu;
			if(errors){
				mus[0] = mu/_MUFAC;
				mus[1] = mu*_MUFAC;
			}
		}
		return this->mu;
	}


	MU_Star_XT()
	:MU_star()
	,b0_active(0.){

	}

//	inline void calc_errors(vector<double>& results, const vector<double>& values, int i_muP, int i_muM)const{
//		double min,max;
//		NS_Utils::parabola_min_max(mus[0],values[1],mu,values[0],mus[1],values[2],min,max);
//
//		results[i_muP]	= max;
//		results[i_muM]	= min;
//	}
//
//	inline void calc_errors(double &mup, double &mum, double value, double value_P, double value_M)const{
//		double min,max;
//		NS_Utils::parabola_min_max(mus[0],value_P,mu,value,mus[1],value_M,min,max);
//
//		mup	= max;
//		mum	= min;
//
//	}
//
//	inline vector<double>::iterator& calc_errors(vector<double>::iterator& results_mup_mum, double value, double value_P, double value_M)const{
//		double min,max;
//		NS_Utils::parabola_min_max(mus[0],value_P,mu,value,mus[1],value_M,min,max);
//
//		*(results_mup_mum++)	= max;
//		*(results_mup_mum++)	= min;
//
//		return results_mup_mum;
//	}
//
//	inline void calc_errors(vector<double>& results, const vector<double>& values)const{
//		calc_errors(results,values,ER_erg_mu_P,ER_erg_mu_M);
//	}

};


class TD_PDF_TO_PLOT:public TD_PDF {
public:
	vector<double> mu_p;
	vector<double> mu_m;
protected:
	bool FT,NORM;
	MU_Star_XT mu_star;
//	const Options *op;
	const NS_CUBA_integration::OPS_CUBA *op_cuba;

	vector<_TMD_PDF_Integrand> _integrand_mu_pm;

	double  _mu[2];
	int 	_nf[2];//nf(mu)
	double  _as[2];//alpha_s(mu)/(4*pi)
	double  _LT[2];//ln( xT^2 mu^2 b_0^(-2) )
	double  _Lmu[2];//ln(mu^2/qT^2)
	double  _LM[2];//ln(M^2/mu^2)

	inline void _init_cuba(){
		if(mu_star.errors){
			cuba.init(*op_cuba,1,3*NB);
		}else{
			cuba.init(*op_cuba,1,NB);
		}
	}

	inline void _init_integrand(){
		TD_PDF::_init_integrand();

		if(mu_star.errors){
			_integrand_mu_pm.resize(2,_td_integrand);
			for(int i=0; i<2; ++i){
				_integrand_mu_pm[i].init(B,pdf,&qT,&xT,&(_mu[i]),&(_nf[i]),&(_as[i]),&(_LT[i]),&(_Lmu[i]),&q_star,res,order,pc,exponent,cutoff,&accuracy,&Lambda,&(_LM[i]),1.,0.);
			}
		}else{
			_integrand_mu_pm.resize(0);
		}
	}

//	inline void _init_parameter()
//	inline void _init_pdf();

	inline void _set_xi_xT_mu(double xi, double xT, double mu);
	inline void _set_xi_xT_mu(double xi, double xT){
//		NS_Utils::DOUT	dout("TD_PDF_TO_PLOT::_set_xi_xT_mu","xi,xT");

		mu	= mu_star.mu_of_xT(xT);
		TD_PDF::_set_xi_xT_mu(xi,xT,mu);

		if(mu_star.errors){
			for(int i=0; i<2; ++i){
				_mu[i]	= mu_star.mus[i];
				_nf[i]	= pdf.nf(_mu[i]);
				_as[i]	= pdf.as(_mu[i]);
				_LT[i]	= log(pow_2(xT*_mu[i]/b_0));
//				_Lmu[i]	= log(pow_2(_mu[i]/qT));

				if(M>0.){
					_LM[i]	= log(pow_2(M/_mu[i]));
				}else{
					_LM[i]	= 0.;
				}

				_integrand_mu_pm[i].set_xi_xT_mu(IG,IQ);
			}
		}

	}

	inline void _set_xi_qT_mu(double xi, double xT, double mu);
	inline void _set_xi_qT_mu(double xi, double qT){
//		NS_Utils::DOUT	dout("TD_PDF_TO_PLOT::_set_xi_qT_mu","xi,qT");

		mu	= mu_star.mu_of_qT(qT);
		TD_PDF::_set_xi_qT_mu(xi,qT,mu);

		if(mu_star.errors){
			for(int i=0; i<2; ++i){
				_mu[i]	= mu_star.mus[i];
				_nf[i]	= pdf.nf(_mu[i]);
				_as[i]	= pdf.as(_mu[i]);
//				_LT[i]	= log(pow_2(xT*_mu[i]/b_0));
				_Lmu[i]	= log(pow_2(_mu[i]/qT));

				if(M>0.){
					_LM[i]	= log(pow_2(M/_mu[i]));
				}else{
					_LM[i]	= 0.;
				}

				_integrand_mu_pm[i].set_xi_qT_mu(IG,IQ);
			}
		}
	}

	template< E_INT e_int >
	inline void _set_xi_xT_or_qT_mu(double xi, double xT_or_qT){
		switch (e_int) {
		case INT_B:
		case INT_XI_B:
		case INT_B_over_Phi:
			_set_xi_xT_mu(xi,xT_or_qT);
			break;
		case INT_Btilde:
		case INT_XI_Btilde:
		case INT_Btilde_over_Phi:
			_set_xi_qT_mu(xi,xT_or_qT);
			break;
		default:
			cerr	<<"*********************************************************************************"<<endl;
			cerr	<<"***** TDD_PDF_TO_PLOT::_set_xi_xT_or_qT_mu<e_int>() WRONG e_int:"<<e_int<<" *****"<<endl;
			cerr	<<"*********************************************************************************"<<endl;
			exit(1);
			break;
		}
	}

	template< E_INT e_int >
	inline double* _integrand(double ff[], double z, double Jac){
//		NS_Utils::DOUT dout("TDD_PDF_TO_PLOT::_integrand<e_int>","ff,z,Jac");
		// INT_B
		ff	= TD_PDF::_integrand<e_int>(ff,z,Jac);

		if(mu_star.errors){
			ff	= _integrand_mu_pm[0].integrand<e_int>(IG,IQ,ff,Jac);
			ff	= _integrand_mu_pm[1].integrand<e_int>(IG,IQ,ff,Jac);
		}
	}

	template< E_INT e_int >
	inline static int _cuba_int(const int *ndim, const double xx[],const int *ncomp, double ff[], void *userdata){
//		NS_Utils::DOUT dout("TDD_PDF_TO_PLOT::_cuba_int<e_int>","...");
		TD_PDF_TO_PLOT* tdd_pdf = static_cast<TD_PDF_TO_PLOT*>(userdata);
		double xi(tdd_pdf->xi);
		double Jac(1.-xi);
		double z(xi + Jac*xx[0] );

		tdd_pdf->_integrand<e_int>(ff,z,Jac);

		return 0;
	}

	template< E_INT e_int >
	inline void _integrate(){
//		NS_Utils::DOUT dout("TDD_PDF_TO_PLOT::_integrate<e_int>","...");
		cuba.cube(_cuba_int<e_int>,results,integration_errors,integration_probabilities,this);
//		cuba.test(1,_cuba_int<e_int>,results,integration_errors,integration_probabilities,this);
	}
//
//	inline double* _integrand_B(double ff[], double z, double Jac){
////		NS_Utils::DOUT	dout("TDD_PDF_TO_PLOT::_cuba_int()","ff,z,Jac");
//
//		ff	= TD_PDF::_integrand_B(ff,z,Jac);
//
//		if(mu_star.errors){
//			ff	= _integrand_mu_pm[0].integrand_B_times_xi(IG,IQ,ff,Jac);
//			ff	= _integrand_mu_pm[1].integrand_B_times_xi(IG,IQ,ff,Jac);
//		}
//
//		return ff;
//	}
//
//	inline static int _cuba_int_B(const int *ndim, const double xx[],const int *ncomp, double ff[], void *userdata){
////		NS_Utils::DOUT	dout("TDD_PDF_TO_PLOT::_cuba_int_()","*ndim...");
//		TD_PDF_TO_PLOT* tdd_pdf = static_cast<TD_PDF_TO_PLOT*>(userdata);
//		double xi(tdd_pdf->xi);
//		double Jac(1.-xi);
//		double z(xi + Jac*xx[0] );
//
//		tdd_pdf->_integrand_B(ff,z,Jac);
//
//		return 0;
//	}
//
//	inline void _integrate_B(){
////		NS_Utils::DOUT	dout("TDD_PDF_TO_PLOT::_integrate()");
//		cuba.cube(_cuba_int_B,results,integration_errors,integration_probabilities,this);
////		cuba.test(1,_cuba_int_B,results,integration_errors,integration_probabilities,this);
//	}
//
//	inline double* _integrand_BTilde(double ff[], double z, double Jac){
////		NS_Utils::DOUT	dout("TDD_PDF_TO_PLOT::_cuba_int()","ff,z,Jac");
//
//		ff	= TD_PDF::_integrand_BTilde(ff,z,Jac);
//
//		if(mu_star.errors){
//			ff	= _integrand_mu_pm[0].integrand_BTilde_times_xi(IG,IQ,ff,Jac);
//			ff	= _integrand_mu_pm[1].integrand_BTilde_times_xi(IG,IQ,ff,Jac);
//		}
//
//		return ff;
//	}
//
//	inline static int _cuba_int_BTilde(const int *ndim, const double xx[],const int *ncomp, double ff[], void *userdata){
////		NS_Utils::DOUT	dout("TDD_PDF_TO_PLOT::_cuba_int_()","*ndim...");
//		TD_PDF_TO_PLOT* tdd_pdf = static_cast<TD_PDF_TO_PLOT*>(userdata);
//		double xi(tdd_pdf->xi);
//		double Jac(1.-xi);
//		double z(xi + Jac*xx[0] );
//
//		tdd_pdf->_integrand_BTilde(ff,z,Jac);
//
//		return 0;
//	}
//
//	inline void _integrate_BTilde(){
////		NS_Utils::DOUT	dout("TDD_PDF_TO_PLOT::_integrate()");
//		cuba.cube(_cuba_int_BTilde,results,integration_errors,integration_probabilities,this);
////		cuba.test(1,_cuba_int_BTilde,results,integration_errors,integration_probabilities,this);
//	}


	inline vector<double>& _set_from_cuba(){
		if(mu_star.errors){
//			NS_Utils::DOUT	dout("TDD_PDF_TO_PLOT::_set_from_cuba()","mu_star.errors");
			mu_p.resize(NB);
			mu_m.resize(NB);

			vector<double>::iterator val(results.begin());
			vector<double>::const_iterator val_p(val+NB);
			vector<double>::const_iterator val_m(val_p+NB);
			vector<double>::iterator it_p(mu_p.begin());
			vector<double>::iterator it_m(mu_m.begin());

			for(int i=0; i<NB; ++i){
				mu_star.calc_min_max(*(val++),*(val_p++),*(val_m++),*(it_p++),*(it_m++));
//				mu_star.calc_errors(*(it_p++),*(it_m++),*(val++),*(val_p++),*(val_m++));
			}
			results.resize(NB);
			integration_errors.resize(NB);
			integration_probabilities.resize(NB);
		}
		return results;
	}

	inline vector<double>& B_of_(double xi, double xT, double mu);
	inline vector<double>& BTilde_of_(double xi, double qT, double mu);

public:

	inline vector<double>& xi_times_B_of_(double xi, double xT){
		_set_xi_xT_or_qT_mu<INT_XI_B>(xi,xT);
		_integrate<INT_XI_B>();
		return _set_from_cuba();
	}

	inline vector<double>& xi_times_BTilde_of_(double xi, double qT){
		_set_xi_xT_or_qT_mu<INT_XI_Btilde>(xi,qT);
		_integrate<INT_XI_Btilde>();
		return _set_from_cuba();
	}

	inline vector<double>& B_over_phi_of_(double xi, double xT){
		_set_xi_xT_or_qT_mu<INT_B_over_Phi>(xi,xT);
		_integrate<INT_B_over_Phi>();
		return _set_from_cuba();
	}

	inline vector<double>& BTilde_over_phi_of_(double xi, double qT){
		_set_xi_xT_or_qT_mu<INT_Btilde_over_Phi>(xi,qT);
		_integrate<INT_Btilde_over_Phi>();
		return _set_from_cuba();
	}

	inline vector<double>& operator()(double xi, double xT_or_qT){
//		NS_Utils::DOUT	dout("TD_PDF_TO_PLOT::operator()","xi,qT");
//		dout()	<<"FT="<<FT<<endl;
		if(FT){
			if(NORM){
				return BTilde_over_phi_of_(xi,xT_or_qT);
			}else{
				return xi_times_BTilde_of_(xi,xT_or_qT);
			}
		}else{
			if(NORM){
				return B_over_phi_of_(xi,xT_or_qT);
			}else{
				return xi_times_B_of_(xi,xT_or_qT);
			}
		}
	}

	inline void init(const SOPS_TDPDF &OP){
//		NS_Utils::DOUT	dout("TDD_PDF_TO_PLOT::init","op");
//		this->op	= &op;
		this->op_cuba	= &(OP.CUBA);

		FT		= OP.TDPDF.TDPDF_Fourier_Transform();
		NORM	= OP.TDPDF.TDPDF_Normalize();

		if(OP.PDF.PDF_Dummy_a_b.is_set){
//			NS_Utils::DOUT	dout("PDF_Dummy_a_b.is_set");
			_init_pdf(OP.PDF.PDFset(),OP.PDF.PDF_Dummy_a_b[0],OP.PDF.PDF_Dummy_a_b[1],OP.PDF.PDF_member());
		}else{
//			NS_Utils::DOUT	dout(op.PDFset());
			_init_pdf(OP.PDF.PDFset(),OP.PDF.PDF_member());
		}
		mu_star.init(OP,pdf.get_A_strong());

		accuracy	= OP.CUBA.accuracy();
		res			= OP.Scheme.resummation();
		M			= OP.Process.M();

		B			= OP.TDPDF.TDPDF_func();
		order		= OP.Scheme.order();

		if(res==RES_LL){
			pc	= OP.Scheme.Power_Counting();
		}else{
			pc	= PC_STD;
		}
		if(improved_pc()){
			if(OP.Scheme.order_epsilon.is_set){
				order	= OP.Scheme.order_epsilon();
			}else{
				order	= 2*order + 1;
			}
		}
		exponent	= OP.Scheme.exponent();

		cutoff		= OP.Scales.cutoff();
		Lambda		= OP.Scales.LambdaNP();
		if( (cutoff!=CUTOFF_NO) && (Lambda==0.) ) cutoff=CUTOFF_NO;


		_init_cuba();
		_init_integrand();
		_init_parameter();
	}

	inline const string FL_exp()const{
		std::stringstream ss;

		ss	<<"######################################################################################"<<endl
			<<"##"<<"\t"<<"as"<<"\t"<<		"= alpha_s(mu)/(4*pi)"<<endl;

		if(FT){
			ss	<<"##"<<"\t"<<"results"<<"\t"<<	"= xi * tilde("<<B<<")(xi,qT,mu)"<<endl;

		}else{
			ss	<<"##"<<"\t"<<"LT"<<"\t"<<		"= ln(xT^2*mu^2/b_0^2) \t\t b_0 = 2*exp(-gamma_E)"<<endl
				<<"##"<<"\t"<<"results"<<"\t"<<	"= xi * "<<B<<"(xi,xT,mu)"<<endl;
		}
		ss	<<"##"<<"\t"<<"delta"<<"\t"<<	"= absolute Integration Error"<<endl
			<<"##"<<"\t"<<"mu+"<<"\t"<<		"= approximated max value in [mu/"<<mu_star._MUFAC<<",mu*"<<mu_star._MUFAC<<"]"<<endl
			<<"##"<<"\t"<<"mu-"<<"\t"<<		"= approximated min value in [mu/"<<mu_star._MUFAC<<",mu*"<<mu_star._MUFAC<<"]"<<endl
			<<"##"<<"\t"<<"G"<<"\t"<<		"= sqrt( G1^2 + G2^2 )"<<endl
			<<"######################################################################################"<<endl;

		return ss.str();
	}

	inline const string FL()const{
		std::stringstream ss;
		std::stringstream p;

		if(FT){
			ss <<"#xi\tqT\tmu\tas";
		}else{
			ss <<"#xi\txT\tmu\tas\tLT";
		}

		if(cutoff!=CUTOFF_NO)	ss<<"\tLambda_NP";

		for(int i=0; i<NB; ++i){
			p<<static_cast<ENUM_TDPDF_Parton>(i);
			ss <<"\t"<<p.str()<<"\t"<<"delta_"<<p.str();
			if(mu_star.errors) ss<<"\t"<<"mu+_"<<p.str()<<"\t"<<"mu-_"<<p.str();
			p.str(string());
		}
		ss<<endl;

		return ss.str();
	}

	inline const string to_String(int ntabs=0)const{
		std::stringstream ss;
		for(int i=0; i<ntabs; ++i) ss<<"\t";
		string tab(ss.str());
		ss.str(string());

		if(FT){
			ss <<tab<<xi<<"\t"<<qT<<"\t"<<mu<<"\t"<<as;
		}else{
			ss <<tab<<xi<<"\t"<<xT<<"\t"<<mu<<"\t"<<as<<"\t"<<LT;
		}

		if(cutoff!=CUTOFF_NO)	ss<<"\t"<<Lambda;

		vector<double>::const_iterator res(results.begin());
		vector<double>::const_iterator del(integration_errors.begin());
		vector<double>::const_iterator mup(mu_p.begin());
		vector<double>::const_iterator mum(mu_m.begin());

		for(int i=0; i<NB; ++i){
			ss <<"\t"<<*(res++)<<"\t"<<*(del++);
			if(mu_star.errors) ss<<"\t"<<*(mup++)<<"\t"<<*(mum++);
		}
		ss<<endl;

		return ss.str();
	}

};

class TD_PDF_POL_PLOT{
public:
	vector<double> results,integration_errors,integration_probabilities;
protected:
	MU_Star_XT mu_star;
	ENUM_B_FUNCS B;
	ENUM_TDPDF_Parton i_B;
	NS_LHAFUNC::VectorPDF pdf;
	NS_Ibar::Ibar_G_Parameter IG;
	NS_Ibar::Ibar_Q_Parameter IQ;
	int 	ord;
	bool	imp;
	bool	exp;
	CUTOFF	cut;
	double	LNP;

	int		ncomp;

	double	xi;
	double	xT_for_td;
	double	mu;
	int		nf;
	double	as;
	static const double as_for_td;// =1.;
	double	LT_for_td;
	double	LM_for_td;

	double 	_mu[2];
	int		_nf[2];
	double	_as[2];


	NS_CUBA_integration::Cuba_Params		cuba;
	vector<NS_TDPDF_Integrand::_TDD_PDF_Integrand>	td;


	template< E_INT e_int >
	inline double* _integrand(double ff[], double z, double Jac){
//		NS_Utils::DOUT dout("TD_PDF_POL_PLOT::_integrand");
		IQ.update(z);
		IG.update(z);

		ff	= td[0].integrand_B_times_xi_Pol_i(i_B,IG,IQ,ff,Jac);

		if(mu_star.errors){
			for(int i=0; i<2; ++i)	ff	= td[i+1].integrand_B_times_xi_Pol_i(i_B,IG,IQ,ff,Jac);
		}
	}

	template< E_INT e_int >
	inline static int _cuba_int(const int *ndim, const double xx[],const int *ncomp, double ff[], void *userdata){
//		NS_Utils::DOUT dout("TD_PDF_POL_PLOT::_cuba_int");
		TD_PDF_POL_PLOT* tdd_pdf = static_cast<TD_PDF_POL_PLOT*>(userdata);
		double xi(tdd_pdf->xi);
		double Jac(1.-xi);
		double z(xi + Jac*xx[0] );

		tdd_pdf->_integrand<e_int>(ff,z,Jac);

		return 0;
	}

	template< E_INT e_int >
	inline void _integrate(){
//		NS_Utils::DOUT dout("TD_PDF_POL_PLOT::_integrate");
		cuba.cube(_cuba_int<e_int>,results,integration_errors,integration_probabilities,this);
//		cuba.test(2,_cuba_int<e_int>,results,integration_errors,integration_probabilities,this);
	}

	inline void _init_cuba(const OPS_CUBA& op){
//		NS_Utils::DOUT dout("TD_PDF_POL_PLOT::_init_cuba");
		if(mu_star.errors){
			cuba.init(op,1,3*ncomp);
		}else{
			cuba.init(op,1,ncomp);
		}
	}

	inline void _init_td(){
//		NS_Utils::DOUT dout("TD_PDF_POL_PLOT::_init_td");
		if(mu_star.errors){
			td.resize(3);
			for(int i=0;i<2;++i)	td[i+1].init(B,pdf,&xT_for_td,&(_mu[i]),&(_nf[i]),&as_for_td,&LT_for_td,ord,PC_STD,false,cut,&LNP,&LM_for_td,1.,0.);
		}else{
			td.resize(1);
		}
		td[0].init(B,pdf,&xT_for_td,&mu,&nf,&as_for_td,&LT_for_td,ord,PC_STD,false,cut,&LNP,&LM_for_td,1.,0.);
	}

	inline void _set_xi_mu(double xi, double mu){
//		NS_Utils::DOUT dout("TD_PDF_POL_PLOT::_set_xi_mu");
		this->xi	= xi;
		this->mu	= mu_star.mu_of_mu(mu);

//		xT
		nf	= pdf.nf(mu);
		as	= pdf.as(mu);
//		LT
//		LM
		IG.set_x(xi);
		IQ.set_x(xi);

		td[0].set_xi_xT_mu(IG,IQ);

		if(mu_star.errors){
			for(int i=0; i<2; ++i){
				_mu[i]	= mu_star.mus[i];
				_nf[i]	= pdf.nf(_mu[i]);
				_as[i]	= pdf.as(_mu[i]);

				td[i+1].set_xi_xT_mu(IG,IQ);
			}
		}
	}



public:
	inline void init(const SOPS_TDPDF& OP){
		NS_Utils::DOUT dout("TD_PDF_POL_PLOT::init");
		B	= E_BFUNC_Bbar;//op.TDPDF_func();
		i_B	= OP.TDPDF.TDPDF_Polynom();

		pdf.create(OP.PDF.PDFset(),OP.PDF.PDF_member());
		if(OP.PDF.PDF_Dummy_a_b.is_set){
			pdf.set_to_dymmy(OP.PDF.PDF_Dummy_a_b());
		}
		mu_star.init(OP,pdf.get_A_strong());


		ord	= 2;//op.order();
		imp	= true;//op.improved();
		exp	= false;//op.exponent();
		cut	= CUTOFF_NO;//op.cutoff();
		LNP	= 0.;//op.LambdaNP();

		if(imp){
			if(OP.Scheme.order_epsilon.is_set){
				ord	= 5;//op.order_epsilon();
			}else{
				ord	= 2*ord + 1;
			}
		}

		if(cut!=CUTOFF_NO && LNP==0.)	cut	= CUTOFF_NO;
		if(cut==CUTOFF_NO && LNP!=0.)	LNP	= 0.;

		ncomp	= (1+ord)*(2+ord)/2;

		IG.init(ord,imp);
		IQ.init(ord,imp);

		_init_cuba(OP.CUBA);
		_init_td();

		dout()	<<"ord="<<ord	<<"\t"<<"ncomp="<<ncomp<<endl;
	}

	template<E_INT int_B>
	inline void update(double xi, double mu){
//		NS_Utils::DOUT dout("TD_PDF_POL_PLOT::update");
		_set_xi_mu(xi,mu);
		_integrate<int_B>();
	}

	inline void operator()(double xi, double mu){
//		NS_Utils::DOUT dout("TD_PDF_POL_PLOT::operator");
		switch (B) {
			case E_BFUNC_Bbar:
				update<INT_XI_B>(xi,mu);
				break;
			default:
				break;
		}
	}

	inline const string FL_exp()const{
//		NS_Utils::DOUT dout("TD_PDF_POL_PLOT::FL_exp");
		std::stringstream ss;

		ss	<<"######################################################################################"<<endl
			<<"###"<<"\t"<<"as"<<"\t"<<		"= alpha_s(mu)/(4*pi)"<<endl;

		ss	<<"###"<<"\t"<<"Lp"<<"\t"<<		"= ln(xT^2*mu^2/b_0^2) \t\t b_0 = 2*exp(-gamma_E)"<<endl
			<<"###"<<"\t"<<"results"<<"\t"<<	"= xi * "<<B<<"_"<<i_B<<"(xi,xT,mu)"<<endl;
		ss	<<"###"<<"\t"<<"delta"<<"\t"<<	"= absolute Integration Error"<<endl
			<<"###"<<"\t"<<"G"<<"\t"<<		"= sqrt( G1^2 + G2^2 )"<<endl
			<<"######################################################################################"<<endl;

		return ss.str();
	}

	inline const string FL()const{
//		NS_Utils::DOUT dout("TD_PDF_POL_PLOT::FL");
		std::stringstream ss;
		std::stringstream p;

		ss <<"#xi\tmu\tas";

		if(mu_star.errors){
			ss <<"\tmu-\tas-\tmu+\tas+";
		}


		if(cut!=CUTOFF_NO)	ss<<"\tLambda_NP";

		for(int n=0; n<=ord; ++n){
			for(int l=0; l<=n; ++l){
				ss<<"\t"<<"as^"<<n<<"Lp^"<<l;
			}
		}

		for(int n=0; n<=ord; ++n){
			for(int l=0; l<=n; ++l){
				ss<<"\t"<<"delta_as^"<<n<<"Lp^"<<l;
			}
		}
		if(mu_star.errors){
			for(int n=0; n<=ord; ++n){
				for(int l=0; l<=n; ++l){
					ss<<"\t"<<"as-^"<<n<<"Lp-^"<<l;
				}
			}
			for(int n=0; n<=ord; ++n){
				for(int l=0; l<=n; ++l){
					ss<<"\t"<<"as+^"<<n<<"Lp+^"<<l;
				}
			}
		}
		ss<<endl;


		return ss.str();
	}

	inline const string to_String(int ntabs=0)const{
//		NS_Utils::DOUT dout("TD_PDF_POL_PLOT::to_String");

		std::stringstream ss;
		for(int i=0; i<ntabs; ++i) ss<<"\t";
		string tab(ss.str());
		ss.str(string());

		ss <<tab<<xi<<"\t"<<mu<<"\t"<<as;

		if(mu_star.errors){
			for(int i=0; i<2; ++i)	ss <<"\t"<<_mu[i]<<"\t"<<_as[i];
		}


		vector<double>::const_iterator it_r(results.begin());
		for(int n=0; n<=ord; ++n){
			for(int l=0; l<=n; ++l){
				ss<<"\t"<<(*(it_r++));
			}
		}

		vector<double>::const_iterator it_d(integration_errors.begin());
		for(int n=0; n<=ord; ++n){
			for(int l=0; l<=n; ++l){
				ss<<"\t"<<(*(it_d++));
			}
		}

		if(mu_star.errors){
			for(int i=0; i<2; ++i){
				for(int n=0; n<=ord; ++n){
					for(int l=0; l<=n; ++l){
						ss<<"\t"<<(*(it_r++));
					}
				}
			}
		}
		ss<<endl;

		return ss.str();
	}
};

const double TD_PDF_POL_PLOT::as_for_td =1.;



} // NS_TDPDF

} // CuTe

#endif // TDPDF_PLOT_H
