#ifndef TDPDF_OPTIONS_H_
#define TDPDF_OPTIONS_H_

#include "param_options.h"
#include "cuba_int.h"

namespace CuTe {
namespace NS_TDPDF {

using namespace NS_CS_Params;
using namespace NS_enums::NS_BFUNCS;
using namespace NS_enums::NS_TD_PARTON;

class OPS_TDPDF: public NS_OPTIONS::_Options_Set{
public:

	Option<ENUM_B_FUNCS> TDPDF_func;
	Option<bool> TDPDF_Fourier_Transform;
	Option<bool> TDPDF_Normalize;

	Option<vector<double> > TDPDF_xi;
	Option<int> TDPDF_xi_steps;
	Option<vector<double> > TDPDF_xT_or_qT;
	Option<int> TDPDF_xT_or_qT_steps;

	Option<ENUM_TDPDF_Parton> TDPDF_Polynom;

protected:

	void v_init_ops(){
		TDPDF_func		= Option<ENUM_B_FUNCS>("TDPDF_func",enum_list<ENUM_B_FUNCS>());
		TDPDF_func.set_def_val(E_BFUNC_B);
		TDPDF_Fourier_Transform	= Option<bool>("TDPDF_Fourier_Transform","[True->qT,False->xT]",false);
		TDPDF_Fourier_Transform.set_def_val(false);
		TDPDF_Normalize	= Option<bool>("TDPDF_Normalize","[True->B/Phi,False->xi*B]",false);
		TDPDF_Normalize.set_def_val(false);
		TDPDF_xi			= Option<vector<double> >("TDPDF_xi","[double...double]");
		TDPDF_xi.set_def_exp("only for CuTe_gPDF");
		TDPDF_xi_steps	= Option<int>("TDPDF_xi_steps","[int]",1);
		TDPDF_xi_steps.set_def_val(1);
		TDPDF_xT_or_qT			= Option<vector<double> >("TDPDF_xT_or_qT","[double...double]");
		TDPDF_xT_or_qT.set_def_exp("only for CuTe_gPDF");
		TDPDF_xT_or_qT_steps	= Option<int>("TDPDF_xT_or_qT_steps","[int]",1);
		TDPDF_xT_or_qT_steps.set_def_val(1);
		TDPDF_Polynom	= Option<ENUM_TDPDF_Parton>("TDPDF_Polynom",enum_list<ENUM_TDPDF_Parton>());
		TDPDF_Polynom.set_def_exp("all values");
	}

	void v_init_vec(){
		options.push_back(&TDPDF_func);
		options.push_back(&TDPDF_Fourier_Transform);
		options.push_back(&TDPDF_Normalize);
		options.push_back(&TDPDF_xi);
		options.push_back(&TDPDF_xi_steps);
		options.push_back(&TDPDF_xT_or_qT);
		options.push_back(&TDPDF_xT_or_qT_steps);
		options.push_back(&TDPDF_Polynom);
	}

public:

	OPS_TDPDF():_Options_Set("TDPDF Options"){
		init();
	}

	OPS_TDPDF(const string& filename):_Options_Set("TDPDF Options"){
		init(filename);
	}

	OPS_TDPDF(const OPS_TDPDF& OS)
	:_Options_Set(OS)
	,TDPDF_func(OS.TDPDF_func)
	,TDPDF_Fourier_Transform(OS.TDPDF_Fourier_Transform)
	,TDPDF_Normalize(OS.TDPDF_Normalize)
	,TDPDF_xi(OS.TDPDF_xi)
	,TDPDF_xi_steps(OS.TDPDF_xi_steps)
	,TDPDF_xT_or_qT(OS.TDPDF_xT_or_qT)
	,TDPDF_xT_or_qT_steps(OS.TDPDF_xT_or_qT_steps)
	,TDPDF_Polynom(OS.TDPDF_Polynom){
		init_vec();
	}

};


class SOPS_TDPDF:public SOPS_CS_Params {
public:
	OPS_TDPDF TDPDF;
	NS_CUBA_integration::OPS_CUBA CUBA;

protected:

//	virtual void v_init_all(){
//		SOPS_CS_Params::v_init_all();
	inline void init(){
		_all.push_back(&TDPDF);
		_all.push_back(&CUBA);
		_init();
	}

public:

	SOPS_TDPDF():SOPS_CS_Params(){
		init();
	}

	SOPS_TDPDF(const string& filename):SOPS_CS_Params(){
		init();
		open(filename);
	}

	SOPS_TDPDF(const SOPS_TDPDF& OP)
	:SOPS_CS_Params(OP)
	,TDPDF(OP.TDPDF)
	,CUBA(OP.CUBA){
		std::cout<<"SOPS_TDPDF(const SOPS_TDPDF& OP)"<<std::endl;
		init();
	}

};



}  // namespace NS_CS_Params
}  // namespace CuTe


#endif //TDPDF_OPTIONS_H_
