#ifndef CUBA_OPTIONS_H_
#define CUBA_OPTIONS_H_

#include "enums.h"
#include "options.h"


namespace CuTe {
namespace NS_CUBA_integration {

using NS_OPTIONS::Option;
using namespace NS_enums::NS_CUBA;


class OPS_CUBA: public NS_OPTIONS::_Options_Set{
public:

	// CUBA
	Option<double>	accuracy;
	Option<CUBA_R>	CUBA_routine;
	Option<int>		CUBA_cores;
//	Option<int>		CUBA_coresmax;
	Option<int>		CUBA_flags;
	Option<int>		CUBA_seed;
	Option<int>		CUBA_mineval;
	Option<int>		CUBA_maxeval;

	// CUBA_VEGAS
	Option<int>		CUBA_VEGAS_nstart;
	Option<int>		CUBA_VEGAS_nincrease;
	Option<int>		CUBA_VEGAS_nbatch;

	// CUBA_SUAVE
	Option<int>		CUBA_SUAVE_nnew;
	Option<int>		CUBA_SUAVE_nmin;
	Option<double>	CUBA_SUAVE_flatness;

	// CUBA_DIVONNE
	Option<int>		CUBA_DIVONNE_key1;
	Option<int>		CUBA_DIVONNE_key2;
	Option<int>		CUBA_DIVONNE_key3;
	Option<int>		CUBA_DIVONNE_maxpass;
	Option<double>	CUBA_DIVONNE_border;
	Option<double>	CUBA_DIVONNE_maxchisq;
	Option<double>	CUBA_DIVONNE_mindeviation;

	// CUBA_CUHRE
	Option<int>		CUBA_CUHRE_key;

protected:

	void v_init_ops(){
//		std::cout<<"\t"<<"OPS_CUBA::v_init_ops()"<<std::endl;
		// CUBA
		accuracy	= Option<double>("accuracy","[double]",1e-3);
		CUBA_routine= Option<CUBA_R>("CUBA_routine",NS_enums::enum_list<CUBA_R>());
		CUBA_routine.set_def_val(CUBA_R_CUHRE);
		CUBA_cores	= Option<int>("CUBA_cores","[int]",-1);
		CUBA_cores.set_def_exp("Set by CUBA lib");
		CUBA_maxeval= Option<int>("CUBA_maxeval","[int]",5e6);

//		CUBA_coresmax	= Option<int>("CUBA_coresmax","[int]");
//		CUBA_coresmax.set_def_exp("Set by CUBA lib");
		CUBA_flags	= Option<int>("CUBA_flags","[int]");
		CUBA_flags.set_def_val(0);
		CUBA_seed	= Option<int>("CUBA_seed","[int]");
		CUBA_seed.set_def_val(42);
		CUBA_mineval= Option<int>("CUBA_mineval","[int]");
		CUBA_mineval.set_def_val(2000);


		// CUBA_VEGAS
		CUBA_VEGAS_nstart		= Option<int>("CUBA_VEGAS_nstart","[int]");
		CUBA_VEGAS_nstart.set_def_val(1e3);
		CUBA_VEGAS_nincrease	= Option<int>("CUBA_VEGAS_nincrease","[int]");
		CUBA_VEGAS_nincrease.set_def_val(1e3);
		CUBA_VEGAS_nbatch		= Option<int>("CUBA_VEGAS_nbatch","[int]");
		CUBA_VEGAS_nbatch.set_def_val(2e3);

		// CUBA_SUAVE
		CUBA_SUAVE_nnew		= Option<int>("CUBA_SUAVE_nnew","[int]");
		CUBA_SUAVE_nnew.set_def_val(1e3);
		CUBA_SUAVE_nmin		= Option<int>("CUBA_SUAVE_nmin","[int]");
		CUBA_SUAVE_nmin.set_def_val(2);
		CUBA_SUAVE_flatness	= Option<double>("CUBA_SUAVE_flatness","[double]");
		CUBA_SUAVE_flatness.set_def_val(25);

		// CUBA_DIVONNE
		CUBA_DIVONNE_key1			= Option<int>("CUBA_DIVONNE_key1","[int]");
		CUBA_DIVONNE_key1.set_def_val(47);
		CUBA_DIVONNE_key2			= Option<int>("CUBA_DIVONNE_key2","[int]");
		CUBA_DIVONNE_key2.set_def_val(1);
		CUBA_DIVONNE_key3			= Option<int>("CUBA_DIVONNE_key3","[int]");
		CUBA_DIVONNE_key3.set_def_val(1);
		CUBA_DIVONNE_maxpass		= Option<int>("CUBA_DIVONNE_maxpass","[int]");
		CUBA_DIVONNE_maxpass.set_def_val(5);
		CUBA_DIVONNE_border			= Option<double>("CUBA_DIVONNE_border","[double]");
		CUBA_DIVONNE_border.set_def_val(1e-3);
		CUBA_DIVONNE_maxchisq		= Option<double>("CUBA_DIVONNE_maxchisq","[double]");
		CUBA_DIVONNE_maxchisq.set_def_val(10);
		CUBA_DIVONNE_mindeviation	= Option<double>("CUBA_DIVONNE_mindev","[double]");
		CUBA_DIVONNE_mindeviation.set_def_val(0.25);

		// CUBA_CUHRE
		CUBA_CUHRE_key	= Option<int>("CUBA_CUHRE_key","[int]");
		CUBA_CUHRE_key.set_def_val(9);
	}

	void v_init_vec(){
//		std::cout<<"\t"<<"OPS_CUBA::v_init_vec()"<<std::endl;
		// CUBA
		options.push_back(&accuracy);
		options.push_back(&CUBA_routine);
		options.push_back(&CUBA_cores);
		options.push_back(&CUBA_maxeval);

//		options.push_back(&CUBA_coresmax);
		options.push_back(&CUBA_flags);
		options.push_back(&CUBA_seed);
		options.push_back(&CUBA_mineval);

		// CUBA_VEGAS
		options.push_back(&CUBA_VEGAS_nstart);
		options.push_back(&CUBA_VEGAS_nincrease);
		options.push_back(&CUBA_VEGAS_nbatch);

		// CUBA_SUAVE
		options.push_back(&CUBA_SUAVE_nnew);
		options.push_back(&CUBA_SUAVE_nmin);
		options.push_back(&CUBA_SUAVE_flatness);

		// CUBA_DIVONNE
		options.push_back(&CUBA_DIVONNE_key1);
		options.push_back(&CUBA_DIVONNE_key2);
		options.push_back(&CUBA_DIVONNE_key3);
		options.push_back(&CUBA_DIVONNE_maxpass);
		options.push_back(&CUBA_DIVONNE_border);
		options.push_back(&CUBA_DIVONNE_maxchisq);
		options.push_back(&CUBA_DIVONNE_mindeviation);

		// CUBA_CUHRE
		options.push_back(&CUBA_CUHRE_key);
	}

public:

	OPS_CUBA():_Options_Set("Cuba Options"){
//		std::cout<<"\t"<<"OPS_CUBA()"<<std::endl;
		init();
	}

	OPS_CUBA(const std::string& filename):_Options_Set("Cuba Options"){
//		std::cout<<"\t"<<"OPS_CUBA("<<filename<<")"<<std::endl;
		init(filename);
	}

	OPS_CUBA(const OPS_CUBA& OS)
	:_Options_Set(OS)
	,accuracy(OS.accuracy)
	,CUBA_routine(OS.CUBA_routine)
	,CUBA_cores(OS.CUBA_cores)
//	,CUBA_coresmax(OS.CUBA_coresmax)
	,CUBA_flags(OS.CUBA_flags)
	,CUBA_seed(OS.CUBA_seed)
	,CUBA_mineval(OS.CUBA_mineval)
	,CUBA_maxeval(OS.CUBA_maxeval)
	,CUBA_VEGAS_nstart(OS.CUBA_VEGAS_nstart)
	,CUBA_VEGAS_nincrease(OS.CUBA_VEGAS_nincrease)
	,CUBA_VEGAS_nbatch(OS.CUBA_VEGAS_nbatch)
	,CUBA_SUAVE_nnew(OS.CUBA_SUAVE_nnew)
	,CUBA_SUAVE_nmin(OS.CUBA_SUAVE_nmin)
	,CUBA_SUAVE_flatness(OS.CUBA_SUAVE_flatness)
	,CUBA_DIVONNE_key1(OS.CUBA_DIVONNE_key1)
	,CUBA_DIVONNE_key2(OS.CUBA_DIVONNE_key2)
	,CUBA_DIVONNE_key3(OS.CUBA_DIVONNE_key3)
	,CUBA_DIVONNE_maxpass(OS.CUBA_DIVONNE_maxpass)
	,CUBA_DIVONNE_border(OS.CUBA_DIVONNE_border)
	,CUBA_DIVONNE_maxchisq(OS.CUBA_DIVONNE_maxchisq)
	,CUBA_DIVONNE_mindeviation(OS.CUBA_DIVONNE_mindeviation)
	,CUBA_CUHRE_key(OS.CUBA_CUHRE_key){
		init_vec();
	}

};



}  // namespace CUBA_int
}  // namespace CuTe

#endif //CUBA_OPTIONS_H_
