#include "gsl_interpolation.h"

#ifdef __cplusplus
extern "C" {
#endif
	#include "gsl/gsl_spline.h"
	#include "gsl/gsl_interp.h"
#ifdef __cplusplus
}
#endif

using CuTe::NS_GSL_Interpolation::GSL_Interpolator;


GSL_Interpolator::GSL_Interpolator():
gsl_interpolation_accelerator(gsl_interp_accel_alloc())
,gsl_interpolation_type(NULL)
,gsl_interpolation_object(NULL)
,N(0)
,x_array(),y_array()
,interpolated(false){

}

GSL_Interpolator::GSL_Interpolator(const GSL_Interpolator &RV):
gsl_interpolation_accelerator(gsl_interp_accel_alloc())
,gsl_interpolation_type(RV.gsl_interpolation_type)
,gsl_interpolation_object(NULL)
,N(RV.N)
,x_array(RV.x_array),y_array(RV.y_array)
,interpolated(false){
	if(RV.gsl_interpolation_object!=NULL){
		gsl_interpolation_object	= static_cast<void*>(gsl_interp_alloc(static_cast<const gsl_interp_type *>(gsl_interpolation_type),N));
		if(RV.interpolated) interpolate();
	}
}

GSL_Interpolator& GSL_Interpolator::operator=(const GSL_Interpolator &RV){

	if(gsl_interpolation_object!=NULL)	gsl_interp_free(static_cast<gsl_interp*>(gsl_interpolation_object));

	gsl_interpolation_type	= (RV.gsl_interpolation_type);
	N		= (RV.N);
	x_array	= (RV.x_array);
	y_array	= (RV.y_array);
	interpolated	= false;

	if(RV.gsl_interpolation_object!=NULL){
		gsl_interpolation_object	= static_cast<void*>(gsl_interp_alloc(static_cast<const gsl_interp_type *>(gsl_interpolation_type),N));
		if(RV.interpolated) interpolate();
	}
	return *this;
}

GSL_Interpolator::~GSL_Interpolator(){
	gsl_interp_accel_free(static_cast<gsl_interp_accel*>(gsl_interpolation_accelerator));
	if(gsl_interpolation_object!=NULL)	gsl_interp_free(static_cast<gsl_interp*>(gsl_interpolation_object));
}

bool GSL_Interpolator::init(const size_t N,const GSL_Interpolation_Types Type){
	interpolated	= false;
	if(gsl_interpolation_object!=NULL)	gsl_interp_free(static_cast<gsl_interp*>(gsl_interpolation_object));

	switch (Type) {
	case GSL_Interpolation_Type_Linear:
		gsl_interpolation_type	= static_cast<const void *>(gsl_interp_linear);
		break;
	case GSL_Interpolation_Type_Polynomial:
		gsl_interpolation_type	= static_cast<const void *>(gsl_interp_polynomial);
		break;
	case GSL_Interpolation_Type_CSpline:
		gsl_interpolation_type	= static_cast<const void *>(gsl_interp_cspline);
		break;
	case GSL_Interpolation_Type_Akima:
		gsl_interpolation_type	= static_cast<const void *>(gsl_interp_akima);
		break;
	default:
		this->N	= 0;
		return false;
		break;
	}
	if(N>gsl_interp_type_min_size(static_cast<const gsl_interp_type *>(gsl_interpolation_type))){

		this->N	= N;
		x_array.resize(N);
		y_array.resize(N);
		gsl_interpolation_object	= static_cast<void*>(gsl_interp_alloc(static_cast<const gsl_interp_type *>(gsl_interpolation_type),N));

		return true;
	}else{
		this->N	= 0;
		return false;
	}
}


GSL_Interpolator::GSL_Interpolator(const size_t N,const GSL_Interpolation_Types Type):
gsl_interpolation_accelerator(gsl_interp_accel_alloc())
,gsl_interpolation_type(NULL)
,gsl_interpolation_object(NULL)
,N(0)
,x_array(),y_array(){
	init(N,Type);
}

void GSL_Interpolator::interpolate(){
	gsl_interp_init(static_cast<gsl_interp*>(gsl_interpolation_object),&(this->x_array[0]),&(this->y_array[0]),N);
	interpolated = true;
}

double GSL_Interpolator::operator()(const double x)const{
	if( (x_array[0]<=x) && (x<=x_array[N-1]) ){	//FIXME Debug
		return gsl_interp_eval(static_cast<gsl_interp*>(gsl_interpolation_object),&(x_array[0]),&(y_array[0]),x,static_cast<gsl_interp_accel*>(gsl_interpolation_accelerator));
	}else{
		cerr	<<"EDOM: x="<<x<<" not in ["<<x_array[0]<<","<<x_array[N-1]<<"]"<<endl;
		return 0.;
	}
}
