#ifndef GSL_INTERPOLATION_H_
#define GSL_INTERPOLATION_H_

#include <vector>
#include <iostream>



namespace CuTe {
namespace NS_GSL_Interpolation {


using std::vector;
using std::ostream;
using std::endl;
using std::cerr;

enum GSL_Interpolation_Types {GSL_Interpolation_Type_Linear,GSL_Interpolation_Type_Polynomial,GSL_Interpolation_Type_CSpline,GSL_Interpolation_Type_Akima};

class GSL_Interpolator {
protected:
	void *gsl_interpolation_accelerator;
	const void *gsl_interpolation_type;
	void *gsl_interpolation_object;

	size_t N;
	vector<double> x_array,y_array;
	bool interpolated;

	vector<double>::iterator x_itr,y_itr;
public:


	GSL_Interpolator();
	GSL_Interpolator(const size_t N, const GSL_Interpolation_Types Type=GSL_Interpolation_Type_Linear);
	GSL_Interpolator(const GSL_Interpolator& RV);
	GSL_Interpolator& operator=(const GSL_Interpolator& RV);
	~GSL_Interpolator();

	bool init(const size_t N, const GSL_Interpolation_Types Type=GSL_Interpolation_Type_Linear);
	void interpolate();

	double operator()(const double x)const;

	ostream& into_stream(ostream& os)const{
		for(size_t i=0; i<N; ++i){
			os	<< x_array[i]	<<"\t"<<(*this)(x_array[i])<<endl;
		}
		return os;
	}

	ostream& into_stream(ostream& os, const size_t N)const{
		double x=x_array[0];
		const double delta = (x_array[this->N-1]-x)/static_cast<double>(N-1.);

		for(size_t i=0; i<N-1; ++i){
			os	<< x	<<"\t"<<(*this)(x)<<endl;
			x +=delta;
		}
		os	<< x	<<"\t"<<(*this)(x_array[this->N-1])<<endl;
		return os;
	}

	inline size_t get_N()const{return N;}

	inline bool set_x_y_i(const double x, const double y, const size_t i){
		if(i<N){
			x_array[i]	= x;
			y_array[i]	= y;
			if(i<(N-1)) return true;
		}
		return false;
	}


	inline size_t itr_init_return_N(){
		interpolated	= false;

		x_itr	= x_array.begin();
		y_itr	= y_array.begin();

		return N;
	}

	inline bool itr_step_return_continue(const double x, const double y){
		*(x_itr++)	= x;
		*(y_itr++)	= y;
		return (x_itr!=x_array.end());
	}

	inline bool itr_interpolate_ok(){
		interpolate();
		return interpolated;
	}
};



}  // namespace NS_GSL_Integration
}  // namespace CuTe
#endif	//GSL_INTERPOLATION_H_
