/*
 * hpl1d.hpp
 *
 *  Created on: Feb 14, 2013
 *      Author: lorentzen
 */

#ifndef HPL1D_HPP_
#define HPL1D_HPP_


#include <complex>
#include <iostream>


namespace hpl {


typedef std::complex<double> dcmplx;


template <class T, const int n1, const int n2>
class FortranArray1d
{
private:
	const static int n_ = n2-n1+1;
	T value_[n_];

public:
	FortranArray1d()
	{
		for (int n=0; n<n_; ++n) value_[n] = T();
	};

	FortranArray1d(const T t[n_]) {for (int n=0; n<n_; ++n) value_[n] = t[n];};

	T operator() (int n) const
	{
		if (n<n1 || n2<n )
		{
			std::cerr << "Warning: template <class T,"
					<< "int n1=" << n1 << ",int n2=" << n2
					<< "> FortranArray1d operator()(int n), n=" << n
					<< " out of range. Return value_[0].\n";
			return value_[0];
		}

		return value_[n-n1];
	};

	T& operator() (int n)
	{
		if (n<n1 || n2<n )
		{
			std::cerr << "Warning: template <class T,"
					<< "int n1=" << n1 << ",int n2=" << n2
					<< "> FortranArray1d operator()(int n), n=" << n
					<< " out of range. Return value_[0].\n";
			return value_[0];
		}

		return value_[n-n1];
	};

	int size1() {return n_;};
};


template <class T, const int m1, const int m2, const int n1, const int n2>
class FortranArray2d
{
private:
	const static int m_ = m2-m1+1;
	const static int n_ = n2-n1+1;
	T value_[n_][m_];

public:
	FortranArray2d()
	{
		for (int n=0; n<n_; ++n)
			for (int m=0; m<m_; ++m) value_[n][m] = T();
	};

	T operator()(int m, int n) const
	{
		if (n<n1 || n2<n || m<m1 || m2<m)
		{
			std::cerr << "Warning: template <class T> FortranArray2d operator()(int m, int n)\n"
				<< "m=" << m << " or n=" << n
				<< " out of range. Return value_[0][0].\n";
			return value_[0][0];
		}
		return value_[n-n1][m-m1];
	};

	T& operator()(int m, int n)
	{
		if (n<n1 || n2<n || m<m1 || m2<m)
		{
			std::cerr << "Warning: template <class T> FortranArray2d operator()(int m, int n)\n"
				<< "m=" << m << " or n=" << n
				<< " out of range. Return value_[0][0].\n";
			return value_[0][0];
		}
		return value_[n-n1][m-m1];
	};

	int size1() {return n_;};
	int size2() {return m_;};
};


template <class T, const int m1, const int m2, const int n1, const int n2,
const int o1, const int o2>
class FortranArray3d
{
private:
	const static int m_ = m2-m1+1;
	const static int n_ = n2-n1+1;
	const static int o_ = o2-o1+1;

	T value_[o_][n_][m_];

public:
	FortranArray3d()
	{
		for (int o=0; o<o_; ++o)
			for (int n=0; n<n_; ++n)
				for (int m=0; m<m_; ++m) value_[o][n][m] = T();
	};

	T operator()(int m, int n, int o) const
	{
		if (o<o1 || o2<o || n<n1 || n2<n || m<m1 || m2<m)
		{
			std::cerr << "Warning: template <class T> "
					"FortranArray3d operator()(int m, int n, int o)\n"
				<< "m=" << m << " or n=" << n << " or o=" << o
				<< " out of range. Return value_[0][0][0].\n";
			return value_[0][0][0];
		}
		return value_[o-o1][n-n1][m-m1];
	};

	T& operator()(int m, int n, int o)
	{
		if (o<o1 || o2<o || n<n1 || n2<n || m<m1 || m2<m)
		{
			std::cerr << "Warning: template <class T> "
					"FortranArray3d operator()(int m, int n, int o)\n"
				<< "m=" << m << " or n=" << n << " or o=" << o
				<< " out of range. Return value_[0][0][0].\n";
			return value_[0][0][0];
		}
		return value_[o-o1][n-n1][m-m1];
	};

	int size1() {return o_;};
	int size2() {return n_;};
	int size3() {return m_;};
};


template <class T, const int m1, const int m2, const int n1, const int n2,
const int o1, const int o2, const int p1, const int p2>
class FortranArray4d
{
private:
	const static int m_ = m2-m1+1;
	const static int n_ = n2-n1+1;
	const static int o_ = o2-o1+1;
	const static int p_ = p2-p1+1;

	T value_[p_][o_][n_][m_];

public:
	FortranArray4d()
	{
		for (int p=0; p<p_; ++p)
			for (int o=0; o<o_; ++o)
				for (int n=0; n<n_; ++n)
					for (int m=0; m<m_; ++m) value_[p][o][n][m] = T();
	};

	T operator()(int m, int n, int o, int p) const
	{
		if (p<p1 || p2<p || o<o1 || o2<o || n<n1 || n2<n || m<m1 || m2<m)
		{
			std::cerr << "Warning: template <class T> "
					"FortranArray4d operator()(int m, int n, int o, int p)\n"
				<< "m=" << m << " or n=" << n << " or o=" << o << " or p=" << p
				<< " out of range. Return value_[0][0][0][0].\n";
			return value_[0][0][0][0];
		}
		return value_[p-p1][o-o1][n-n1][m-m1];
	};

	T& operator()(int m, int n, int o, int p)
	{
		if (p<p1 || p2<p || o<o1 || o2<o || n<n1 || n2<n || m<m1 || m2<m)
		{
			std::cerr << "Warning: template <class T> "
					"FortranArray4d operator()(int m, int n, int o, int p)\n"
				<< "m=" << m << " or n=" << n << " or o=" << o << " or p=" << p
				<< " out of range. Return value_[0][0][0][0].\n";
			return value_[0][0][0][0];
		}
		return value_[p-p1][o-o1][n-n1][m-m1];
	};

	int size1() {return p_;};
	int size2() {return o_;};
	int size3() {return n_;};
	int size4() {return m_;};
};


template <class T, const int m1, const int m2, const int n1, const int n2,
const int o1, const int o2, const int p1, const int p2, const int q1, const int q2>
class FortranArray5d
{
private:
	T value_[q2-q1+1][p2-p1+1][o2-o1+1][n2-n1+1][m2-m1+1];
	const static int m_ = m2-m1+1;
	const static int n_ = n2-n1+1;
	const static int o_ = o2-o1+1;
	const static int p_ = p2-p1+1;
	const static int q_ = q2-q1+1;

public:
	FortranArray5d()
	{
		for (int q=0; q<q_; ++q)
			for (int p=0; p<p_; ++p)
				for (int o=0; o<o_; ++o)
					for (int n=0; n<n_; ++n)
						for (int m=0; m<m_; ++m) value_[q][p][o][n][m] = T();
	};

	T operator()(int m, int n, int o, int p, int q) const
	{
		if (o<o1 || o2<o || p<p1 || p2<p || o<o1 || o2<o || n<n1 || n2<n || m<m1 || m2<m)
		{
			std::cerr << "Warning: template <class T> "
					"FortranArray5d operator()(int m, int n, int o, int p, int q)\n"
				<< "m=" << m << " or n=" << n << " or o=" << o << " or p=" << p  << " or q=" << q
				<< " out of range. Return value_[0][0][0][0][0].\n";
			return value_[0][0][0][0][0];
		}
		return value_[q-q1][p-p1][o-o1][n-n1][m-m1];
	};

	T& operator()(int m, int n, int o, int p, int q)
	{
		if (o<o1 || o2<o || p<p1 || p2<p || o<o1 || o2<o || n<n1 || n2<n || m<m1 || m2<m)
		{
			std::cerr << "Warning: template <class T> "
					"FortranArray5d operator()(int m, int n, int o, int p, int q)\n"
				<< "m=" << m << " or n=" << n << " or o=" << o << " or p=" << p  << " or q=" << q
				<< " out of range. Return value_[0][0][0][0][0].\n";
			return value_[0][0][0][0][0];
		}
		return value_[q-q1][p-p1][o-o1][n-n1][m-m1];
	};

	int size1() {return q_;};
	int size2() {return p_;};
	int size3() {return o_;};
	int size4() {return n_;};
	int size5() {return m_;};
};


typedef FortranArray1d<int,1,3> FA1dI13;
typedef FortranArray1d<int,-1,1> FA1dI11;
typedef FortranArray1d<double,-1,1> FA1dD11;
typedef FortranArray1d<std::complex<double>,-1,1> FA1dC11;
typedef FortranArray2d<double,-1,1,-1,1> FA2dD11;
typedef FortranArray2d<std::complex<double>,-1,1,-1,1> FA2dC11;
typedef FortranArray3d<double,-1,1,-1,1,-1,1> FA3dD11;
typedef FortranArray3d<std::complex<double>,-1,1,-1,1,-1,1> FA3dC11;
typedef FortranArray4d<double,-1,1,-1,1,-1,1,-1,1> FA4dD11;
typedef FortranArray4d<std::complex<double>,-1,1,-1,1,-1,1,-1,1> FA4dC11;


void hplog(double x, int nw,
		FA1dC11& Hc1, FA2dC11& Hc2, FA3dC11& Hc3, FA4dC11& Hc4,
		FA1dD11& Hr1, FA2dD11& Hr2, FA3dD11& Hr3, FA4dD11& Hr4,
		FA1dD11& Hi1, FA2dD11& Hi2, FA3dD11& Hi3, FA4dD11& Hi4,
		int n1, int n2);


namespace details {


	const double pi = 3.14159265358979324;
	const double pinv = 0.318309886183790672;

	void eval1dhplat0(double y, int nw,
			FA1dC11& H1,  FA2dC11& H2,  FA3dC11& H3,  FA4dC11& H4,
			FA1dD11& HY1, FA2dD11& HY2, FA3dD11& HY3, FA4dD11& HY4,
			FA1dD11& Hi1, FA2dD11& Hi2, FA3dD11& Hi3, FA4dD11& Hi4,
			int n1, int n2, int infilldim, FA1dI13 infill);


	void eval1dhplin1(double y, int nw,
			FA1dC11& H1,  FA2dC11& H2,  FA3dC11& H3,  FA4dC11& H4,
			FA1dD11& HY1, FA2dD11& HY2, FA3dD11& HY3, FA4dD11& HY4,
			FA1dD11& Hi1, FA2dD11& Hi2, FA3dD11& Hi3, FA4dD11& Hi4,
			int n1, int n2, int infilldim, FA1dI13 infill);


	void eval1dhplat1(double y, int nw,
			FA1dC11& H1,  FA2dC11& H2,  FA3dC11& H3,  FA4dC11& H4,
			FA1dD11& HY1, FA2dD11& HY2, FA3dD11& HY3, FA4dD11& HY4,
			FA1dD11& Hi1, FA2dD11& Hi2, FA3dD11& Hi3, FA4dD11& Hi4,
			int n1, int n2, int infilldim, FA1dI13 infill);


	void eval1dhplatinf(double y, int nw,
			FA1dC11& H1,  FA2dC11& H2,  FA3dC11& H3,  FA4dC11& H4,
			FA1dD11& HY1, FA2dD11& HY2, FA3dD11& HY3, FA4dD11& HY4,
			FA1dD11& Hi1, FA2dD11& Hi2, FA3dD11& Hi3, FA4dD11& Hi4,
			int n1, int n2, int infilldim, FA1dI13 infill);


	void eval1dhplinm1(double y, int nw,
			FA1dC11& H1,  FA2dC11& H2,  FA3dC11& H3,  FA4dC11& H4,
			FA1dD11& HY1, FA2dD11& HY2, FA3dD11& HY3, FA4dD11& HY4,
			FA1dD11& Hi1, FA2dD11& Hi2, FA3dD11& Hi3, FA4dD11& Hi4,
			int n1, int n2, int infilldim, FA1dI13 infill);


	void eval1dhplatm1(double y, int nw,
			FA1dC11& H1,  FA2dC11& H2,  FA3dC11& H3,  FA4dC11& H4,
			FA1dD11& HY1, FA2dD11& HY2, FA3dD11& HY3, FA4dD11& HY4,
			FA1dD11& Hi1, FA2dD11& Hi2, FA3dD11& Hi3, FA4dD11& Hi4,
			int n1, int n2, int infilldim, FA1dI13 infill);


	void eval1dhplatminf(double y, int nw,
			FA1dC11& H1,  FA2dC11& H2,  FA3dC11& H3,  FA4dC11& H4,
			FA1dD11& HY1, FA2dD11& HY2, FA3dD11& HY3, FA4dD11& HY4,
			FA1dD11& Hi1, FA2dD11& Hi2, FA3dD11& Hi3, FA4dD11& Hi4,
			int n1, int n2, int infilldim, FA1dI13 infill);


	void setzero(int nw, FA1dD11& Hi1, FA2dD11& Hi2, FA3dD11& Hi3, FA4dD11& Hi4, int n1, int n2);


	void fillred1dhpl(int nw,
			FA1dC11& H1,  FA2dC11& H2,  FA3dC11& H3,  FA4dC11& H4,
			FA1dD11& HY1, FA2dD11& HY2, FA3dD11& HY3, FA4dD11& HY4,
			FA1dD11& Hi1, FA2dD11& Hi2, FA3dD11& Hi3, FA4dD11& Hi4,
			int n1, int n2, int infilldim, FA1dI13 infill);


	void fillredhpl2(int iflag, FA1dC11& H1, FA2dC11& H2, int i1, int i2, int na, int nb);


	void fillredhpl3(int iflag, FA1dC11& H1, FA2dC11& H2, FA3dC11& H3,
			int i1, int i2, int ia, int ib, int ic);


	void fillredhpl4(int iflag, FA1dC11& H1, FA2dC11& H2, FA3dC11& H3, FA4dC11& H4,
			int i1, int i2, int ia, int ib, int ic, int id);


	void printer2(int na, int nb);


	void printer3(int na, int nb, int nc);


	void printer4(int na, int nb, int nc, int nd);


	void subprint(int n, int na);


	void fillh1(double y, FA1dC11& H1, FA1dD11& HY1, FA1dD11& Hi1, int n1, int n2);


	void fillirr1dhplat0(double y, int nw, FA1dD11& HY1, FA2dD11& HY2, FA3dD11& HY3, FA4dD11& HY4,
			int n1, int n2);


	void fillirr1dhplat1(double r, int nw,
			FA1dD11& HR1, FA2dD11& HR2, FA3dD11& HR3, FA4dD11& HR4,
			FA1dD11& HY1, FA2dD11& HY2, FA3dD11& HY3, FA4dD11& HY4,
			FA1dD11& Hi1, FA2dD11& Hi2, FA3dD11& Hi3, FA4dD11& Hi4,
			int n1, int n2);


	void fillirr1dhplatinf(double x, int nw,
			FA1dD11& HX1, FA2dD11& HX2, FA3dD11& HX3, FA4dD11& HX4,
			FA1dD11& HY1, FA2dD11& HY2, FA3dD11& HY3, FA4dD11& HY4,
			FA1dD11& Hi1, FA2dD11& Hi2, FA3dD11& Hi3, FA4dD11& Hi4,
			int n1, int n2);


	void fillirr1dhplin1(double y, int nw,
			FA1dD11& HY1, FA2dD11& HY2, FA3dD11& HY3, FA4dD11& HY4,
			int n1, int n2);


}  // namespace details


}  // namespace hpl


#endif /* HPL1D_HPP_ */
