/**
* @file lhafunc.h
* @author Daniel Wilhelm
* @date	26.12.2012
* @brief Functions to handle the LHAPDF library
*/

#ifndef LHAFUNC_h
#define LHAFUNC_h

#include "enums.h"
#include "Anomalous_Dim.h"
#include <complex>

//Forward Declare LHAPDF classes
namespace LHAPDF {
	class AlphaS;
	class PDF;
	class PDFSet;
}  // namespace LHAPDF

namespace CuTe {

namespace NS_LHAFUNC {

using NS_Anomalous_Dimension::AD_Beta;
using namespace NS_enums::NS_PROD;
using namespace NS_Utils::NS_pow;

using namespace NS_enums::NS_Partons;

using std::complex;
using std::cout;
using std::endl;
using std::string;
using std::swap;
using std::vector;

class OWN_LHAPDF {
public:
	bool own;
	LHAPDF::PDF* pdf;
	LHAPDF::AlphaS* alphas;

	inline OWN_LHAPDF()
	:own(false),pdf(NULL),alphas(NULL){
	}

	inline bool operator()()const{ return own;}

	inline void unset(){
		own			= false;
	}

	inline void set(LHAPDF::PDF* pdf,LHAPDF::AlphaS* alphas){
		own			= true;
		this->pdf	= pdf;
		this->alphas= alphas;
	}

	void del();
};

std::ostream& operator<<(std::ostream& os, const OWN_LHAPDF& my_lhapdf);





//class Parton {
//public:
//	int i;
////	int _lha;
//
//	inline Parton()
//	:i(N_FMAX)
////	,_lha(0)
//	{};
//	inline explicit Parton(const int _lha_)
//	:i(_lha_+N_FMAX)
////	,_lha(_lha_)
//	{}
//
//	inline Parton& init_by_lha(int lha){
//		i	= lha + N_FMAX;
//		return *this;
//	}
//
//	inline Parton& init_by_index(int i){
//		this->i	= i;
//		return *this;
//	}
//
//	inline const Parton bar()const{
//		return Parton(-lha());
//	}
//
//	inline int lha()const{return (i-N_FMAX);}
//
//	inline bool is_downtype()const{ return i%2; }
//	inline bool is_uptype()const{ return !is_downtype(); }
//
//	inline Parton& operator++(){
////		++_lha;
//		++i;
//		return *this;
//	}
//
//	inline Parton& operator--(){
////		--_lha,
//		--i;
//		return *this;
//	}
//
//	inline const Parton operator++(int){
//		Parton tmp(lha());
//		++(*this);
//		return tmp;
//	}
//
//	inline const Parton operator--(int){
//		Parton tmp(lha());
//		--(*this);
//		return tmp;
//	}
//
//	inline const Parton operator-()const{
//		return Parton(-lha());
//	}
//
//	inline bool operator<(const Parton& q)const{
//		return i<q.i;
//	}
//
//	inline bool operator<=(const Parton& q)const{
//		return i<=q.i;
//	}
//
//	inline bool operator>(const Parton& q)const{
//		return i>q.i;
//	}
//
//	inline bool operator>=(const Parton& q)const{
//		return i>=q.i;
//	}
//
//	inline bool operator==(const Parton& q)const{
//		return i==q.i;
//	}
//
//	inline bool operator!=(const Parton& q)const{
//		return i!=q.i;
//	}
//
//	inline int flv()const{
//		return abs(lha());
//	}
//
//	inline double sgn()const{
//		const int tmp(lha());
//		if(tmp>0) return +1.;
//		if(tmp<0) return -1.;
//		return 0.;
//	}
//};
//
//static const Parton Parton_tbar(-6);
//static const Parton Parton_bbar(-5);
//static const Parton Parton_cbar(-4);
//static const Parton Parton_sbar(-3);
//static const Parton Parton_ubar(-2);
//static const Parton Parton_dbar(-1);
//static const Parton Parton_g( 0);
//static const Parton Parton_d( 1);
//static const Parton Parton_u( 2);
//static const Parton Parton_s( 3);
//static const Parton Parton_c( 4);
//static const Parton Parton_b( 5);
//static const Parton Parton_t( 6);
//
//std::ostream& operator<<(std::ostream& os, const Parton& q);





class A_strong{
public:
	static const int oMax=3;
	static const double gen;//=1e-8
protected:
	const LHAPDF::AlphaS* lha_alpha_s;

	mutable struct _A_strong_parameter{
		int Order_QCD;

		double mu_0;
		double a_0;
		int nf;

		AD_Beta Beta;

		double Lambda_QCD;
	} p;

	template< typename T>
	inline static const T _a(const T& L, const AD_Beta& Beta, const int ord){
		T _a(0.);
		const T den(1./(L*Beta(0)));
		const T LL(log(L));

		const double b30 = Beta(3,0);
		const double b20 = Beta(2,0);
		const double b10 = Beta(1,0);

		if(ord>=3){
			_a	+= 0.5*(b30-6.*b10*b20*LL+pow_3(b10)*(-1.+LL*(4. +LL*(5.-2.*LL))) );
			_a	*= den;
		}
		if(ord>=2){
			_a	+= (b20+(-1.+LL*(-1.+LL))*pow_2(b10));
			_a	*= den;
		}
		if(ord>=1){
			_a	+= -b10*LL;
			_a	*= den;
		}
		_a	+= 1.;
		_a	*= den;

		return _a;
	}

	inline static double _DaDL(const double L, const AD_Beta& Beta, const int ord){
		double _Da(0.);
		const double den(1./(L*Beta(0)));
		const double LL(log(L));

		const double b30 = Beta(3,0);
		const double b20 = Beta(2,0);
		const double b10 = Beta(1,0);

		if(ord>=3){
			_Da	+= 3.*b10*b20+2.*b30-12.*b10*b20*LL+pow_3(b10)*(-4.+LL*(3.+LL*(13.-4.*LL)));
			_Da	*= den;
		}
		if(ord>=2){
			_Da	+= 3.*b20+pow_2(b10)*(-2.+LL*(-5.+3.*LL));
			_Da	*= den;
		}
		if(ord>=1){
			_Da	+= b10*(1.-2.*LL);
			_Da	*= den;
		}
		_Da	+= 1.;
		_Da	*= den;

		_Da	*= -1./(L);
		return _Da;
	}

	inline static double _DaDLambda(const double mu,const  double Lambda, const AD_Beta& Beta, const int ord){
		const double L(2.*log(mu/Lambda));
		return -2./Lambda*_DaDL(L,Beta,ord);
	}

	inline static double _b(const double mu, const double Lambda, const AD_Beta& Beta, const int ord){
		double _b(0.);
		const double  L(log(pow_2(mu/Lambda)));
		const double LL(log(L));
		const double den(1./(L*Beta(0)));

		const double b30 = Beta(3,0);
		const double b20 = Beta(2,0);
		const double b10 = Beta(1,0);

		if(ord>=3){
			_b	+= 3.*b10*b20+2.*b30-12.*b10*b20*LL+pow_3(b10)*(-4.+LL*(3.+LL*(13.-4.*LL)));
			_b	*= den;
		}
		if(ord>=2){
			_b	+= 3.*b20+pow_2(b10)*(-2.+LL*(-5.+3.*LL));
			_b	*= den;
		}
		if(ord>=1){
			_b	+= b10*(1.-2.*LL);
			_b	*= den;
		}
		_b	+= 1.;
		_b	*= den;

		_b	*= -2./(L);
		return _b;
	}

	inline static double f(const double Lambda, void *params){
		_A_strong_parameter* p = (_A_strong_parameter*) params;
		const double L(2.*log(p->mu_0/Lambda));
//		return (_a<double>(L,p->Beta,p->Order_QCD) - p->a_0);
		return (_a<double>(L,p->Beta,p->Order_QCD)/p->a_0 - 1.);
	}

	inline static double df(const double Lambda, void *params){
		_A_strong_parameter* p = (_A_strong_parameter*) params;
//		return _Da(p->mu_0,Lambda,p->Beta,p->Order_QCD);
		return _DaDLambda(p->mu_0,Lambda,p->Beta,p->Order_QCD)/p->a_0;
	}

	inline static void fdf (const double Lambda, void *params, double *y, double *dy){
	     *y = f(Lambda,params);
	     *dy = df(Lambda,params);
	}

	inline static double fL(const double L, void *params){
		_A_strong_parameter* p = (_A_strong_parameter*) params;
//		return (_a<double>(L,p->Beta,p->Order_QCD) - p->a_0);
		return (_a<double>(L,p->Beta,p->Order_QCD)/p->a_0 - 1.);
	}

	inline static double dfL(const double L, void *params){
		_A_strong_parameter* p = (_A_strong_parameter*) params;
//		return _Da(p->mu_0,Lambda,p->Beta,p->Order_QCD);
		return _DaDL(L,p->Beta,p->Order_QCD)/p->a_0;
	}

	inline static void fdfL(const double L, void *params, double *y, double *dy){
	     *y = fL(L,params);
	     *dy = dfL(L,params);
	}

	int _calc_Lambda()const;


public:

	double get_Lambda_QCD(const double mu)const;

	void init(const LHAPDF::AlphaS& lha_alpha_s, const int OrderQCD, const double estimate=0.2);

	inline A_strong()
	:lha_alpha_s(NULL),p(){
	}

	double as(const double mu)const;

	complex<double> as(const complex<double> &mu)const;

	inline double bs(double mu)const{
		get_Lambda_QCD(mu);
		return _b(mu,p.Lambda_QCD,p.Beta,p.Order_QCD);
	}

	inline double operator()(double mu)const{ return as(mu);}
	inline complex<double> operator()(complex<double> mu)const{ return as(mu);}

	int nf(double mu)const;

	inline int nf(complex<double> mu)const{ return nf(abs(mu));}

	inline int orderQCD()const{ return p.Order_QCD; }

	double quarkMass(const Parton& q)const;
	double quarkThreshold(const Parton& q)const;

};



//class CKM_MATRIX{
//protected:
//	Matrix<N_FMAX+1,double> MX;
//public:
//	Vector<N_FMAX+1,double>::iterator
//				 Vud,Vus,Vub
//				,Vcd,Vcs,Vcb
//				,Vtd,Vts,Vtb;
//	Vector<N_FMAX+1,double>::const_iterator
//				 CVud,CVus,CVub
//				,CVcd,CVcs,CVcb
//				,CVtd,CVts,CVtb;
//
//	inline double operator() (const Parton& i,const Parton& j)const{
//		return MX[i.flv()][j.flv()];
//	}
//
//	inline void init(const vector<double>& ckm){
//		vector<double>::const_iterator it(ckm.begin());
//		*Vud	= (*(it++));
//		*Vus	= (*(it++));
//		*Vub	= (*(it++));
//		//
//		*Vcd	= (*(it++));
//		*Vcs	= (*(it++));
//		*Vcb	= (*(it++));
//		//
//		*Vtd	= (*(it++));
//		*Vts	= (*(it++));
//		*Vtb	= (*(it++));
//	}
//
//	inline CKM_MATRIX()
//	:MX(0.)
//	,Vud(MX.begin(Parton_u.flv())+Parton_d.flv()),Vus(MX.begin(Parton_u.flv())+Parton_s.flv()),Vub(MX.begin(Parton_u.flv())+Parton_b.flv())
//	,Vcd(MX.begin(Parton_c.flv())+Parton_d.flv()),Vcs(MX.begin(Parton_c.flv())+Parton_s.flv()),Vcb(MX.begin(Parton_c.flv())+Parton_b.flv())
//	,Vtd(MX.begin(Parton_t.flv())+Parton_d.flv()),Vts(MX.begin(Parton_t.flv())+Parton_s.flv()),Vtb(MX.begin(Parton_t.flv())+Parton_b.flv())
//	,CVud(Vud),CVus(Vus),CVub(Vub)
//	,CVcd(Vcd),CVcs(Vcs),CVcb(Vcb)
//	,CVtd(Vtd),CVts(Vts),CVtb(Vtb){
//		*Vud	= 0.97427;
//		*Vus	= 0.22534;
//		*Vub	= 0.00351;
//		//
//		*Vcd	= 0.22520;
//		*Vcs	= 0.97344;
//		*Vcb	= 0.0412;
//		//
//		*Vtd	= 0.00867;
//		*Vts	= 0.0404;
//		*Vtb	= 0.999146;
////		cout	<<"CKM_MATRIX()\t"
////					<<"("<<*(CVud)<<","<<CVud<<") "
////					<<"("<<*(CVub)<<","<<CVub<<") "
////					<<"("<<*(CVus)<<","<<CVus<<") "
////					<<"("<<*(CVcd)<<","<<CVcd<<") "
////					<<"("<<*(CVcb)<<","<<CVcb<<") "
////					<<"("<<*(CVcs)<<","<<CVcs<<") "
////					<<"("<<*(CVtd)<<","<<CVtd<<") "
////					<<"("<<*(CVtb)<<","<<CVtb<<") "
////					<<"("<<*(CVts)<<","<<CVts<<") "
////				<<endl;
//	}
//
//	inline CKM_MATRIX(const CKM_MATRIX& other)
//	:MX(other.MX)
//	,Vud(MX.begin(Parton_u.flv())+Parton_d.flv()),Vus(MX.begin(Parton_u.flv())+Parton_s.flv()),Vub(MX.begin(Parton_u.flv())+Parton_b.flv())
//	,Vcd(MX.begin(Parton_c.flv())+Parton_d.flv()),Vcs(MX.begin(Parton_c.flv())+Parton_s.flv()),Vcb(MX.begin(Parton_c.flv())+Parton_b.flv())
//	,Vtd(MX.begin(Parton_t.flv())+Parton_d.flv()),Vts(MX.begin(Parton_t.flv())+Parton_s.flv()),Vtb(MX.begin(Parton_t.flv())+Parton_b.flv())
//	,CVud(Vud),CVus(Vus),CVub(Vub)
//	,CVcd(Vcd),CVcs(Vcs),CVcb(Vcb)
//	,CVtd(Vtd),CVts(Vts),CVtb(Vtb){
////		cout	<<"CKM_MATRIX(other)\t"
////					<<"("<<*(CVud)<<","<<CVud<<"("<<other.CVud<<")) "
////					<<"("<<*(CVub)<<","<<CVub<<"("<<other.CVub<<")) "
////					<<"("<<*(CVus)<<","<<CVus<<"("<<other.CVus<<")) "
////					<<"("<<*(CVcd)<<","<<CVcd<<"("<<other.CVcd<<")) "
////					<<"("<<*(CVcb)<<","<<CVcb<<"("<<other.CVcb<<")) "
////					<<"("<<*(CVcs)<<","<<CVcs<<"("<<other.CVcs<<")) "
////					<<"("<<*(CVtd)<<","<<CVtd<<"("<<other.CVtd<<")) "
////					<<"("<<*(CVtb)<<","<<CVtb<<"("<<other.CVtb<<")) "
////					<<"("<<*(CVts)<<","<<CVts<<"("<<other.CVts<<")) "
////				<<endl;
//	}
//
//	inline CKM_MATRIX& operator=(const CKM_MATRIX& other){
//		MX=other.MX;
////		cout	<<"CKM_MATRIX = other\t"
////					<<"("<<*(CVud)<<","<<CVud<<"("<<other.CVud<<")) "
////					<<"("<<*(CVub)<<","<<CVub<<"("<<other.CVub<<")) "
////					<<"("<<*(CVus)<<","<<CVus<<"("<<other.CVus<<")) "
////					<<"("<<*(CVcd)<<","<<CVcd<<"("<<other.CVcd<<")) "
////					<<"("<<*(CVcb)<<","<<CVcb<<"("<<other.CVcb<<")) "
////					<<"("<<*(CVcs)<<","<<CVcs<<"("<<other.CVcs<<")) "
////					<<"("<<*(CVtd)<<","<<CVtd<<"("<<other.CVtd<<")) "
////					<<"("<<*(CVtb)<<","<<CVtb<<"("<<other.CVtb<<")) "
////					<<"("<<*(CVts)<<","<<CVts<<"("<<other.CVts<<")) "
////				<<endl;
//		return *this;
//	}
//
//	inline ~CKM_MATRIX(){
//
//	}
//};
//
//
//
//namespace Pauli_Matrices {
//
//enum e_Pauli{ Pauli_1, Pauli_2, Pauli_3, Pauli_Plus, Pauli_Minus};
//
//template < e_Pauli i >
//Matrix< 2, complex<double> > Pauli_i();
//
//template < >
//Matrix< 2, complex<double> > Pauli_i< Pauli_1 >();
//
//template < >
//Matrix< 2, complex<double> > Pauli_i< Pauli_2 >();
//
//template < >
//Matrix< 2, complex<double> > Pauli_i< Pauli_3 >();
//
//template < >
//Matrix< 2, complex<double> > Pauli_i< Pauli_Plus >();
//
//template < >
//Matrix< 2, complex<double> > Pauli_i< Pauli_Minus >();
//
//}  // namespace Pauli_Matrices
//
//
//
//class Charge{
//friend std::ostream& operator<<( std::ostream& os, Charge const &C );
//public:
//	static const Matrix<2,complex<double> > pauli_three;
//	static const Matrix<2,complex<double> > pauli_plus;
//protected:
//	double SThetaW;		//=sin(Theta_W)
//	double iS2ThetaW;	//=1./sin(2.*Theta_W)
//	double TThetaW;		//=tan(Theta_W)
//	Vector< N_PARTONS,double > qf;	// el. charge of flavour
//
//	Matrix< N_PARTONS,double > r;
//	Matrix< N_PARTONS,double > l;
//
//	Matrix< N_PARTONS,double > rr;
//	Matrix< N_PARTONS,double > ll;
//
//	Matrix< N_PARTONS,double > qq;
//
//	CKM_MATRIX ckm;
//
//	inline void init_qf(){
//		double tmp = 1./3.;
//		for(Parton q(N_FMAX); q>Parton_g; q--){
//			if(q.is_downtype()){
//				qf[q.i]			= -tmp;
//				qf[q.bar().i]	= +tmp;
//			}else{
//				qf[q.i]			= +2.*tmp;
//				qf[q.bar().i]	= -2.*tmp;
//			}
//		}
//		qf[Parton_g.i] = 0.;
//	}
//
//	inline double l_Photon(const Parton& i, const Parton& j)const{
//		if(i==-j) return qf[i.i];
//		else return 0.;
//	}
//	inline double r_Photon(const Parton& i, const Parton& j)const{
//		return l_Photon(i,j);
//	}
//	inline double r_W(const Parton& i, const Parton& j)const{
//		return 0.;
//	}
//	inline double l_W(const Parton& i, const Parton& j)const{
//		return pauli_plus[i.is_uptype()][j.is_uptype()].real()*M_SQRT1_2/SThetaW*ckm(j,i);
//	}
//	inline double l_Z(const Parton& i, const Parton& j)const{
//		if(i==-j) return pauli_three[i.is_downtype()][j.is_downtype()].real()*iS2ThetaW*j.sgn()-qf[j.i]*TThetaW;
//		return 0.;
//	}
//	inline double r_Z(const Parton& i, const Parton& j)const{
//		if(i==-j) return -qf[j.i]*TThetaW;
//		return 0.;
//	}
//
//	inline double r_von(const PROD prod, const Parton& i, const Parton& j)const{
//		if( i.sgn()*j.sgn() >= 0) return 0.;
//		if( j<Parton_g) return -l_von(prod,j,i);//conj(l_von(prod,j,i));
//		if(prod==PROD_DY)	return r_Photon(i, j);
//		if(prod==PROD_W)		return r_W(i, j);
//		if(prod==PROD_Z) 		return r_Z(i, j);
//		return 0.;
//	}
//	inline double l_von(const PROD prod, const Parton& i, const Parton& j)const{
//		if( i.sgn()*j.sgn() >= 0) return 0.;
//		if( j<Parton_g) return -r_von(prod,j,i);//conj(r_von(prod,j,i));
//		if(prod==PROD_DY)	return l_Photon(i, j);
//		if(prod==PROD_W)		return l_W(i, j);
//		if(prod==PROD_Z) 		return l_Z(i, j);
//		return 0.;
//	}
//
//
//
//public:
//	inline void init(const PROD prod){
////		cout << "->Charge::init(" << prod << ")" << endl;
//		init_qf();
//		for(Parton i(-N_FMAX); i<Parton(N_FMAX); ++i) for(Parton j(-N_FMAX); j<Parton(N_FMAX); ++j){
//			 r[i.i][j.i] =	r_von(prod,i,j);
//			 l[i.i][j.i] =	l_von(prod,i,j);
//
//			rr[i.i][j.i] =	pow_2(r[i.i][j.i]);//norm(r[i.i][j.i]);
//			ll[i.i][j.i] =	pow_2(l[i.i][j.i]);//norm(l[i.i][j.i]);
//
//			qq[i.i][j.i] =	rr[i.i][j.i]+ll[i.i][j.i];
//
////			cout	<< "Q:"<<endl<<qq<<endl;
//		}
////		cout << endl << "<-Charge::init(" << prod << ")" << endl;
//	}
//
//	inline void init(const PROD prod, double Theta_W){
//		SThetaW		= sin(Theta_W);
//		iS2ThetaW	= 1./sin(2.*Theta_W);
//		TThetaW		= tan(Theta_W);
//		init(prod);
//	}
//
//	inline void init(const PROD prod,const CKM_MATRIX& ckm_, double Theta_W){
//		ckm = ckm_;
//		init(prod,Theta_W);
//	}
//
//	inline Charge()
//	:SThetaW(0.),iS2ThetaW(0.),TThetaW(0.)
//	,qf(0.)
//	,r(0.),l(0.)
//	,rr(0.),ll(0.)
//	,qq(0.)
//	,ckm(){
//	}
//
//	inline Charge(PROD prod,const CKM_MATRIX& ckm_, double Theta_W)
//	:SThetaW(sin(Theta_W)),iS2ThetaW(1./sin(2.*Theta_W)),TThetaW(tan(Theta_W))
//	,qf(0.)
//	,r(0.),l(0.)
//	,rr(0.),ll(0.)
//	,qq(0.)
//	,ckm(ckm_){
//		if(prod!=PROD_WRONG_STATEMENT) init(prod);
//	}
//
//	inline ~Charge(){}
//
//	inline double q(const Parton& i)const{ return qf[i.i];}
//	inline const double& R(const Parton& i,const Parton& j)const{ return r[i.i][j.i];}
//	inline const double& L(const Parton& i,const Parton& j)const{ return l[i.i][j.i];}
//
//	inline double RR(const Parton& i, const Parton& j)const{ return rr[i.i][j.i];}
//	inline double LL(const Parton& i, const Parton& j)const{ return ll[i.i][j.i];}
//	inline double  Q(const Parton& i, const Parton& j)const{ return qq[i.i][j.i];}
//
//};
//std::ostream& operator<<( std::ostream& os, Charge const &C );

class PDF{
public:
	int sign;
protected:
	OWN_LHAPDF my_lhapdf;
	A_strong		*_my_as;

	const LHAPDF::PDF	*pdf;
	const A_strong		*a_s;

	int a,b;

	void _delete_own();

    void xfxQ(double x, double mu, std::vector<double>& fxQ)const;

public:

	inline double dummy(const double x)const{
		return pow(x,a)*pow(1.-x,b);
	}

	inline vector<double>& update(double x, double mu, vector<double>& fxQ)const{
//		NS_Utils::DOUT dout("PDF::update","x,mu,fxQ");

		if(pdf!=NULL){
			double c(1./x);
			xfxQ(x,mu,fxQ);

			vector<double>::iterator		  it	= fxQ.begin();
			vector<double>::reverse_iterator rit	= fxQ.rbegin();

			for(int i=0; i<N_FMAX; ++i){
				(*it) = c*(*it);
				(*rit) = c*(*rit);
				if(sign<0) swap(*it,*rit);
				++it;
				++rit;
			}
			(*it)	= c*(*it);
#ifdef ONLY_QUARKS
			*it		= 0.;
#endif
#ifdef ONLY_GLUONS
			double g = *it;
			fxQ.assign(N_PARTONS,0.);
			*it = g;
#endif
		}else{	//Dummy
#ifdef ONLY_GLUONS
			fxQ.assign(13,0.);
			fxQ[6]	= dummy(x);
#else
			fxQ.assign(13,dummy(x));
#ifdef ONLY_QUARKS
			fxQ[6]	= 0.;
#endif
#endif
		}
		return fxQ;
	}

	inline void init(const LHAPDF::PDF *pdf, const A_strong &a_s,int sign=+1){
		_delete_own();
		this->sign	= sign;
		this->pdf	= pdf;
		this->a_s	= &a_s;
	}

	void create(const string &PDFname, int member,int sign=+1);

	inline void set_to_dummy(int a, int b){
//		NS_Utils::DOUT dout("PDF::set_to_dummy");
		this->a	= a;
		this->b	= b;
		pdf	= NULL;
	}


	inline PDF()
	:sign(1)
	,my_lhapdf(),_my_as(NULL)
	,pdf(NULL),a_s(NULL)
	,a(),b(){
	}

	inline PDF(const LHAPDF::PDF *pdf, const A_strong *a_s, int sign=+1)
	:sign(sign)
	,my_lhapdf(),_my_as(NULL)
	,pdf(pdf),a_s(a_s)
	,a(),b(){
	}

	inline PDF(const PDF& other)
	:sign(other.sign)
	,my_lhapdf(),_my_as(NULL)
	,pdf(other.pdf),a_s(other.a_s)
	,a(other.a),b(other.b){
	}

	inline PDF& operator=(const PDF& other){
		_delete_own();
		sign= (other.sign);
		pdf	= (other.pdf);
		a_s	= (other.a_s);
		a	= (other.a);
		b	= (other.b);
		return *this;
	}

	inline ~PDF(){
//		NS_Utils::DOUT dout("~PDF");
		_delete_own();
//		NS_Utils::DOUT dout("~PDF");
//		dout()	<<"this\t= "<<this	<<std::endl;
//		dout()	<<"lha_PDF\t= "<<pdf	<<endl;
//		dout()	<<"a\t= "<<a_s	<<endl;
	}

	double operator()(const Parton& i, double x, double mu)const;

	inline const A_strong* as()const{return a_s;}

	inline double as(double mu)const{ return a_s->as(mu);}
	inline complex<double> as(complex<double> mu)const{ return a_s->as(mu);}
	inline int nf(double mu)const{return a_s->nf(mu);}
	inline double quarkMass(const Parton& q)const{ return a_s->quarkMass(q); }
	inline double quarkThreshold(const Parton& q)const{ return a_s->quarkThreshold(q); }

	inline const string properties(int ntabs=0)const{
		std::stringstream ss;
		for(int i=0; i<ntabs; ++i) ss<<"\t";
		string tab(ss.str());
		ss.str(string());

		ss<<tab<<"PDF:"<<endl;
		ss<<tab<<"\t"<<my_lhapdf<<endl;
		ss<<tab<<"\t"<<"_my_as\t="<<_my_as<<endl;
		ss<<tab<<"\t"<<"pdf\t="<<pdf<<endl;
		ss<<tab<<"\t"<<"a_s\t="<<a_s<<endl;
		ss<<tab<<"\t"<<"a\t="<<a<<"\t"<<"b\t="<<b<<endl;

		return ss.str();
	}

	inline void print_pdf(const string s="")const{
		cout	<<s<<"\tpdf="<<pdf<<endl;
	}

};


class PDFUncertainty {
public:
	double central, errplus, errminus, errsymm, scale;

};

class LHA_PDFSET{
protected:
	const LHAPDF::PDFSet	*pdfset;
	vector<LHAPDF::PDF*> 	 pdfs;
	vector<LHAPDF::AlphaS*>  alpha_s;

	bool owns_pdfset;
	bool owns_pdfs_and_as;

	bool	with_errors;
	int		member;

	int N;
	vector<A_strong> as;

	PDFUncertainty errors;

protected:

	void _clear_();

	template< typename T >
	static const T string_to_T(const string& s){
		T tmp;
		std::stringstream ss;
		ss	<< s;
		ss	>> tmp;

		return tmp;
	}

	inline LHA_PDFSET(const LHA_PDFSET& other);
	inline LHA_PDFSET& operator=(const LHA_PDFSET& other);

	A_strong& _init_as();

public:

//	struct alphas_parameter{
//		const int* OrderQCD;
//		const double* MZ;
//		const double* AlphaS_MZ;
//		const bool* FlavorScheme;
//		const int* NumFlavors;
//		const vector<double>* Thresholds;
//
//		alphas_parameter()
//		:OrderQCD(NULL)
//		,MZ(NULL)
//		,AlphaS_MZ(NULL)
//		,FlavorScheme(NULL)
//		,NumFlavors(NULL)
//		,Thresholds(NULL){
//
//		}
//	};


	A_strong& init(const LHAPDF::PDFSet* lha_pdfset, const bool with_errors, int member=0, const int lhapdf_verbosity=0);
	A_strong& init(const string& PDFname, const bool with_errors, const int member=0, const int lhapdf_verbosity=0);
	A_strong& init(OWN_LHAPDF *own);


	inline PDF& update(PDF &onepdf, int sign=+1, int member=0)const{
		if(member>=N){
			std::cerr	<<"LHA_PDFSET::update(PDF,sign,member) member="<<member<<" > "<<N<<" = N"<<endl;
			exit(1);
		}
		onepdf.init(pdfs[member],as[member],sign);
		return onepdf;
	}

	inline vector<PDF>& update(vector<PDF> &pdfvec, int sign=+1)const{
		vector<LHAPDF::PDF*>::const_iterator	it_pdf(pdfs.begin());
		vector<LHAPDF::PDF*>::const_iterator	end(pdfs.end());
		vector<A_strong>::const_iterator		it_as(as.begin());

		pdfvec.resize(N);
		vector<PDF>::iterator TI(pdfvec.begin());

		while(it_pdf!=end){
			(*(TI++)).init((*(it_pdf++)),(*(it_as++)),sign);
		}

		return pdfvec;
	}

	inline vector< vector<PDF> >& update(vector< vector<PDF> >& pdfvec, const int sign[2])const{
		pdfvec.resize(N);

		vector<LHAPDF::PDF*>::const_iterator	it_pdf(pdfs.begin());
		vector<LHAPDF::PDF*>::const_iterator	end(pdfs.end());
		vector<A_strong>::const_iterator		it_as(as.begin());

		vector< vector<PDF> >::iterator TI(pdfvec.begin());

		while(it_pdf!=end){
			(*TI).resize(2);
			(*TI)[0].init(*it_pdf,*it_as,sign[0]);
			(*TI)[1].init(*it_pdf,*it_as,sign[1]);

			++TI;
			++it_pdf;
			++it_as;
		}
		return pdfvec;
	}


	inline LHA_PDFSET()
	:pdfset(NULL)
	,pdfs(),alpha_s()
	,owns_pdfset(false),owns_pdfs_and_as(false)
	,with_errors(),member()
	,N(0)
	,as()
	,errors(){

	}

	inline ~LHA_PDFSET(){
		_clear_();
	}

//	inline int AlphaS_OrderQCD()const;
//	inline float MZ()const;
//	inline float AlphaS_MZ()const;
//	inline bool FlavorScheme()const;
//	inline int NumFlavors()const;
//	inline void Thresholds(vector<double>& thresholds)const;

	const PDFUncertainty& set_uncertainty(const vector<double>& values);
	double get_uncertainty_central()const;
	double get_uncertainty_errminus()const;
	double get_uncertainty_errplus()const;
	double get_uncertainty_errsymm()const;
	double get_uncertainty_scale()const;

};



template< typename T >
class PartonVector{
public:
	typedef std::vector< T >							container;

	typedef typename container::iterator				iterator;
	typedef typename container::const_iterator			const_iterator;
	typedef typename container::reverse_iterator		reverse_iterator;
	typedef typename container::const_reverse_iterator	const_reverse_iterator;

protected:

	vector< T > _vals;
	iterator gluon;
	reverse_iterator bar_gluon;
	vector< T > sum_of_q_qb;
	vector< T > sum_of_qp;
	vector< T > sum_of_bp;

	inline const iterator _begin(){ return _vals.begin();}
	inline const iterator _end(){ return _vals.end();}
	inline const reverse_iterator _rbegin(){ return _vals.rbegin();}
	inline const reverse_iterator _rend(){ return _vals.rend();}
	inline const const_iterator _begin()const{ return _vals.begin();}
	inline const const_iterator _end()const{ return _vals.end();}
	inline const const_reverse_iterator _rbegin()const{ return _vals.rbegin();}
	inline const const_reverse_iterator _rend()const{ return _vals.rend();}

	inline void _set_gluon(){
		gluon		=  _begin()+6;
		bar_gluon	= _rbegin()+6;
	}

	inline void _set_soq(){	//FIXME faster
//		NS_Utils::DOUT dout("PartonVector::_set_soq");
		_set_gluon();
		iterator sit(sum_of_q_qb.begin());
		iterator sqit(sum_of_qp.begin());
		iterator sbit(sum_of_bp.begin());
		*(sit)=*(sqit)=*(sbit)=0;
		for(int i=0; i<N_FMAX; ++i){
			T tmp;
			tmp	= (*(sqit++))	+ (*(++gluon));
			*(sqit)	= tmp;
			*(++sit)	= tmp;
			tmp	= (*(sbit++))	+ (*(++bar_gluon));
			*(sbit)	= tmp;
			*(sit)	+= tmp;
		}
		_set_gluon();
	}

public:

	inline const iterator 				 it_(int flavour)	  { return 	gluon	+ flavour;}
	inline const const_iterator			 it_(int flavour)const{ return 	gluon	+ flavour;}
	inline const reverse_iterator 		rit_(int flavour)	  { return bar_gluon- flavour;}
	inline const const_reverse_iterator rit_(int flavour)const{ return bar_gluon- flavour;}

	inline const T soq(int nf)const{ return sum_of_q_qb[nf];};
	inline const T soqp(int nf)const{ return sum_of_qp[nf];};
	inline const T sobp(int nf)const{ return sum_of_bp[nf];};

	inline const T operator[](const Parton& i)const{return _vals[i.i];}

	inline const T get_element_(int i)const{return _vals[i];}

	inline PartonVector()
	:_vals(vector<T>(N_PARTONS,0.))
	,gluon(),bar_gluon()
	,sum_of_q_qb(vector<T>(N_FMAX+1,0.))
	,sum_of_qp(vector<T>(N_FMAX+1,0.))
	,sum_of_bp(vector<T>(N_FMAX+1,0.)){
		_set_gluon();
	}

	inline PartonVector(const PartonVector& PV)
	:_vals(PV._vals)
	,gluon(),bar_gluon()
	,sum_of_q_qb(PV.sum_of_q_qb)
	,sum_of_qp(PV.sum_of_qp)
	,sum_of_bp(PV.sum_of_bp){
		_set_gluon();
	}

	inline PartonVector& operator=(const PartonVector& PV){
		_vals			= (PV._vals);
		sum_of_q_qb		= (PV.sum_of_q_qb);
		sum_of_qp		= (PV.sum_of_qp);
		sum_of_bp		= (PV.sum_of_bp);
		_set_gluon();
		return *this;
	}

	inline const std::string properties()const{
		std::stringstream ss;
		using NS_Utils::operator <<;
		ss	<<"VectorPDF:"	<<endl
			<<"\t"<<"_vals="<<_vals	<<endl
			<<"\t"<<"soq="<<sum_of_q_qb <<endl
			<<"\t"<<"soqp="<<sum_of_qp <<endl
			<<"\t"<<"sobp="<<sum_of_bp <<endl;

		return ss.str();
	}

	inline std::ostream& to_stream(std::ostream& os)const{
		const_iterator it(_begin());
		os	<<"(";
		for(;it!=gluon;++it) os<<" "<<(*it);
		os	<<" )( "<<(*(it++))<<" )(";
		for(;it!=it_(N_FMAX+1);++it) os<<" "<<(*it);
		os	<<" )";
		for(;it!=_end();++it) os<<"("<<(*it)<<")";
		return os;
	}

	inline ~PartonVector(){
//		NS_Utils::DOUT dout("~PartonVector");
//		dout()	<<"this\t= "<<this	<<std::endl;
//		dout()	<<"_vals\t= "<<&_vals	<<endl;
	}
};

template< typename T >
std::ostream& operator<<(std::ostream& os, const PartonVector< T >& PV){
	return PV.to_stream(os);
}

class VectorPDF:public PartonVector<double>{
protected:
	PDF*		_mypdf;
	const PDF*	pdf;
	double 		_soq;
	int 		_nf;

	inline void _delete_own(){
		delete _mypdf;	_mypdf = NULL;
	}

public:

	inline void update(double x, double mu){
//		NS_Utils::DOUT dout("VectorPDF::update","x,mu");

		pdf->update(x,mu,_vals);
		_set_soq();
		_nf 	= pdf->nf(mu);
		_soq	= sum_of_q_qb[_nf];
	}

	inline void create(const string &PDFname, int member=0,int sign=+1){
		NS_Utils::DOUT dout("VectorPDF::create");
		_delete_own();
		_mypdf	= new PDF();
		_mypdf->create(PDFname,member,sign);
		pdf	= _mypdf;
	}

	inline void set_to_dymmy(int a, int b){
		NS_Utils::DOUT dout("VectorPDF::set_to_dymmy");
		_mypdf->set_to_dummy(a,b);
	}

	inline void set_to_dymmy(const vector<int> &ab){
		NS_Utils::DOUT dout("VectorPDF::set_to_dymmy");
		_mypdf->set_to_dummy(ab[0],ab[1]);
	}

	inline void init(const PDF* pdf){
		_delete_own();
		this->pdf=pdf;
	}

	inline void init(const PDF& pdf){
		_delete_own();
		this->pdf=&pdf;
	}

	inline VectorPDF()
	:PartonVector<double>()
	,_mypdf(NULL)
	,pdf(NULL)
	,_soq(0.),_nf(0){
	}

	inline VectorPDF(const PDF* pdf)
	:PartonVector<double>()
	,_mypdf(NULL)
	,pdf(pdf)
	,_soq(0.),_nf(0){
	}

	inline VectorPDF(const PDF& pdf)
	:PartonVector<double>()
	,_mypdf(NULL)
	,pdf(&pdf)
	,_soq(0.),_nf(0){
	}

	inline VectorPDF(const VectorPDF& other)
	:PartonVector<double>(other)
	,_mypdf(NULL)
	,pdf(other.pdf)
	,_soq(other._soq),_nf(other._nf){
	}

	inline VectorPDF& operator=(const VectorPDF& other){
		PartonVector<double>::operator =(other);
		_delete_own();
		pdf		= (other.pdf);
		_soq	= (other._soq);
		_nf		= (other._nf);
		return *this;
	}

	inline ~VectorPDF(){
		_delete_own();
//		NS_Utils::DOUT dout("VectorPDF");
//		dout()	<<"this\t= "<<this	<<std::endl;
//		dout()	<<"PDF\t= "<<pdf	<<endl;
	}

	inline double operator()(const Parton& i, double x, double mu)const{ return (*pdf)(i,x,mu);}
	inline double soq(){return _soq;}
	inline int nf(){return _nf;}

	inline double as(double mu)const{ return pdf->as(mu);}
	inline complex<double> as(complex<double> mu)const{ return pdf->as(mu);}
	inline int nf(double mu)const{
//		NS_Utils::DOUT	dout("VectorPDF::nf","mu");
//
//		dout()	<<"_my_pdf\t= "	<<_mypdf<<endl;
//		dout()	<<"pdf\t= "		<<pdf	<<endl;
		return pdf->nf(mu);
	}
	inline double quarkMass(const Parton& q)const{ return pdf->quarkMass(q); }
	inline double quarkThreshold(const Parton& q)const{ return pdf->quarkThreshold(q); }

	inline const A_strong* get_A_strong()const{ return pdf->as();}


	inline const string properties(int ntabs=0)const{
		std::stringstream ss;
		for(int i=0; i<ntabs; ++i) ss<<"\t";
		string tab(ss.str());
		ss.str(string());

		ss<<tab<<"VectorPDF:"<<endl;
		ss<<tab<<"\t"<<"_mypdf\t="<<_mypdf<<endl;
		ss<<tab<<"\t"<<"pdf\t="<<pdf<<endl;
		ss<<pdf->properties(ntabs+1);
		return ss.str();
	}

	inline void print_pdf(const string s="")const{
		pdf->print_pdf(s);
	}
};




}  // namespace NS_LHAFUNC


}  // namespace CuTe



#endif
