#ifndef PARAM_H
#define PARAM_H

#include "lhafunc.h"
#include "charges.h"
#include "param_options.h"

namespace CuTe {

namespace NS_Kinematics {
	class Kinematics;
}  // namespace NS_Kinematics

namespace NS_CS_Params {

using NS_Kinematics::Kinematics;
using NS_Anomalous_Dimension::AD_Beta;
using namespace NS_enums::NS_COL;
using namespace NS_enums::NS_PROD;
using namespace NS_enums::NS_RES;
using namespace NS_enums::NS_NP;
using namespace NS_enums::NS_MU_OPS;
using namespace NS_enums::NS_PC;
using namespace NS_Utils::NS_pow;

using std::complex;
using std::endl;
using std::ofstream;

class PDFset{
public:
	typedef vector<NS_LHAFUNC::PDF>						value;
	typedef vector< value >								container;

	typedef container::iterator				iterator;
	typedef container::const_iterator			const_iterator;
	typedef container::reverse_iterator		reverse_iterator;
	typedef container::const_reverse_iterator	const_reverse_iterator;
protected:
	NS_LHAFUNC::LHA_PDFSET lha_pdfset;

	container pdfs;
	int sign[2];

	NS_LHAFUNC::A_strong _AS;

//	inline void _set_as(const Options& op){
//		bool set_special = false;
//
//		NS_LHAFUNC::LHA_PDFSET::alphas_parameter p;
//		if(op.as_order.is_set){			p.OrderQCD		= &(op.as_order.value);			set_special=true;}
//		if(op.as_MZ.is_set){			p.MZ			= &(op.as_MZ.value);			set_special=true;}
//		if(op.as_Alpha_s_at_MZ.is_set){	p.AlphaS_MZ		= &(op.as_Alpha_s_at_MZ.value);	set_special=true;}
//		if(op.as_FlavorScheme.is_set){	p.FlavorScheme	= &(op.as_FlavorScheme.value);	set_special=true;}
//		if(op.as_Max_Flavor.is_set){	p.NumFlavors	= &(op.as_Max_Flavor.value);	set_special=true;}
//		if(op.as_Thresholds.is_set){	p.Thresholds	= &(op.as_Thresholds.value);	set_special=true;}
//
//		if(set_special)	lha_pdfset.set_as(p,op.as_error());
//		else lha_pdfset.set_as(op.as_error());
//	}


public:
	inline void set_dummys(vector<int> a_b){
		NS_Utils::DOUT dout("PDFset::set_dummys","a_b");
		for(int i=0; i<pdfs.size(); ++i){
			for(int j=0; j<pdfs[i].size(); ++j){
				pdfs[i][j].set_to_dummy(a_b[0],a_b[1]);
			}
		}
	}

	void change_sign(const SOPS_CS_Params&);

	void change_PDF(const SOPS_CS_Params&);
	void change_PDF(const SOPS_CS_Params&,LHAPDF::PDFSet*);
	void change_PDF(NS_LHAFUNC::OWN_LHAPDF*);

	void init(const SOPS_CS_Params&);
	void init(const SOPS_CS_Params&,NS_LHAFUNC::OWN_LHAPDF*);
	void init(const SOPS_CS_Params&,LHAPDF::PDFSet*);


	inline PDFset()
	:lha_pdfset()
	,pdfs()
	,sign()
	,_AS(){
//		NS_Utils::DOUT dout("PDFset");
	}

	inline const NS_LHAFUNC::A_strong* as_central(){return &_AS;}

	inline int size()const{return pdfs.size();}

	inline int N_error()const{return size()-1;}

	inline value& operator[] (int i){ return pdfs[i]; }

	inline const iterator 		begin()		{return pdfs.begin();}
	inline const const_iterator begin()const{return pdfs.begin();}
	inline const iterator 		end()		{return pdfs.end();}
	inline const const_iterator end()  const{return pdfs.end();}

	inline const reverse_iterator 		rbegin()		{return pdfs.rbegin();}
	inline const const_reverse_iterator rbegin()const	{return pdfs.rbegin();}
	inline const reverse_iterator 		rend()			{return pdfs.rend();}
	inline const const_reverse_iterator rend()  const	{return pdfs.rend();}

	void calc_errors(vector<double>& results, const vector<double>& values);

	inline double get_alpha_s_of(const double mu)const{
		return 4.*PI*_AS(mu);
	}


};


class Q_star{
protected:
	static const int MAX_ITER = 100000;
	double q_star;
//	NS_LHAFUNC::A_strong a_strong;
	const NS_LHAFUNC::A_strong* as;
	double G_0_B; // GAMMA_0 * C_B

	double gen;
	double M;

	inline double eta(const double mu){
		return G_0_B*as->as(mu)*2.*log(M/mu);
	}

	inline double deta(const double mu){
//		cout << " -> Q_star::deta("<<mu<<")" << endl;
//		double tmp(2.*G_0_B*(log(M/mu)*a_strong.b(mu)-a_strong(mu))/mu);
//		cout << " <- Q_star::deta("<<mu<<") = " << tmp<< endl;
		return 2.*G_0_B*(log(M/mu)*as->bs(mu)-as->as(mu))/mu;
	}

	inline static double f(double x, void* params){
		Q_star* p = (Q_star*) params;
//		debug[0].push_back(x);
//		debug[1].push_back(p->eta(x));
		//cout << "\tx = " << x << endl;
		//cout << "\ta = " << p->a_strong(x) << "\tb = " << p->a_strong.b(x) << endl;
		//cout << "\tf = " << p->eta(x)-1.;
		return (p->eta(x)-1.);
	}

	inline static double df(double x, void* params){
		Q_star* p = (Q_star*) params;
//		debug[2].push_back(p->deta(x));
		//cout << "\tdf = " << p->deta(x) << endl;
		return p->deta(x)*10;// Times 10, so walk 10 times slower, thus the danger to walk into alpha_s=inf is smaller.
	}

	inline static void fdf (double x, void *params, double *y, double *dy){
		*y = f(x,params);
	    *dy = df(x,params);
	}

	int NSolve(double &q_star, const double estimate)const;

public:
	inline Q_star()
	:q_star(0.)
	,as(NULL)
	,G_0_B(0.)
	,gen(1e-8)
	,M(){
	}

	inline Q_star(const NS_LHAFUNC::A_strong* as, const double M, const double G_0_B, const double gen=1e-8)
	:q_star(0.)
	,as(as)
	,G_0_B(G_0_B)
	,gen(gen)
	,M(M){
//		NS_Utils::DOUT dout("Q_star");

		NSolve(q_star,2.*G_0_B/3.);

//		std::cout	<<"\talpha_s(MZ="<<DEFAULT_MZ<<") = "<<4.*M_PI*as->as(DEFAULT_MZ)<<"\talpha_s(MH="<<DEFAULT_MH<<") = "<<4.*M_PI*as->as(DEFAULT_MH)<<"\tM="<<M<<"\tG_0="<<G_0_B<<std::endl;
//		std::cout	<<"\tq_* = "<<q_star<<"\talpha_s(q_*) = "<<4.*M_PI*as->as(q_star)<<std::endl;
	}

	inline Q_star(const NS_LHAFUNC::A_strong* as, const SOPS_CS_Params &OP, const double gen=1e-8)
	:q_star(0.)
	,as(as)
	,G_0_B()
	,gen(gen)
	,M(){
		M	= OP.Process.M();
		switch (OP.Process.prod()) {
		case PROD_Higgs:
		case PROD_H_RS:
			G_0_B	= C_A*GAMMA_0;
			break;
		case PROD_DY:
		case PROD_Z:
		case PROD_W:
			G_0_B	= C_F*GAMMA_0;
			break;
		default:
			std::cerr	<<"**************************************************************"<<endl;
			std::cerr	<<"**** Q_star::Q_star() Wrong prod ="<<OP.Process.prod()<<" ****"<<endl;
			std::cerr	<<"**************************************************************"<<endl;
			exit(1);
			break;
		}
		if(OP.Scales.q_star.is_set){
			q_star	= OP.Scales.q_star();
		}else{
			NSolve(q_star,2.*G_0_B/3.);
		}
	}

	inline double init(const NS_LHAFUNC::A_strong* as, const double M, const double G_0_B, const double gen=1e-8){
		this->as	= as;
		this->G_0_B	= G_0_B;
		this->M		= M;
		this->gen	= gen;
		NSolve(q_star,2.*G_0_B/3.);
		return q_star;
	}

	inline double operator()()const{ return q_star;}

};

class MU_star {
protected:
	bool to_calc;
	double q_star;
	double mu;

//	vector<double> mufac;

//	double _NMU;
public:
	double _MUFAC;
	bool errors;
	vector<double> mus;


	void init(const PROD prod, const double M, const OPS_Scales& op_s,const NS_LHAFUNC::A_strong* as);

	inline void init(const SOPS_CS_Params& op_cs,const NS_LHAFUNC::A_strong* as){
		return init(op_cs.Process.prod(),op_cs.Process.M(),op_cs.Scales,as);
	}

	void change_qstar(const double qstar);
	void change_qstar(const PROD prod, const double M,const NS_LHAFUNC::A_strong* as);

	inline int N_errors()const{return mus.size();}

	inline double operator() (){
		return mu;
	}

	inline double operator() (const double qT){
		if(to_calc){
			mu = qT;
			if(q_star!=0.){
				mu += q_star*exp(-qT/q_star);
			}
			if(errors){
				mus[0] = mu/_MUFAC;
				mus[1] = mu*_MUFAC;
			}
		}
		return mu;
	}


	MU_star()
	:to_calc(true)
	,q_star(0.),mu(0.)
	,_MUFAC(0.)
	,errors(false){

	}

	bool calc_min_max(const double centre, const double value0, const double value1, double &min, double &max)const;
	bool calc_error_MP(const double centre, const double value0, const double value1, double &E_M, double &E_P)const;
	bool calc_error(const vector<double> &values_c_0_1, vector<double> &results)const;
};


class CS_Parameter{
public:
	const RES res;
	//PDFs
	vector<NS_LHAFUNC::PDF> pdfs;
	const NS_LHAFUNC::A_strong* a_strong;

	//SM
	CuTe::NS_CHARGES::CKM_MATRIX ckm;

	double alpha_EM;
	double G_Fermi;
	double Theta_W;

	//CS
	PROD prod;
	double C_B;	// C_A || C_F
	int order_as;		// if(improved): order in epsilon, else order_a

	//Const
	double S;
	double M;
//	double q_star;
	double M_t;

	//Scales
	double mu;
	int nf;		//= nf(mu)
	double as;	//= alpha_s(mu)/(4Pi)

	double LM;	//= log(M^2/mu^2)
	double Lmu;	//= log(mu^2/qT^2)
	double eta;	//= Gamma_0*a*L_M
	double etaB;//= C_B*eta


	int order;
//	bool improved;
	POWER_COUNTING pc;	//Power_Counting
	int order_eps;	//= order in alpha_s

	bool exponent;
	bool piquadrat;

	CUTOFF cutoff;
	double Lambda;

	double mu_h;
	double mu_t;

	int nf_h;
	complex<double> as_h;	//= alpha_s(mu_h)/(4Pi)
	int nf_t;
	double as_t;			//= alpha_s(mu_t)/(4Pi)

	Vector<NS_enums::NS_Partons::N_FMAX+1,bool> active_flv_charges;

protected:

	inline virtual void v_init(const SOPS_CS_Params *OP){
		std::cerr	<<"***********************************************"<<endl;
		std::cerr	<<"******* CS_Parameter::v_init(OP) called *******"<<endl;
		std::cerr	<<"***********************************************"<<endl;
		exit(1);
	}

	inline virtual void v_update(){
		std::cerr	<<"*************************************************"<<endl;
		std::cerr	<<"******* CS_Parameter::v_update(OP) called *******"<<endl;
		std::cerr	<<"*************************************************"<<endl;
		exit(1);
	}

	inline CS_Parameter(RES res)
	:res(res),pdfs(),a_strong()
	,ckm()
	,alpha_EM(),G_Fermi(),Theta_W()
	,prod(),C_B()
	,order_as()
	,S(),M()/*,q_star()*/,M_t()
	,mu()
	,nf(),as()
	,LM(),Lmu()
	,eta(),etaB()
	,order()
//	,improved()
	,pc(),order_eps()
	,exponent(),piquadrat()
	,cutoff(),Lambda()
	,mu_h(),mu_t()
	,nf_h(),as_h()
	,nf_t(),as_t()
	,active_flv_charges(true){
	}

	inline CS_Parameter(const CS_Parameter &p)
	:res(p.res),pdfs(p.pdfs),a_strong(p.a_strong)
	,ckm(p.ckm)
	,alpha_EM(p.alpha_EM),G_Fermi(p.G_Fermi),Theta_W(p.Theta_W)
	,prod(p.prod),C_B(p.C_B)
	,order_as(p.order_as)
	,S(p.S),M(p.M)/*,q_star(p.q_star)*/,M_t(p.M_t)
	,mu(p.mu)
	,nf(p.nf),as(p.as)
	,LM(p.LM),Lmu(p.Lmu)
	,eta(p.eta),etaB(p.etaB)
	,order(p.order)
//	,improved(p.improved)
	,pc(p.pc),order_eps(p.order_eps)
	,exponent(p.exponent),piquadrat(p.piquadrat)
	,cutoff(p.cutoff),Lambda(p.Lambda)
	,mu_h(p.mu_h),mu_t(p.mu_t)
	,nf_h(p.nf_h),as_h(p.as_h)
	,nf_t(p.nf_t),as_t(p.as_t)
	,active_flv_charges(p.active_flv_charges){
	}

	inline CS_Parameter& operator=(const CS_Parameter &p){
//		res				= (p.res);
		pdfs			= (p.pdfs);
		a_strong		= (p.a_strong);
		ckm				= (p.ckm);
		alpha_EM		= (p.alpha_EM);
		G_Fermi			= (p.G_Fermi);
		Theta_W			= (p.Theta_W);
		prod			= (p.prod);
		C_B				= (p.C_B);
		order_as		= (p.order_as);
		S				= (p.S);
		M				= (p.M);
//		q_star			= (p.q_star);
		M_t				= (p.M_t);
		mu				= (p.mu);
		nf				= (p.nf);
		as				= (p.as);
		LM				= (p.LM);
		Lmu				= (p.Lmu);
		eta				= (p.eta);
		etaB			= (p.etaB);

		order			= (p.order);
//		improved		= (p.improved);
		pc				= (p.pc);
		order_eps		= (p.order_eps);
		exponent		= (p.exponent);
		piquadrat		= (p.piquadrat);
		cutoff			= (p.cutoff);
		Lambda			= (p.Lambda);
		mu_h			= (p.mu_h);
		mu_t			= (p.mu_t);
		nf_h			= (p.nf_h);
		as_h			= (p.as_h);
		nf_t			= (p.nf_t);
		as_t			= (p.as_t);

		active_flv_charges	= (p.active_flv_charges);

		return *this;
	}

private:

	inline CS_Parameter();

public:

	inline void update_qT_mu(const double qT,const double mu){
		this->mu	= (mu);

		nf			= a_strong->nf(mu);
		as			= a_strong->as(mu);

		LM			= log(pow_2(M/mu));
		Lmu			= log(pow_2(mu/qT));
		eta			= GAMMA_0*as*LM;
		etaB		= C_B*eta;

		v_update();
	}

	void update_qT_mu(const Kinematics *Kin,const double mu);

	inline void init(const SOPS_CS_Params& OP, vector<NS_LHAFUNC::PDF>& pdfs/*, const double q_star*/){
//		NS_Utils::DOUT dout("CS_Parameter::init","OP,pdfs");
		//		res

//		-> change_PDF
//		//PDFs
//		this->pdfs	= (pdfs);
//		a_strong	= (pdfs[0].as());


		//SM
		ckm.init(OP.SM.ckm());

		alpha_EM	= (OP.SM.alpha_EM());
		G_Fermi		= (OP.SM.G_Fermi());
		Theta_W		= (OP.SM.Theta_W());

		active_flv_charges[0]	= true;	// gluon
		for(int i=0; i<6; ++i){
			active_flv_charges[i+1]	= OP.SM.Active_Flavour_Charges[i];
//			dout()	<<"OP.active_flv_charges:\t"<<OP.SM.Active_Flavour_Charges<<endl;
//			dout()	<<"active_flv_charges:\t"<<active_flv_charges<<endl;
		}

		//Process
		prod		= (OP.Process.prod());
		switch (prod) {
			case PROD_Higgs:
			case PROD_H_RS:
				C_B	= C_A;
				break;
			default:
				C_B	= C_F;
				break;
		}
		order_as	= (OP.Scheme.order());
		S			= pow_2(OP.Process.sqrts());
		M			= (OP.Process.M());
		M_t			= (OP.SM.M_t());

//		this->q_star=q_star;	-> change_PDF

		//		mu			= M;
		//		nf			= 5;
		//		as			= 1.;
		//		LM			= 1.;
		//		Lmu			= 1.;
		//		eta			= 1.;
		//		etaB		= 1.;


		v_init(&OP);

		change_PDF(pdfs/*,q_star*/);

		update_qT_mu(M,M);
	}

//	inline void init(const SOPS_CS_Params& OP, vector<NS_LHAFUNC::PDF>& pdfs){
//		init(OP,pdfs,0.);
////		//Scale
////		if(OP.Scales.q_star.is_set){
////			q_star	= OP.Scales.q_star();
////		}else{
////			Q_star QS(a_strong,M,C_B*GAMMA_0);
////			q_star	= QS();
////		}
//	}

	inline virtual ~CS_Parameter(){

	}

	const bool improved_pc()const{ return (pc==PC_IMP);}

	inline double calc_qstar()const{
		Q_star QS(a_strong,M,C_B*GAMMA_0);
		return QS();
	}


	inline void change_PDF(vector<NS_LHAFUNC::PDF>& pdfs/*, const double q_star*/){
		this->pdfs	= (pdfs);
		a_strong	= (pdfs[0].as());
//		this->q_star=q_star;

		v_change_PDF();
	}


protected:

	inline virtual void v_change_PDF(){

	}

};

class _CS_Parameter_fo: public CS_Parameter{
protected:

	inline void v_init(const SOPS_CS_Params *OP){
		order			= order_as;

//		improved		= false;
		pc				= PC_NO_RES;
		order_eps		= 0;
		exponent		= false;
		piquadrat		= false;
		cutoff			= CUTOFF_NO;
		Lambda			= 0.;
	}

	inline void v_update(){
		mu_h	= mu;
		mu_t	= mu;
		nf_h	= nf;
		nf_t	= nf;
		as_h	= as;
		as_t	= as;
	}

	inline _CS_Parameter_fo();

public:
	inline _CS_Parameter_fo(const RES res)
	:CS_Parameter(res){
		if( (res!=RES_LO)&&(res!=RES_LLO) ){
			std::cerr	<<"*****************************************************"<<endl;
			std::cerr	<<"CS_Parameter_LO::CS_Parameter_LO(res) WRONG res="<<res<<endl;
			std::cerr	<<"*****************************************************"<<endl;
			exit(1);
		}
	}
};

class CS_Parameter_LO: public _CS_Parameter_fo{
public:
	inline CS_Parameter_LO()
	:_CS_Parameter_fo(RES_LO){
	}
};

class CS_Parameter_LLO: public _CS_Parameter_fo{
public:
	inline CS_Parameter_LLO()
	:_CS_Parameter_fo(RES_LLO){
	}
};

class CS_Parameter_LL: public CS_Parameter{
protected:

	inline void v_init(const SOPS_CS_Params *OP){
		pc			= OP->Scheme.Power_Counting();
		if(pc==PC_NO_RES){
			std::cerr	<<"**********************************************************************"<<endl;
			std::cerr	<<"***** CS_Parameter_LL::v_init(OP) Wrong power-counting: "<<pc<<" *****"<<endl;
			std::cerr	<<"**********************************************************************"<<endl;
			exit(1);
		}
//		improved	= ( (OP->Scheme.improved.is_set)&&(OP->Scheme.improved()));
//		if(improved){
		if(improved_pc()){
			if(OP->Scheme.order_epsilon.is_set){
				order_eps= OP->Scheme.order_epsilon();
				order_as = order_eps/2;
			}else{
				order_eps	= 2*order_as+1;
			}
			order		= order_eps;
		}else{
			order_eps	= order_as;
			order		= order_as;
		}

		exponent	= ( (OP->Scheme.exponent.is_set)&&(OP->Scheme.exponent()) );
		piquadrat	= ( (OP->Scheme.PiQuadrat.is_set)&&(OP->Scheme.PiQuadrat()) );

		change_NPE(OP->Scales.LambdaNP(),( (OP->Scales.cutoff.is_set) ? (OP->Scales.cutoff()) : CUTOFF_NO ));

		//Const

		mu_h		= ( (OP->Scales.mu_h.is_set) ? OP->Scales.mu_h() : M );
		mu_t		= ( (OP->Scales.mu_t.is_set) ? OP->Scales.mu_t() : M );

//		-> v_change_PDF
//		nf_h		= a_strong->nf(mu_h);
//		if(piquadrat){
//			as_h	= a_strong->as(complex<double>(0.,mu_h));
//		}else{
//			as_h	= a_strong->as(mu_h);
//		}
//		nf_t		= a_strong->nf(mu_t);
//		as_t		= a_strong->as(mu_t);
	}

	inline void v_update(){

	}

public:

	inline CS_Parameter_LL()
	:CS_Parameter(RES_LL){
	}

	inline void change_NPE(const double Lambda_NP, const CUTOFF cutoff){
		if(Lambda_NP<=0.){
			this->Lambda	= 0.;
			this->cutoff	= CUTOFF_NO;
		}else{
			this->cutoff	= cutoff;
			this->Lambda	= Lambda_NP;
		}
	}

protected:

	inline virtual void v_change_PDF(){
		nf_h		= a_strong->nf(mu_h);
		if(piquadrat){
			as_h	= a_strong->as(complex<double>(0.,mu_h));
		}else{
			as_h	= a_strong->as(mu_h);
		}
		nf_t		= a_strong->nf(mu_t);
		as_t		= a_strong->as(mu_t);
	}

};


}  // namespace NS_CS_Params
}  // namespace CuTe

#endif
