#ifndef POLYNOM_h
#define POLYNOM_h

#include <iostream>
#include <cmath>
#include <vector>
#include <complex>


namespace NS_Polynom {

inline double min(double a, double b){ return ( ( (a)<(b) ) ? (a) : (b) );}
inline double max(double a, double b){ return ( ( (a)>(b) ) ? (a) : (b) );}

/*#ifndef int
#define int unsigned int
#endif*/
//typedef unsigned int int;

	inline double fac(int n){
		if(n<=1) return 1.;
		return n*fac(n-1);
	}

	template<typename T>
	inline const T ifac(int n){
		return T(1./fac(n));
	}

	inline double bin(int n, int k){
		return ((double)fac(n)/((double)(fac(k)*fac(n-k))));
	}

template < typename T>
class Pol;

template < class T >
const Pol< T > exp(const Pol< T >& P);


template < typename T >
class Pol{
public:
//
//	// Daten
//	typedef T											value_type;
//	typedef T&											reference;
//	typedef T const&									const_reference;
	typedef std::vector< T >					container;
//
//	// Dimension
//	typedef typename container_type::size_type			size_type;
////	typedef mitrax::dimension< size_type >				dimension_type;
//
//	// Iteratoren
//	typedef typename container_type::iterator			iterator;
//	typedef typename container_type::const_iterator		const_iterator;
//	typedef typename container_type::difference_type	difference_type;
//
////	// Elementzugriff
////	typedef mitrax::row_proxy< matrix >          row_proxy;
////	typedef mitrax::row_const_proxy< matrix >    row_const_proxy;
////	typedef mitrax::column_proxy< matrix >       column_proxy;
////	typedef mitrax::column_const_proxy< matrix > column_const_proxy;
//
//

typedef typename container::iterator         		iterator;
typedef typename container::const_iterator   		const_iterator;
typedef typename container::reverse_iterator        reverse_iterator;
typedef typename container::const_reverse_iterator  const_reverse_iterator;

protected:
	int _order;
	container _P;

public:
	inline Pol ( const T& VALUE):_order(0),_P(1,VALUE){}
//----------------------          Constructor
	inline Pol():_order(0),_P(1){}
	inline Pol(int order, const T& t):_order(order),_P(order+1,t){}
	inline Pol(const Pol& B):_order(B.order()),_P(B._P){}

//	inline Pol& operator=(const Pol&B){
//		std::cout	<<"Pol::operator=(B)"<<std::endl;
//		_order	= B.order();
//		_P		= B._P;
//		return *this;
//	}

//----------------------          get
	inline int order()const{return _order;}

	inline const std::vector< T >& vec()const{ return _P;}

	inline const T operator()(int i)const{ return ((_order>=i) ? _P[i] : T()) ;}
	inline const T& operator[](int i)const{return _P[i];}
	inline T& operator[](int i){
//		if(i>_order) std::cerr <<i<<">"<<_order<<std::endl;
		return _P[i];
	}

//----------------------          Iterator
	inline const iterator begin(){return _P.begin();}
	inline const const_iterator begin()const{return _P.begin();}
	inline const iterator end(){return _P.end();}
	inline const const_iterator end()const{return _P.end();}

	inline const reverse_iterator rbegin(){return _P.rbegin();}
	inline const const_reverse_iterator rbegin()const{return _P.rbegin();}
	inline const reverse_iterator rend(){return _P.rend();}
	inline const const_reverse_iterator rend()const{return _P.rend();}

//----------------------          set
	inline void order(int order_){
		if(_order==order_) return;
		_order=order_;
		_P.resize(order_+1);
	}

// Pol addition

	inline const Pol operator-()const{
		Pol tmp(*this);
		for(iterator it(tmp.begin()); it!=tmp.end(); ++it) (*it) = -(*it);
		return tmp;
	}

	template< typename T2 >
	inline Pol& add_this_Pol(const Pol< T2 >& A, int to_ord){
//		cout	<<"Pol::add_this_Pol:"	<<endl
//				<<"\tthis("<<_order<<") = "<<_P	<<endl
//				<<"\tthat("<<A.order()<<") = "<<A	<<endl
//				;

		int oa(min(to_ord,A.order()));
		int  o(min(to_ord,min(_order,oa)));
		if(o<oa) order(oa);
		iterator it(begin());
		typename Pol< T2 >::const_iterator iA(A.begin());
		for(int i=0  ; i<=o ; ++i){
//			cout <<"\t\ti="<<i << endl;
//			_P[i] += A[i];
			(*it)	+=	(*iA);
			++it;
			++iA;
		}
		for(int i=o+1; i<=oa; ++i){
//			cout <<"\t\ti="<<i << endl;
//				_P[i] = A[i];
			(*it)	=	(*iA);
			++it;
			++iA;
		}
//		cout	<<"\tErg = "<<_P	<<endl
//				;
		return *this;
	}

	template< typename T2 >
	inline Pol& add_this_Pol(const Pol< T2 >& A){
		return add_this_Pol(A,max(_order,A.order()));
	}

	template< typename T2 >
	inline Pol& add_to_this_x_times_P(const T2& x, const Pol& B, const int to_ord){
		if(x==T2(0)) return *this;

		const int ob(min(to_ord,B.order()));
		const int oa(min(to_ord,min(_order,ob)));

		if(oa<ob) order(ob);

		iterator iA(begin());
		const_iterator iB(B.begin());

		for(int i=0  ; i<=oa ; ++i){
			(*(iA++))	+=	(*(iB++))*x;
		}
		for(int i=oa+1; i<=ob; ++i){
			(*(iA++))	=	(*(iB++))*x;
		}

		return *this;
	}

	template< typename T2 >
	inline const Pol add_Pol(const Pol< T2 >& A)const{
		Pol tmp(*this);
		tmp.add_this_Pol(A);
		return tmp;
	}

	inline Pol& operator+= (const Pol& A){
		return add_this_Pol(A);
	}

	inline Pol& operator-= (const Pol& A){
		return add_this_Pol(-A);
	}

	inline const Pol operator+ (const Pol& A)const{
		return add_Pol(A);
	}
	inline const Pol operator- (const Pol& A)const{
		return add_Pol(-A);
	}

// scalar multiplikation

	template< typename T2 >
	inline const Pol& mul_this_by_scalar(const T2& t2){
		for(iterator it(begin()); it!=end(); ++it) (*it) *= t2;
//		for(int i=0; i<=_order; ++i) _P[i]*=t2;
		return *this;
	}

	template< typename T2 >
	inline const Pol mul_by_scalar(const T2& t2)const{
		Pol tmp(*this);
		tmp.mul_this_by_scalar(t2);
		return tmp;
	}

	inline Pol& operator*=(const T& t){
		for(iterator it(begin()); it!=end(); ++it) (*it) *= t;
//		for(int i=0; i<=_order; ++i) _P[i]*=t;
		return *this;
	}

	inline const Pol< T > operator*(const T& t)const{
		Pol tmp(*this);
		tmp*=t;
		return tmp;
	}


// Pol multiplikation

	inline Pol& mul_this_by_Pol(const Pol& P2,int o){
		o	= min(o,_order+P2.order());
		int o1(min(_order,o));
		order(o);

		reverse_iterator	rPk,rPi,rPend(rend());
		const_iterator		P2ik,P2end(P2.end());
		int delta(0);
		if(o1<o) delta = o-o1;
		for(rPk=rbegin(); rPk!=rPend; ++rPk){
			rPi		= rPk+delta;
			P2ik	= P2.begin()+delta;
			(*rPk)	= (*rPi)*(*P2ik);
			while(((++rPi)!=rPend)&&((++P2ik)!=P2end)){
				(*rPk)	+= (*rPi)*(*P2ik);

			}
			if(delta>0) --delta;
		}


		return *this;
	}

	inline const Pol mul_by_Pol(const Pol& P2,int o)const{
		Pol tmp(*this);
		tmp.mul_this_by_Pol(P2,o);
		return tmp;
	}


	inline Pol& operator*=(const Pol& P){
		mul_this_by_Pol(P,_order+P.order());
		return *this;
	}

	inline const Pol operator*(const Pol& P)const{
		Pol tmp(*this);
		tmp.mul_this_by_Pol(P,_order+P.order());
		return tmp;
	}

// --------------------  von

	inline const T of_1(const int order)const{
		const_iterator iP(begin());
		T von(*iP);
		for(int i=1; i<=min(order,_order); ++i){
//			von+=_P[i];
			++iP;
			von+=(*iP);
		}
		return von;
	}

	inline const T of_1()const{
		return of_1(_order);
	}

	inline const T of(T const &x, int order)const{
		int o = min(order,_order) ;
//		std::cout	<<"Pol::of(that)"	<<std::endl
////				<<"\tthis = "<<_P	<<std::endl
//				<<"\tthat = "<<x	<<std::endl
//				<<"\torder = "<<order	<<std::endl
//				<<"\t_order = "<<_order	<<std::endl
//				<<"\to = "<<o	<<std::endl
//				;
		const_iterator iP(begin()+o);
//		T von = _P[o];
		T von(*iP);
		if(o){
			for(int i(o-1); i>=0; --i){
//				cout  << "Pol::of() i="<< i << endl;
				von*=x;
//				von+=_P[i];
				--iP;
				von+=(*iP);
			}
		}
		return von;
	}

	inline const T of(T const &x)const{ return of(x,_order); }

	inline Pol& set_of(const T& x){
		T X(x);
		iterator iP(begin()+1);
//		if(_order)_P[1]*=X;
		if(_order)(*iP)*=X;
		for(int i=2; i<=_order; ++i){
			X*=x;
//			_P[i]*=X;
			++iP;
			(*iP)*=X;
		}
		return *this;
	}

	inline Pol& timesX(){
		order(_order+1);
		for(int i=_order; i>0; --i) _P[i]=_P[i-1];
		_P[0]-=_P[0];
		return *this;
	}


	Pol& exponentiate(){
//		cout	<<"Pol::exp()"	<<endl
//				<<"\tthis="<<_P	<<endl
//				;
		int n = _order;
		T P0(_P[0]);
		_P[0] -= P0;
		Pol tmp_P(*this);
		Pol tmp1(*this);
//		cout	<<"\ttmp1="<<tmp1	<<endl;

		for(int i=1; i<n; ++i){
			tmp1.mul_this_by_Pol(tmp_P,n);
//			cout	<<"\ttmp1="<<tmp1	<<endl;
			operator+=( tmp1.mul_by_scalar(ifac<T>(i+1)) );
		}
		using ::exp;
		P0		=	exp(P0);
		operator*=(P0);
		_P[0] 	=	P0;
		return *this;
	}

	inline Pol& reduce_order(){
//		if(_order==0) return *this;
//		cout	<<"Pol::reduce_order()"	<<endl
//				<<"\tthis:"<<_P	<<endl;
		reverse_iterator iP(rbegin());
		T zero = T();
		int o;
		for(o=_order; o>=0; --o){
			if((*iP)!=zero) break;	//calls reduce_order on sub Pols
			++iP;
		}
		order(max(o,0));
//		return o;
		return *this;
//		cout	<<"\t  to:"<<_P	<<endl;
	}

	inline bool operator==(Pol& P2){
		reduce_order();
		P2.reduce_order();
		if(_order!=P2.order()) return false;

		iterator iP(begin());
		iterator iP2(P2.begin());
		for(int i=0; i<=_order; ++i){
			if((*iP)!=(*iP2)) return false;
		}
		return true;
	}

	inline bool operator!=(Pol& P2){
		return !operator==(P2);
	}

	inline Pol& set_equalt_to(const Pol& P2, const int O){
		order(O);
		iterator IT(begin());
		const_iterator TI(P2.begin());

		const int OP2(P2.order());

		const int minO(min(O,OP2));

		(*IT)=(*TI);

		int o;
		for(o=1; o<=minO; ++o){
			*(++IT)=(*(++TI));
		}
		for(;o<=O;++o){
			(*(++IT)) *=0. ;
		}
		return *this;
	}

	inline Pol& set_equalt_to(const Pol& P2){
		return set_equalt_to(P2,P2.order());
	}
};



template < typename T >
const Pol< T > exp(const Pol< T >& P){
	Pol<T> tmp(P);
	tmp.exponentiate();
	return tmp;
}


template < typename T>
std::ostream& operator<<( std::ostream& os, Pol<T> const &P )
{
	os << "(" << " " << P[0];
	for(int i=1; i<= P.order(); ++i) os << " + " << P[i] << "*x^" << i;
	os << ")";
	return os;
}





//--------------------------- complex

/*template < typename T >
inline void operator =(Pol< T > &lhs,Pol< T > const &rhs){
	int n= rhs.order();
	lhs.order(n);
	for(int i=0; i<=n; ++i) lhs[i]= rhs[i];
}*/
//
//template < typename T >
//inline Pol< T > const real(Pol< T > const &P){
//	int n = P.order();
//	Pol< T > Real;
//	Real.order(n);
//	for(int i=0; i<=n; ++i)Real[i]=real(P[i]);
//	return Real;
//}
//
//template < typename T >
//inline Pol< T > const imag(Pol< T > const &P){
//	int n = P.order();
//	Pol< T > Imag(n);
//	for(int i=0; i<=n; ++i)Imag[i]=imag(P[i]);
//	return Imag;
//}


//class Continious_Order_Reduction{
//protected:
//	int order_as;
//	double reduction;
//
//public:
//	void init(const int order_as, const double reduction=1.){
//		this->order_as	= order_as;
//		this->reduction	= reduction;
//	}
//
//	void update(const double reduction){
//		this->reduction	= reduction;
//	}
//
//
//	template < typename T >
//	inline Pol< T >& reduce_this_Pol_a(Pol< T > & P)const{
//		std::cerr	<<"Continious_Order_Reduction::reduce_this_Pol_a"<<std::endl;
////		if(reduction!=1.){
////			for(int o=order_as+1; o<P.order(); ++o) P[o]*=reduction;
////		}
//		return P;
//	}
//
//	template < typename T >
//	inline Pol< Pol< T > >& reduce_this_Pol_a_l(Pol< Pol< T > > & P)const{
//		std::cerr	<<"Continious_Order_Reduction::reduce_this_Pol_a_l"<<std::endl;
//		if(reduction!=1.){
////			for(int o=order_as+1; o<P.order(); ++o) P[o].mul_this_by_scalar(reduction);
//			for(int o=order_as+1; o<P.order(); ++o) P[o].set_of(reduction);
//		}
//		return P;
//	}
//
//	template < typename T >
//	inline Pol< Pol< T > >& reduce_this_Pol_aEta_l(Pol< Pol< T > > & P)const{
//		std::cerr	<<"Continious_Order_Reduction::reduce_this_Pol_aEta_l"<<std::endl;
//		if(reduction!=1.){
////			for(int o=order_as; o<P.order(); ++o) P[o].mul_this_by_scalar(reduction);
//			reduce_this_Pol_a_l(P);
//		}
//		return P;
//	}
//
//	template < typename T >
//	inline const Pol< T >& reduce_Pol_a(Pol< T > P)const{
//		return reduce_this_Pol_a(P);
//	}
//
//	template < typename T >
//	inline const Pol< Pol< T > >& reduce_Pol_a_l(Pol< Pol< T > > P)const{
//		return reduce_this_Pol_a_l(P);
//	}
//
//	template < typename T >
//	inline const Pol< Pol< T > >& reduce_Pol_aEta_l(Pol< Pol< T > > P)const{
//		return reduce_this_Pol_aEta_l(P);
//	}
//};


template < typename T >
inline Pol< Pol< T > >& improve_Pol_a_l(Pol< Pol< T > > & P, int E){
//	std::cout	<<"->improve_Pol_a_l(P,"<<E<<")"<<std::endl;
//	std::cout	<<"\t"<<"P_a_l:"<<P<<std::endl;
	int A(P.order());
	E	= min(E,2*A);
	int* Lofa = new int[A+1];
	int* Lofe = new int[E+1];
	int e,a,l;

	for(a=0; a<=A; ++a) Lofa[a]=(P[a].reduce_order()).order();
	Pol< Pol< T > > tmp(P);

	P.order(E);
	for(e=0; e<=E; ++e){
		Lofe[e]=0;
		P[e].order(0);
		P[e][0]=0.;
	}

	for(a=A; a>=0; --a){
		for(l=Lofa[a]; l>=max(2*a-E,0); --l){
			e		= max(2*a-l,0);
			Lofe[e]	= max(l,Lofe[e]);
			P[e].order(Lofe[e]);
			P[e][l]	+= tmp[a][l];
		}
	}

//	for(e=0; e<=E; ++e){
//		for(a=A; a>=0; --a){
//			l=2*a-e;
//			if(l<=L[a]){
//				P[e]=Pol< T >(l,0.);
//				a=-1;
//			}
//		}
//	}
//	for(a=0; a<=A; ++a){
//		for(l=max(0,2*a-E); l<=L[a]; ++l){
//			e= max(2*a-l,0);
//			P[e][l]=tmp[a][l];
//		}
//	}
//	delete[] L;	// No = NULL
	delete[] Lofa;	// No = NULL
	delete[] Lofe;	// No = NULL
//	std::cout	<<"\t"<<"P_e_l:"<<P<<std::endl;
	return P;
}

template < typename T >
inline Pol< T >& improve_Pol_a(Pol< T > & P, int E){
	P.order(E);
	for(int e=E; e>0; --e){
		if(e%2) P[e]=0.;
		else P[e]=P[e/2];
	}
	return P;
}



}  // namespace NS_Polynom

#endif
