#ifndef TEST_XXINT_H_
#define TEST_XXINT_H_

#include "QDCS.h"
#include "CuTe_Options.h"
#include "Kinematics.h"
#include "cuba_int.h"

namespace CuTe {

using namespace NS_QDCS;

using NS_CUBA_integration::Cuba_Params;
using NS_enums::NS_PC::PC_IMP;
using NS_Kinematics::Kinematics;



using std::ostream;
using std::endl;

class test_QDCS:protected Quad_Differential_Crosssection{
public:
	size_t N;
	vector< vector< unsigned long long > > xx_count;


	test_QDCS()
	:Quad_Differential_Crosssection()
	,N(0),xx_count(){

	}

	void init(const size_t N, const SOPS_CS_Params& op, vector<PDF>& pdfs, const Res_Kinematic_Parameters *LL_Kin, const FO_Kinematic_Paramete_Set *FO_Kin, const bool not_exponent=false, const bool without_combined_result=false){
		this->N	= N;
		xx_count = vector< vector< unsigned long long > >(N,vector< unsigned long long >(N,0));

		Quad_Differential_Crosssection::init(op,pdfs,LL_Kin,FO_Kin,not_exponent,without_combined_result);
	}


	template< RES thisres>
	inline void set_qT_mu_y(const double qT, const Res_Kinematic_Parameters *LL_Kin, const FO_Kinematic_Paramete_Set *FO_Kin, double mu, double mu_star, const double res_reduction=1.){
		set_qT_mu(qT,LL_Kin,FO_Kin,mu,mu_star,res_reduction);
		set_y<thisres>(LL_Kin,FO_Kin);
		xx_count = vector< vector< unsigned long long > >(N,vector< unsigned long long >(N,0));
	}


	template< RES thisres>
	inline double update_xx(const double xx[2], const Res_Kinematic_Parameters *LL_Kin, const FO_Kinematic_Paramete_Set *FO_Kin, double dy){
		++(xx_count[xx[0]*N][xx[1]*N]);
		return Quad_Differential_Crosssection::update_xx<thisres>(LL_Kin,FO_Kin,dy);
	}



	ostream& xx_to_stream(ostream& os)const{
//		NS_Utils::DOUT dout("test_QDCS::xx_to_stream");
		vector< vector< unsigned long long > >::const_iterator itr_i(xx_count.begin());
		const vector< vector< unsigned long long > >::const_iterator end_i(xx_count.end());

		vector< unsigned long long >::const_iterator itr_j;
		vector< unsigned long long >::const_iterator end_j;

		while(itr_i!=end_i){
			itr_j	= (*itr_i).begin();
			end_j	= (*itr_i).end();
			while(itr_j!=end_j){
				os	<< *(itr_j++)	<< "\t";
			}
			os	<<"\n";
			++itr_i;
		}

		return os;
	}

};


class test_CS{
public:
	vector<double> results22;

	Kinematics	Kin;
	Cuba_Params	cuba;

	RES res;
	FO_Kinematic_Paramete_Set *FPS;
	Res_Kinematic_Parameters *RPS;


	NS_CS_Params::PDFset pdfset;

	NS_CS_Params::MU_star mu_star;
	double q_star;

	test_QDCS QDCS;

	template < RES thisres >
	inline void _fill_xx(const double xx[], double ff[]){

		if( ( (FPS!=NULL)&&Kin.update_zzb_and_xixib_get_OK(xx) ) || Kin.update_zzb_get_OK(xx)){
			const double Jac_y	= Kin.Jac_y;

			if(RPS!=NULL) RPS->update_zz(&Kin);
			if(FPS!=NULL) FPS->update_xx(&Kin);

				(*ff)	= QDCS.update_xx<thisres>(xx,RPS,FPS,Jac_y);
		}else{
			(*ff)	= 0.;
		}
	}

	template < RES thisres >
	inline static int _cuba_int_xx(const int *ndim, const double xx[],const int *ncomp, double ff[], void *userdata){
//		NS_Utils::DOUT dout("Crosssection::_cuba_int_xx");

		test_CS* CS = static_cast<test_CS*>(userdata);

		CS->_fill_xx<thisres>(xx,ff);

		return 0;
	}


	template< RES thisres >
	inline bool _set_qT_mu_y_get_OK(const double qT, const double mu, const double y){
//		NS_Utils::DOUT dout("Crosssection::_set_qT_y_get_OK","qT,y");

		const bool OK(Kin.set_qT_y_get_ok(qT,y));
		if(OK){
			if(RPS!=NULL)	RPS->set_qT(&Kin);
			if(FPS!=NULL)	FPS->set_qT(&Kin);

			results22.assign(2,0.);

	//		NS_Utils::DOUT dout("Crosssection::_set_qT_y_get_OK","Kin.set_y_get_OK(y)=true");
			if(RPS!=NULL)	RPS->set_y(&Kin);
			if(FPS!=NULL)	FPS->set_y(&Kin);

			QDCS.set_qT_mu_y<thisres>(qT,RPS,FPS,mu,mu);

			return true;
		}else{
			return false;
		}
	}


	vector<double>& _update(NS_CUBA_integration::cuba_integrand_t integrand){
//		NS_Utils::DOUT dout("Crosssection::_update","dim,integrand");

	//	NS_Utils::Timer timer;

		cuba.set(2,1);
		cuba.cube(integrand,this);
	//	cuba.test(1,integrand,this);//FIXME debug
		results22[0]	= cuba.integral[0];
		results22[1]	= cuba.error[0];


	//	using NS_OPTIONS::operator <<;
	//	cout	<<"("<<timer.stop()<<")"<<"\t"<<results<<endl;

		return results22;
	}

	inline void _init_params(const SOPS_CuTe& OP){
	//	NS_Utils::DOUT dout("Crosssection::_init_params","OP");
		this-> res	= OP.Scheme.resummation();
		delete	RPS;	RPS = NULL;
		delete	FPS;	FPS = NULL;

		int order_a		= OP.Scheme.order();
		int order		= order_a;
		bool improved	= false;


		switch (res) {
		case RES_LL:
			improved	= (OP.Scheme.Power_Counting()==PC_IMP);
			RPS	= new Res_Kinematic_Parameters();
			break;
		case RES_LLO:
			RPS	= new Res_Kinematic_Parameters();
			break;
		case RES_LO:
			FPS	= new FO_Kinematic_Paramete_Set();
			break;
		case RES_MATCHED:
			improved	= (OP.Scheme.Power_Counting()==PC_IMP);
			RPS	= new Res_Kinematic_Parameters();
			FPS	= new FO_Kinematic_Paramete_Set();
			break;
		case RES_MATCHING_CORR:
			RPS	= new Res_Kinematic_Parameters();
			FPS	= new FO_Kinematic_Paramete_Set();
			break;
		case RES_RES_CORR:
			improved	= (OP.Scheme.Power_Counting()==PC_IMP);
			RPS	= new Res_Kinematic_Parameters();
			FPS	= new FO_Kinematic_Paramete_Set();
			break;
		default:
			break;
		}

		if(RPS!=NULL){
			if(improved){
				if(OP.Scheme.order_epsilon.is_set){
					order	= OP.Scheme.order_epsilon();
				}else{
					order	= 2*order_a + 1;
				}
			}

			RPS->init(&Kin,OP.Process.prod(),order,improved,order_a);
		}

		if(FPS!=NULL){
			FPS->init(&Kin,order_a);
		}

	//	dout()	<<"exp_errors = "<<exp_errors<<endl;

	}

	vector<double>& _unset_results(){
		results22.assign(2,0.);

		return results22;
	}

public:

	void init(const size_t N, SOPS_CuTe& OP){
		NS_Utils::DOUT dout("Crosssection::init","op");
		dout()	<<endl
				<<OP.Process.prod		<<endl
				<<OP.Scheme.resummation	<<endl
				<<OP.Scheme.order	<<endl
				<<OP.Scheme.Power_Counting<<endl;
		if(OP.Scheme.order_epsilon.is_set && OP.Scheme.Power_Counting()==PC_IMP){
			dout()	<<OP.Scheme.order_epsilon<<endl;
		}
		dout()	<<endl;

		{
			//	//		NS_Utils::DOUT dout("init","Kinematics");
			Kin.init(OP.Process.sqrts(),OP.Process.M(),OP.Cuts);
			cuba.init(OP.CUBA,3,1);

			_init_params(OP);
		}

		NS_CS_Params::PDFset::iterator it_pdf;
		{
			//	//		NS_Utils::DOUT dout("init","pdfset");
			OP.PDF.PDF_Error.value = false;

			pdfset.init(OP);

			it_pdf	= (pdfset.begin());
		}

		{
			//	//		NS_Utils::DOUT dout("init","mu_star");
			OP.Scales.scale_variation.set(0.);
//			OP.Scales.Scale_Error.value = false;

			mu_star.init(OP,pdfset.as_central());

			NS_CS_Params::Q_star QS(pdfset.as_central(),OP);
			q_star	= QS();
		}


		{
			//	//		NS_Utils::DOUT dout("init","QDCS_central");
			QDCS.init(N, OP,*it_pdf,RPS,FPS,q_star);

		}

	}

	vector<double>& update_qT_mu_y(const double qT, const double mu, const double y){
//		NS_Utils::DOUT dout("Crosssection::update_qT_y");
		bool OK;
		NS_CUBA_integration::cuba_integrand_t integrand;

		switch (res) {
		case RES_LL:
			OK	= _set_qT_mu_y_get_OK<RES_LL>(qT,mu,y);
			integrand = _cuba_int_xx<RES_LL>;
			break;
		case RES_LLO:
			OK	= _set_qT_mu_y_get_OK<RES_LLO>(qT,mu,y);
			integrand = _cuba_int_xx<RES_LLO>;
			break;
		case RES_LO:
			OK	= _set_qT_mu_y_get_OK<RES_LO>(qT,mu,y);
			integrand = _cuba_int_xx<RES_LO>;
			break;
		case RES_MATCHING_CORR:
			OK	= _set_qT_mu_y_get_OK<RES_MATCHING_CORR>(qT,mu,y);
			integrand = _cuba_int_xx<RES_MATCHING_CORR>;
			break;
		case RES_MATCHED:
			OK	= _set_qT_mu_y_get_OK<RES_MATCHED>(qT,mu,y);
			integrand = _cuba_int_xx<RES_MATCHED>;
			break;
		case RES_RES_CORR:
			OK	= _set_qT_mu_y_get_OK<RES_RES_CORR>(qT,mu,y);
			integrand = _cuba_int_xx<RES_RES_CORR>;
			break;
		default:
			OK	= false;
			break;
		}
		if(OK){
			return _update(integrand);
		}else{
			return _unset_results();
		}
	}

	inline vector<double>& update_qT_y(const double qT, const double qT_for_mu, const double y){
		return update_qT_mu_y(qT,mu_star(qT_for_mu),y);
	}

	inline vector<double>& update_qT_y(const double qT, const double y){
		return update_qT_y(qT,qT,y);
	}

	inline double get_q_star()const{
		return q_star;
	}

	inline double get_alpha_s_MZ(const double MZ=DEFAULT_MZ)const{
		return pdfset.get_alpha_s_of(MZ);
	}


	test_CS()
	:results22(2,0.)
	,res(RES_WRONG_STATEMENT)
	,RPS(NULL),FPS(NULL)
	,QDCS()
	,pdfset()
	,mu_star(),q_star()
	,cuba()
	{

	}


	ostream& xx_to_stream(ostream& os)const{
//		NS_Utils::DOUT dout("test_CS::xx_to_stream");
		return QDCS.xx_to_stream(os);
	}


};


}  // namespace CuTe

#endif //TEST_XXINT_H_
